;****** Auto-Revision Header (do not edit) *******************************
;*
;*  Copyright by PetiWare - This is Freeware!
;*
;* Filename         : AssemblerGetty.asm
;* Created on       : 20-Feb-93
;* Created by       : Peter Simons
;* Current revision : V0.219
;*
;*
;* Purpose: Ersatz fr Matt Dillons Getty
;*
;*
;* V0.219 : Added echo-switch that makes AGetty send read characters
;*          back. Currently only used during login. Doesn't filter
;*          control characters at the moment.
;*                                               Peter Simons (10-Jan-94)
;*
;* V0.218 : Even better debugging method added: AGetty now supports my
;*          log daemon.
;*
;* V0.217 : Added extensive debugging information. The infos are written
;*          to "SYS:DEBUGLOG".
;*
;* V0.216 : Thousands of tests later, the piping routine seens to work
;*          reliable. However, there's a danger of crashed due to the
;*          pipe-handler blocking when more then 4k are buffered! But
;*          AGetty should be fast enough to avoid this.
;*
;* V0.215 : Beginnt das auszufhrende Kommando mit einem Stern, startet
;*          AGetty einen Subtask, der eingehende Daten vom serial.device
;*          in den Standardinput des gestarteten Programms kopiert.
;*          Das selbe sollte auch mit auf den Standardoutput ausgegebenen
;*          Daten geschehen, allerdings braucht die Pipingroutine noch
;*          etwas Ruhe. :-)
;*
;* V0.214 : MakeConnect() erkennt jetzt Faxconnects und gibt eine -2
;*          statt -1 zurck. AGetty startet dann den Eintrag "fax".
;*
;* V0.213 : Fehler in WaitForString() behoben. In seltenen Fllen wurde
;*          der Suchstring nicht erkannt.
;*
;* V0.212 : Wurde AGetty mit -A gestartet, wurde versucht eine Message
;*          abzuholen, die nie angefordert wurde ---> Crash!
;*
;* V0.211 : Wenn ein anderes Ereignis eintrat als das der ReadRequest
;*          erfllt wurde, wurde dieser nicht abgebrochen!
;*
;* V0.210 : WaitIO() lscht manchmal das Signalflag fr den ReadReqIO
;*          nicht. Workaround: In der Hauptschleife wird vor dem Instal-
;*          lieren der RING-detection das entsprechende Bit resetet.
;*                   Thanks to Mike Meyer and C.A.Wichura for their help!
;*
;* V0.209 : Debugging-Output in die Hauptschleife, BorrowDevice() und
;*          ModemInit() eingefgt. Kann ber ein Flag beim Assemblieren
;*          zugeschaltet werden.
;*
;* V0.208 : Die Eingaben fr Login und Passwort werden jetzt geflushed,
;*          bevor sie verglichen werden. Dadurch wird auch verhindert,
;*          da das Logfile durch Steuerzeichen zerhackt wird.
;*
;* V0.207 : ReadSerial() gibt jetzt eine Null zurck wenn keine Zeichen
;*          anliegen und der ganze Puffer ausgelesen werden sollte.
;*
;* V0.206 : Neues Keyword im Configfile: LOGCONNECT. Wenn dieser Schalter
;*          auf "yes" gesetzt wird, wird der Connectstring ins Logfile
;*          geschrieben.
;*
;* V0.205 : MakeConnect schreibt jetzt den Connectstring des Modems in
;*          einen reservierten Puffer. Wurde kein "CONNECT" gefunden ist
;*          dieser Puffer leer.
;*
;* V0.204 : Fehler in MakeConnect() behoben.
;*                                    Thanks to Mike Meyer for reporting.
;*
;* V0.203 : Neues Keyword und Kommandozeilen-Option: ANSWERCOMMAND.
;*          Hier kann der User das Kommando bestimmen, das zum Modem ge-
;*          schickt wird um den Anruf zu beantworten.
;*
;* V0.202 : MakeConnect() verbessert: Da nur eingehende Anrufe gehand-
;*          habt werden mssen, kann auf einen "RINGING"-Check verzichtet
;*          werden.
;*
;* V0.201 : Locking-Mechanismus frs Logfile eingefhrt. Allerdings wird
;*          dieser erst ab DUUCP v1.16 richtig untersttzt.
;*                  Thanks to Christpher A. Wichura for suggesting this.
;*
;* V0.200 : Den Kommandozeilen-Aufruf des AGetties gendert: Die Keywords
;*          beginnen nicht mehr mit einem "-".
;*          "7WIRE" in "HARDWIRE" umbenannt.
;*
;* V0.199 : MakeLogin() erneut gendert: Die Passwortabfrage wird ber-
;*          sprungen wenn als Passwort ein einzelner Stern ("*") einge-
;*          tragen ist. Ein Stern vor dem CommandToRun hat eine andere
;*          Bedeutung ---> TODO.
;*
;* V0.198 : Vor dem senden des Logintexts, wird eine Sekunde verzgert,
;*          da sonst der Text vom Modem verschluckt wird.
;*
;* V0.197 : Vor dem Login-Prompt wird nur ein eine Leerzeile gemacht.
;*
;* V0.196 : Wird als Login lediglich ein <CR> eingegeben, wird der Login-
;*          prompt erneut angezeigt.
;*
;* V0.195 : MakeLogin() gendert: Wenn jemand unter einem Namen einloggt,
;*          dessen CommandToRun mit einem Stern beginnt, wird die Pass-
;*          wort abfrage bersprungen.
;*
;* V0.194 : Den BorrowDevice() Mechanismus umgestellt: Das Modeminit()
;*          findet jetzt nicht mehr in der eigentlichen Routine statt.
;*          Vorher konnte ein fehlgeschlagenes Modeminit() einen
;*          Device-Locking Fehler auslsen.
;*
;* V0.193 : MakeConnect() auf das Wesentliche reduziert. Der ganze
;*          Connect-Pattern Kram ist beim AGetty ja vllig unntig. Ab
;*          jetzt wird nur noch geprft ob ein Carrier detected ist.
;*
;* V0.192 : Neuer Configeintrag: DELAYBEFOREINIT. Mit Hilfe dieser Ver-
;*          zgerung knnen (hoffentlich) auch Modems betrieben werden,
;*          die eine sehr lange Resetphase haben. Um "Endlos"-Verzger-
;*          ungen entgegenzuwirken, ist dieser Eintrag auf 10s begrenzt.
;*
;* V0.191 : HangUp() fhrt keinen Modem-Init mehr durch, da dies nicht
;*          zu den Aufgaben der Routine gehrt.
;*
;* V0.190 : Bei LockDevUnit() wird nicht mehr "AGetty" als Name angege-
;*          ben, sondern der erstellte Taskname, damit es nicht zu ber-
;*          schneidungen kommt.
;*
;* V0.189 : AGetty startet nicht, wenn QUIT angegeben ist, aber kein
;*          andere AGetty lief.
;*
;* V0.188 : Der CLI-Prozessname wird auch gesetzt, soda man bei Status
;*          auseinanderhalten kann, welcher AGetty welcher ist.
;*
;* V0.187 : -NoCarrierCheck umbenannt in -7WIRE, da eine Nullmodemver-
;*          bindung doch mehr erfordert, als lediglich auf den Carrier
;*          zu verzichten, wir haben ja garkein Modem!
;*          Das alles ist allerdings noch ungetestet!
;*
;* V0.186 : Neue Funktion: Wird AGetty mit QUIT aufgerufen, so wird ein
;*          eventuell noch laufender Getty fr diesen Port beendet.
;*
;* V0.185 : Bei einigen Keywords der Kommandozeile den Bindestrich vor-
;*          gestellt: "-NCC=-NOCARRIERCHECK".
;*
;* V0.184 : AGetty beendet sich nicht mehr selbst, wenn die Modem-
;*          Initialisierung fehlgeschlagen ist.
;*
;* V0.183 : AGetty mu jetzt mit CTRL/D beendet werden.
;*
;* V0.182 : Neuer Commandline-Parameter: NoCarrierCheck.
;*          Ist dieses Flag gesetzt, so wird der CarrierCheck ausge-
;*          schaltet. Dient zum Beispiel zur Verwendung bei Nullmodem
;*          Verbindungen.
;*
;* V0.181 : In MakeLogin() wurde ein CTRL/C-Break nicht richtig gehand-
;*          habt.
;*
;* V0.180 : Fr jeden Eintrag der Configzeile im Passwdfile existiert
;*          jetzt ein Keyword ber das man eine lokale Variable festlegen
;*          kann, in die der Inhalt des entsprechenden Eintrages ge-
;*          schrieben wird.
;*
;* V0.173 : Der HomeDir-Eintrag des Passwd-Files wird jetzt benutzt. Vor
;*          dem Starten des Kommandos, wird in das Directory gewechselt.
;*          Sollte kein Directory angegeben sein, wird "UUCP:" als
;*          Default verwendet.
;*
;* V0.172 : Die Timeout-Logmeldung entfernt.
;*
;* V0.171 : Neues Keywort hinzugefgt: IMMEDIATEANSWER. Wird hier "yes"
;*          gewhlt, so hebt AGetty bereits nach dem ersten Klingeln ab
;*          und wartet nicht auf ein erneutes "RING".
;*
;* V0.170 : Anzahl der Logeintrge reduziert.
;*
;* V0.162 : Loginname und Passwort sind jetzt auf 32 Zeichen begrenzt.
;*
;* V0.161 : War ein Login nicht erfolgreich, wird der Loginname und das
;*          angegebene Passwort im Logfile vermerkt.
;*
;* V0.160 : AGetty fragt jetzt erst Login und Password ab, bevor er
;*          berhaupt vergleicht. Dadurch kann ein Hacker nicht ber-
;*          prfen, ob sein Login oder sein Passwort unkorrekt sind.
;*
;* V0.156 : Die letzte Version hatte noch den AT&T1-String drin, drum
;*          habe ich die Versions-Nummer erhht um Miverstndnisse zu
;*          vermeiden.
;*
;* V0.155 : Wenn das Auflegen ber "+++ATH" nicht klappt, dann dropped
;*          OffHook fr kurze Zeit den DTR.
;*
;* V0.154 : OffHook() mal wieder umgestellt. Jetzt werden garkeine Rck-
;*          gaben mehr abgefragt sondern nur noch der "+++ATH" geschickt,
;*          verzgert und getestet ob der Carrier noch da ist.
;*
;* V0.153 : AGetty braucht jetzt v2.xxx der PTool.Library.
;*
;* V0.152 : CmpStrings auf die Library umgestellt und dadurch ber 256
;*          Byte gespart.
;*
;* V0.151 : OffHook() unterbricht jetzt auch laufende Connectversuche,
;*          obwohl kein Carrier besteht.
;*
;* V0.150 : WaitForString() hat ziemlich seltsame Sachen gemacht, wenn
;*          die Lnge des Suchstrings nicht bergeben wurde.
;*
;* V0.149 : Das CarrierUp-Flag hatte ich einmal zu viel gelscht, und vor
;*          allem: Zu frh!
;*
;* V0.148 : Ich habe vergessen, das CarrierUp-Flag wieder zu lschen.
;*                             Thanks to Stefan Thielscher for reporting.
;*
;* V0.147 : Ich habe die Delays bei SendModemCMD noch ein bisschen ver-
;*          krzt. Sie sind jetzt auf 2 Sekunden nach einem Break und
;*          1 Sekunde nach dem LineFeed.
;*
;* V0.146 : SendModemCMD() sendet sicherheitshalber vor dem eigentlichen
;*          Kommando noch ein Linefeed, um eventuell laufende "ATA"s ab-
;*          zubrechen, oder restliche Zeichen in der Kommandozeile aus-
;*          zufhren.
;*
;* V0.145 : SendModemCMD() umgeschrieben: Ab jetzt kann/mu der Dial- und
;*          Initstring "\r" enthalten, wenn ein Return gesendet werden
;*          soll. Die Routine wartet dann die Reaktion des Modems ab.
;*          So sind Kommandos wie: "ATZ\rATS0=0\r" mglich.
;*                        Thanks to Christopher A. Wichura for supposing.
;*
;* V0.144 : Bei allen Modemkommandos wird das Break ("+++") nur noch ge-
;*          sendet, wenn der Carrier steht.
;*
;* V0.143 : Wenn der Carrier bereits weg ist, wird das Auflegen ber-
;*          sprungen.
;*
;* V0.142 : Die Verzgerung vor dem OwnerDevUnit() vergrert, um dem an-
;*          deren Task Zeit zu geben, den Namen einzutragen.
;*
;* V0.141 : Login-Meldung war nicht korrekt! Der Name des anrufenden
;*          Systems wurde berschrieben.
;*
;* V0.140 : Das Modem liefert kein "RINGING" wenns klingelt, sondern le-
;*          diglich ein "RING"! Peinlich!
;*
;* V0.139 : AGetty setzt seine Prioritt selber
;*
;* V0.138 : Nachdem GetCall() aufgerufen wurde, liest sie den ganzen
;*          Puffer aus und wartet auf das erneute Eintreffen einer
;*          "RINGING"-Meldung. So wird das Abheben aus "alten" Meldungen
;*          vermieden. Answer-Mode wird erkannt und korrekt bearbeitet!
;*
;* V0.137 : Den Aufbau des internen Locale-Tables so gendert, da fr
;*          die Strings keine Relocs mehr gebraucht werden. Alleine da-
;*          durch ber 100 Byte gespart! Allerdings ist das Format nicht
;*          mehr zu CatComp kompatibel, was durch einen Switch abgefangen
;*          wird!
;*
;* V0.136 : AGetty luft nur noch mit v3.0 der OwnDevUnit.Library.
;*          Thanks to Christopher A. Wichura for adding OwnerDevUnit()!
;*
;* V0.135 : Fehler in ParseLine() behoben: Ein leeres Feld ",," wurde
;*          nicht korrekt gehandhabt!
;*
;* V0.134 : AGetty vollkommen lokalisiert! MakeLogEntry() verwendet be-
;*          kommt jetzt als Parameter die Nummer des Logstrings und be-
;*          sorgt sich den String selbst ber GetString().
;*
;* V0.133 : Jetzt wird auch das passende Programm mittels Execute()
;*          aufgerufen.
;*
;* V0.132 : Neuer Parameter: SingleRun! Beendet Getty nach dem Anruf,
;*          funktioniert jedoch nur mit Answer zusammen!
;*
;* V0.131 : Wenn der Login nicht klappt, wird dem "Gegner" eine Meldung
;*          zugeschickt und bis zu dreimal wiederholt. Sollte jemand ein
;*          falsches Passwort verwenden wird dies in den Logs vermerkt!
;*
;* V0.130 : Das Passwordfile wird jetzt geparsed. Einige Zwischenspeicher
;*          sowie der CallernameBuffer rausgeworfen, da alle bentigten
;*          Daten jetzt im ConfigEntryBuffer zu finden sind.
;*
;* V0.129 : Modeminitialisation in eine Unterroutine ausgelagert: Wird
;*          jetzt nach jedem Call aufgerufen!
;*
;* V0.128 : MakeLogin() fhrt den Login durch und liefert entweder einen
;*          Fehler oder den Loginnamen zurck. Was noch fehlt ist das
;*          parsen des Passwd-Files und das Aufrufen des Transport-
;*          Agents.
;*
;* V0.127 : ConvertModemText() ersetzt bei einem normalen Text die
;*          Returns durch die Kombination Linefeed/Carrige Return, da
;*          sonst die Texte im Terminal-Programm nicht richtig lesebar
;*          sind.
;*
;* V0.126 : Wenn kein Logfile angegeben ist, werden die "Logs" nur auf
;*          den Bildschirm ausgegeben.
;*
;* V0.125 : Der Login-Text wird geladen und dem Anrufer ausgegeben.
;*
;* V0.124 : GetCall() verzgert 2 Sekunden, bevor sie zurckkommt, da
;*          nach dem Hangup noch eine "NO CARRIER"-Meldung kommt, die
;*          sonst einen erneuten Anruf auslsen wrde.
;*
;* V0.123 : Im Config kann man jetzt ein Textfile angeben, da dem An-
;*          rufer entgegen geschickt wird.
;*
;* V0.122 : GetCall() erkennt das Klingeln und macht einen Log-Eintrag.
;*          Dann versucht es zu connecten.
;*
;* V0.121 : Will ein anderes Programm unser Device, gibt BorrowDevice()
;*          es frei und wartet auf die Rckabe. Gibt das andere Programm
;*          das Device nicht mehr frei, hngen wir. :-)
;*
;* V0.120 : Wenn das Device ordnungsgem uns gehrt, wird das Modem ini-
;*          tialisiert und ein 1-Byte-ReadRequest ausgesetzt. Per Wait()
;*          wird erkannt wenn ein anderes Programm das Device haben will,
;*          wenn der Benutzer CTRL/C drckt oder wenn Daten am Modem
;*          ankommen ("RINGING").
;*
;* V0.100 : Fast alle Modemroutinen aus PetiMail bernommen und an AGetty
;*          anepat.
;*
;* V0.004 : Das Device wird jetzt geffnet. Diesesmal wird wirklich jeder
;*          verdammte Fehler der auftreten kann dem Benutzer mitgeteilt!
;*
;* V0.003 : AGetty setzt seinen Prozessnamen auf "AGetty_device_unit"
;*          und fragt vorher ab, ob dieser Name schon existiert. So kann
;*          erkannt werden, wenn zwei AGetties fr denselben Port ge-
;*          startet werden.
;*
;* V0.002 : AGetty versucht bereits das Device zu locken. Dabei wird der
;*          OwnDevUnit.Library ein Sinalbit bergeben, unter dem wir zu
;*          erreichen sind, wenn jemand anderes das Device haben will.
;*
;* V0.001 : Das gesamte Gerst von AGetty neu entworfen, da AGetty
;*          PURE sein soll, soda Sites mit mehreren Ports nicht mehrmals
;*          das selbe Executable im Speicher haben.
;*
;* V0.000 : --- Initial release ---
;*
;*************************************************************************
;
;
;

;***************************************************************************
;*                                                                         *
;* SEKTION: Labels, Macros, Switches, Structures                           *
;*                                                                         *
;***************************************************************************

;-------------------------------------- Switches -----------
DEBUG           set     1       ; Set this label <> 0 to enable debug ouput

                MC68000
                NEWSYNTAX
        IFEQ DEBUG
                EXEOBJ
                OBJFILE "RAM:AGetty"
        ELSE
                OBJFILE "RAM:AGetty.o"
        ENDC
                PURE
                INCDIR  "INCLUDE:"
                DEBUG on
                SYM
;               MULTIPASS
;               VERBOSEOPTIM

;-------------------------------------- Labels ------------
                MACLIB  "LIB:Offsets.mac"
                MACFILE "exec/exec.i"
                MACFILE "devices/serial.i"
                MACFILE "devices/timer.i"
                MACFILE "dos/dostags.i"
                MACFILE "dos/dosextens.i"
                MACFILE "dos/datetime.i"
                MACFILE "dos/var.i"
                MACFILE "libraries/PToolLibrary.i"
                MACFILE "syslog.i"

KICKVERSION     equ     37
PARAMETERS      equ     SERF_SHARED|SERF_7WIRE|SERF_RAD_BOOGIE|SERF_XDISABLED
BUFFERSIZE      equ     10*1024
ODUERR_LEADCHAR equ     $07

;-------------------------------------- Strukturen --------
ConfigLine      CLRSO
cl_User         SO.l    1
cl_Password     SO.l    1
cl_UID          SO.l    1
cl_GroupID      SO.l    1
cl_FingerInfo   SO.l    1
cl_HomeDir      SO.l    1
cl_CommandToRun SO.l    1
cl_SizeOf       SOVAL

AppString       CLRSO
as_ID           SO.l    1
as_Str          SO.w    1
AppString_SIZEOF SOVAL

;-------------------------------------- Macros ------------
REVISION        MACRO
                dc.b "0.219"
                ENDM
REVDATE         MACRO
                dc.b "10-Jan-94"
                ENDM
;;;
INITDEVERROR    MACRO
                bne.b   .GoOn\@
                pea     \1
                bra     .OpenDevError
.GoOn\@
                ENDM
LOCALEENTRY     MACRO
                dc.l    \1
                dc.w    \2-*
                ENDM


;***************************************************************************
;*                                                                         *
;* SEKTION: Programm                                                       *
;*                                                                         *
;***************************************************************************

;-------------------------- Platzhalter erzeugen --------------------------
START:          link.w  a5,#StackSIZE

                move.l  SP,a0
                move.l  #-(StackSIZE/2)-1,d0
...             clr.w   (a0)+                   ; Stackeintrge vorher
                dbra    d0,...                  ; lschen

                move.l  #-1,(OldPri,a5)
                move.l  #-1,(MySignalBit,a5)
                move.l  #10,(Timeout,a5)

;---------------------- Systemumgebung initialisieren ---------------------
                lea     (ptoollibname,PC),a1
                moveq   #2,d0
                move.l  ($4).W,a6
                jsr     (_LVOOpenLibrary,a6)
                move.l  d0,(_PToolBase,a5)
                bne.b   .PToolLibOkay
                bsr     _NoPToolLibrary
                bra     .Exit

.PToolLibOkay   move.l  d0,a6
                move.l  (ptl_DOSBase,a6),(_DOSBase,a5)
                move.l  (ptl_LocaleBase,a6),(_LocaleBase,a5)

        IFNE DEBUG
                bsr     _InitSysLog
        ENDC
                SYSLOG  <"Checking the OS version.">

                move.l  #CCF_OS_VERSION,d0
                moveq   #KICKVERSION,d1
                jsr     (_LVOCheckConfig,a6)
                tst.l   d0
                beq     .ConfigOkay
                lea     (PrgName,PC),a0
                moveq   #0,d2
                jsr     (_LVODisplayConfigErrorDos,a6)
                bra     .ClosePToolLib

.ConfigOkay     SYSLOG  <"The OS is okay. I'm going to open OwnDevUnit.">
                lea     (owndevname,PC),a1
                move.l  ($4).W,a6
                moveq   #3,d0
                jsr     (_LVOOpenLibrary,a6)
                ERRORHANDLE (_OwnDevUnitBase,a5),9,.ClosePToolLib,(owndevname,PC),3

                sub.l   a0,a0
                lea     (CatalogName,PC),a1
                sub.l   a2,a2
                move.l  (_LocaleBase,a5),d0
                SYSLOG  <"Checking for Locale.Library. Base at 0x%lx">,d0
                beq     .SkipOpenCatalog
                move.l  d0,a6
                jsr     (_LVOOpenCatalogA,a6)
                SYSLOG  <"Locale is there. MessageCatalog at 0x%lx">,d0
                move.l  d0,(_MessageCatalog,a5)
.SkipOpenCatalog
;------------------------------ Argument-Parsing --------------------------
                SYSLOG  <"Let's parse the commandline.">
                lea     (RDArgsTemplate,PC),a0
                move.l  a0,d1
                lea     (_Device,a5),a0
                move.l  a0,d2
                moveq   #0,d3
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOReadArgs,a6)
                ERRORHANDLE (_RDArgsStruct,a5),0,(.CloseCatalog,PC)

;------------------------------ Read ConfigFiles --------------------------
                SYSLOG  <"Looking for my configfiles.">
                lea     (ConfigFile,PC),a0
                lea     (ConfigTagList,PC),a1
                lea     (_Logfile,a5),a2
                lea     (_RememberKey,a5),a3
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOInterpretConfigFileRemember,a6)
                ERRORHANDLE ,1,.FreeRDArgs,(ReadConfigError,PC)

                move.l  (DelayBefInit,a5),d0
                bclr.l  #31,d0                  ; Zahl mu positiv sein
                cmp.l   #11,d0
                bmi     .DelayValid             ; Zu grossen Delays, insbe-
                moveq   #0,d0                   ; sondere durch negative
.DelayValid     mulu.w  #50,d0                  ; Zahlen vorbeugen
                and.l   #$0000FFFF,d0
                move.l  d0,(DelayBefInit,a5)

                SYSLOG  <"Now I need the Passwdfile.">
                move.l  (_PasswdFile,a5),a2
                move.l  a2,a0
                lea     (_RememberKey,a5),a1
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOLoadFileRemember,a6)
                ERRORHANDLE (_PasswdBuffer,a5),1,.FreeRDArgs,(a2)

;------------------------------ ProzessNamen setzen -----------------------
                SYSLOG  <"This one is tricky: I'm going to modify my processname!">
                move.l  (_Device,a5),d0
                bne     .GetUnit
                move.l  (_StandardDev,a5),d0
                bne     .GetUnit
                move.l  #serialdev,d0
.GetUnit        move.l  d0,a2
                move.l  (_Unit,a5),d2
                beq     .LockDevice
                move.l  d2,a0
                move.l  (a0),d2
.LockDevice     movem.l a2/d2,(OwnDevUnit,a5)

                lea     (TaskName,a5),a0
                bsr     _MakeProcessName
                lea     (TaskName,a5),a1
                SYSLOG  <"This is the name I want: '%s'">,a1
                move.l  ($4).w,a6
                jsr     (_LVOFindTask,a6)
                tst.l   d0
                beq     .NameOkay
                SYSLOG  <"Ooops, there is already a AGetty working !?!">

                tst.l   (Quit,a5)
                beq     .TaskExists
                SYSLOG  <"Ahhh, the user wants us to signal the other AGetty to quit.">
                move.l  d0,a1
                move.l  #SIGBREAKF_CTRL_D,d0            ; Dem Task das
                jsr     (_LVOSignal,a6)                 ; Signal zum Beenden
                bra     .FreeBuffers                    ; schicken

.TaskExists     SYSLOG  <"The user doesn't need us. I'm going to quit.">
                moveq   #MSG_ERROR_TASKEXISTS,d0
                bsr     _GetString
                move.l  d0,-(SP)
                move.l  SP,a1
                moveq   #11,d0
                bsr     _ErrorHandle
                addq.w  #4,SP
                bra     .FreeBuffers

.NameOkay       tst.l   (Quit,a5)               ; Wenn QUIT gewnscht ist,
                bne     .FreeBuffers            ; dann starten wir keinen
                                                ; neuen AGetty, oder?
                sub.l   a1,a1
                SYSLOG  <"We have to set the new name for our task.">
                jsr     (_LVOForbid,a6)
                jsr     (_LVOFindTask,a6)
                move.l  d0,a1
                lea     (TaskName,a5),a0
                move.l  (LN_NAME,a1),d0
                move.l  a0,(LN_NAME,a1)
                jsr     (_LVOPermit,a6)
                move.l  d0,(_OldTaskName,a5)

                lea     (TaskName,a5),a0
                move.l  a0,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOSetProgramName,a6)

;---------------------------- Task-Prioritt setzen -----------------------
                sub.l   a1,a1
                move.l  ($4).w,a6
                jsr     (_LVOFindTask,a6)
                move.l  d0,a1
                move.l  (_Pri,a5),d0
                beq     .UseConfigPri
                move.l  d0,a0
                move.l  (a0),d0
                bra     .SetPri
.UseConfigPri   move.l  (MyPri,a5),d0
                SYSLOG  <"We're going to priority %ld.">,d0
.SetPri         jsr     (_LVOSetTaskPri,a6)
                move.l  d0,(OldPri,a5)

;------------------------- TimerDevice initialisieren ---------------------
                SYSLOG  <"Init the timer device, boy!">
                lea     (_AlarmIOReq,a5),a0
                moveq   #UNIT_VBLANK,d0
                bsr     _InitTimerDevice
                ERRORHANDLE ,4,.RestorePri,(timerdev,PC)

;------------------------- Serial.Device reservieren ----------------------
                SYSLOG  <"We need a signalbit for OwnDevUnit...">
                moveq   #-1,d0
                move.l  ($4).w,a6
                jsr     (_LVOAllocSignal,a6)    ; Signalbit reservieren, das
                cmp.l   #-1,d0                  ; gesetzt wird, wenn ein
                bne     .SigBitOkay             ; anderes Programm unser
                moveq   #MSG_ERROR_SIGNALBIT,d0 ; Device haben will
                bsr     _GetString
                move.l  d0,-(SP)
                move.l  SP,a1
                moveq   #11,d0
                bsr     _ErrorHandle
                addq.w  #4,SP
                bra     .FreeTimer

.SigBitOkay     SYSLOG  <"We got Bit number %ld.">,d0
                move.l  d0,(MySignalBit,a5)
                move    d0,d1
                movem.l (OwnDevUnit,a5),d0/a0

                cmp.l   #-1,(ImmediateAnswer,a5)        ; Kludge to make
                beq     .LockDevOkay                    ; it work with TrapDoor

                SYSLOG  <"Let's allocate the device: '%s', unit: %ld">,a0,d0
                lea     (TaskName,a5),a1
                move.l  (_OwnDevUnitBase,a5),a6
                tst.l   (Bruteforce,a5)         ; Ist das Bruteforce-Flag
                bne     .BruteForce             ; gesetzt, kommt diese
                pea     (.BruteForceEnd,PC)     ; Routine nicht zurck,
                SYSLOG  <"No bruteforce, please.">
                jmp     (_LVOAttemptDevUnit,a6) ; bevor das Device nicht
.BruteForce     SYSLOG  <"BruteForce!!">
                jsr     (_LVOLockDevUnit,a6)    ; frei ist!
.BruteForceEnd  tst.l   d0
                beq     .LockDevOkay
                SYSLOG  <"Something went wrong with ODU!">
                move.l  d0,a0
                cmp.b   #ODUERR_LEADCHAR,(a0)
                bne     .HolderName
                pea     (1,a0)
                move.l  SP,a1
                moveq   #11,d0
                bsr     _ErrorHandle
                addq.w  #4,SP
                bra     .FreeSignalBit
.HolderName     move.l  d0,-(SP)
                pea     (a2)
                move.l  SP,a1
                moveq   #10,d0
                bsr     _ErrorHandle
                addq.w  #8,SP
                bra     .FreeSignalBit
.LockDevOkay    SYSLOG  <"Ha! I have that fuck*** device!">
;------------------------- Serial.Device reservieren ----------------------
                SYSLOG  <"I need a buffer of %ld byte.">,#BUFFERSIZE
                lea     (_RememberKey,a5),a0
                move.l  #BUFFERSIZE,d0
                move.l  #MEMF_ANY|MEMF_CLEAR,d1
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOPTAllocRemember,a6)
                ERRORHANDLE (_InternalBuffer,a5),2,.FreeSignalBit,BUFFERSIZE,(AnyMemString,PC)
                move.l  d0,a0

                SYSLOG  <"And now open the device:">

                move.l  ($4).W,a6
                jsr     (_LVOCreateMsgPort,a6)          ; SerialMP
                SYSLOG  <"CreateMsgPort() = 0x%lx">,d0
                move.l  d0,(_SerialMP,a5)
                INITDEVERROR (.FreeDevUnit,PC)

                move.l  d0,a0
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                SYSLOG  <"CreateIORequest() = 0x%lx">,d0
                move.l  d0,(_SerialIOReq,a5)
                INITDEVERROR (.FreeMsgPorts,PC)
                move.l  d0,a1
                move.b  #PARAMETERS,(IO_SERFLAGS,a1)
                move.l  (4+OwnDevUnit,a5),a0
                move.l  (OwnDevUnit,a5),d0
                moveq   #0,d1
                jsr     (_LVOOpenDevice,a6)
                SYSLOG  <"OpenDevice() = %ld (0 is 'NO ERROR')">,d0
                tst.l   d0
                beq     .DevOkay
                pea     (.FreeIOReqs,PC)
                bra     .OpenDevError

.DevOkay        jsr     (_LVOCreateMsgPort,a6)          ; ReadMP
                SYSLOG  <"CreateMsgPort() = 0x%lx">,d0
                move.l  d0,(_ReadMP,a5)
                INITDEVERROR (.CloseSerDevice,PC)
                move.l  d0,a0
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                SYSLOG  <"CreateIORequest() = 0x%lx">,d0
                move.l  d0,(_ReadIOReq,a5)
                INITDEVERROR (.CloseSerDevice,PC)
                move.l  (_SerialIOReq,a5),a0
                move.l  d0,a1
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCopyMem,a6)
                move.l  (_ReadIOReq,a5),a0
                move.l  (_ReadMP,a5),(MN_REPLYPORT,a0)

                jsr     (_LVOCreateMsgPort,a6)          ; WriteMP
                SYSLOG  <"CreateMsgPort() = 0x%lx">,d0
                move.l  d0,(_WriteMP,a5)
                INITDEVERROR (.CloseSerDevice,PC)
                move.l  d0,a0
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                SYSLOG  <"CreateIORequest() = 0x%lx">,d0
                move.l  d0,(_WriteIOReq,a5)
                INITDEVERROR (.CloseSerDevice,PC)
                move.l  (_SerialIOReq,a5),a0
                move.l  d0,a1
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCopyMem,a6)
                move.l  (_WriteIOReq,a5),a0
                move.l  (_WriteMP,a5),(MN_REPLYPORT,a0)
                bra     .SerialOkay

.OpenDevError   moveq   #4,d0
                lea     (4+OwnDevUnit,a5),a1
                bsr     _ErrorHandle
                rts
.SerialOkay
                SYSLOG  <"Wow, that was a lot of work, but the device is ready.">
;---------------------------- Modem initialisieren ------------------------
                bsr     _InitModem
                tst.l   d0
                beq     .CloseSerDevice

;--------------------- Statusmeldung und Log-Eintrag ausgeben -------------
                moveq   #MSG_COPYRIGHT,d0
                bsr     _GetString
                move.l  d0,a0
                pea     (NoPToolError,PC)
                move.l  SP,a1
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOPrintFSimple,a6)
                addq.w  #4,SP

                moveq   #MSG_LOG_STARTUP,d0
                move.l  (OwnDevUnit,a5),-(SP)
                move.l  (4+OwnDevUnit,a5),-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                add.w   #4,SP
                add.w   #8,SP

;----------------------------- Auf Anrufe warten -------------------------
                SYSLOG  <"Startup worked fine!">

                tst.l   (Answer,a5)
                bne     .AnswerImmediat

                SYSLOG  <"No Answer-flag set!">

.MainLoop       SYSLOG  <"Due to a bug in WaitIO(), I have to clear the signals myself.">
                moveq   #0,d0
                moveq   #0,d1
                move.l  (_ReadMP,a5),a1
                move.b  (MP_SIGBIT,a1),d0
                bset    d0,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOCheckSignal,a6)

                SYSLOG  <"I'm going to set up the RING-detection.">
                move.l  (_ReadIOReq,a5),a1
                move.l  #1,(IO_LENGTH,a1)
                move.l  (_InternalBuffer,a5),(IO_DATA,a1)
                move.w  #CMD_READ,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVOSendIO,a6)

                SYSLOG  <"RING-detection is on job. Waiting for signals to reach me.">

                move.l  (_ReadMP,a5),a1
                moveq   #0,d0
                moveq   #0,d2
                move.b  (MP_SIGBIT,a1),d2
                bset    d2,d0
                move.l  (MySignalBit,a5),d1
                bset    d1,d0
                bset    #SIGBREAKB_CTRL_D|SIGBREAKB_CTRL_C,d0
                jsr     (_LVOWait,a6)

                SYSLOG  <"Received a signal: 0x%lx">,d0
                btst    #SIGBREAKB_CTRL_D,d0
                bne     .ReceivedBreak
                btst    d2,d0
                beq     .NoIncomming
.Answer         SYSLOG  <"Seems like somebody wants to talk to us. I should answer the phone...">
                move.l  (_ReadIOReq,a5),a1
                move.l  ($4).w,a6
                jsr     (_LVOWaitIO,a6)
.AnswerImmediat bsr     _GetCall
                bsr     _InitModem
;                tst.l   d0                     ; Nicht beenden, wenn die
;                beq     .leave                 ; Modeminitialisierung fehl-
                tst.l   (SingleRun,a5)          ; schlgt
                beq     .MainLoop
                tst.l   (Answer,a5)
                beq     .MainLoop
                SYSLOG  <"Shit... SINGLERUN is set. I have to leave!">
                bra     .leave
.NoIncomming    SYSLOG  <"Somebody wants my device!">
                move.l  (_ReadIOReq,a5),a1
                jsr     (_LVOAbortIO,a6)
                move.l  (_ReadIOReq,a5),a1
                jsr     (_LVOWaitIO,a6)

                bsr     _BorrowDevice
                tst.l   d0
                bne     .BorrowError
                bsr     _InitModem
                bra     .MainLoop

.BorrowError    SYSLOG  <"Something went wrong with the BorrowDevice() routine. I quit!">
                moveq   #4,d0
                lea     (4+OwnDevUnit,a5),a1
                bsr     _ErrorHandle
;               bra     .ReceivedBreak


.ReceivedBreak  SYSLOG  <"Whew... I got a CTRL-D!">
                move.l  ($4).w,a6
                move.l  (_ReadIOReq,a5),a1
                jsr     (_LVOAbortIO,a6)
                move.l  (_ReadIOReq,a5),a1
                jsr     (_LVOWaitIO,a6)

;-------------------------- Systemumgebung freigeben ----------------------
.leave          moveq   #MSG_LOG_BYEBYE,d0
                move.l  (OwnDevUnit,a5),-(SP)
                move.l  (4+OwnDevUnit,a5),-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                add.w   #4,SP
                add.w   #8,SP

.CloseSerDevice lea     (_SerialIOReq,a5),a1
                tst.l   (a1)
                beq     .FreeMsgPorts
                move.l  (a1),a1
                tst.l   (IO_DEVICE,a1)
                beq     .FreeIOReqs
                move.l  ($4).w,a6
                SYSLOG  <"CloseDevice()">
                jsr     (_LVOCloseDevice,a6)

.FreeIOReqs     lea     (_SerialIOReq,a5),a2
                move.l  ($4).w,a6
                moveq   #3-1,d2
...             tst.l   (a2)
                beq     .FreeMsgPorts
                move.l  (a2),a0
                SYSLOG  <"DeleteIORequest()">
                jsr     (_LVODeleteIORequest,a6)
                subq.w  #4,a2
                dbra    d2,...

.FreeMsgPorts   lea     (_SerialMP,a5),a2
                move.l  ($4).w,a6
                moveq   #3-1,d2
...             tst.l   (a2)
                beq     .FreeDevUnit
                move.l  (a2),a0
                SYSLOG  <"DeleteMsgPort()">
                jsr     (_LVODeleteMsgPort,a6)
                subq.w  #4,a2
                dbra    d2,...

.FreeDevUnit    movem.l (OwnDevUnit,a5),a0/d0
                move.l  (_OwnDevUnitBase,a5),a6
                SYSLOG  <"FreeDevUnit()">
                jsr     (_LVOFreeDevUnit,a6)

.FreeSignalBit  move.l  (MySignalBit,a5),d0
                move.l  ($4).W,a6
                SYSLOG  <"FreeSignal()">
                jsr     (_LVOFreeSignal,a6)

.FreeTimer      lea     (_AlarmIOReq,a5),a0
                bsr     _CloseTimerDevice

.RestorePri     sub.l   a1,a1
                move.l  ($4).w,a6
                jsr     (_LVOFindTask,a6)
                move.l  d0,a1
                move.l  (OldPri,a5),d0
                cmp.b   #-1,d0
                beq     .RestoreTskName
                SYSLOG  <"Reset to priority %ld.">,d0
                jsr     (_LVOSetTaskPri,a6)

.RestoreTskName sub.l   a1,a1
                move.l  ($4).w,a6
                jsr     (_LVOFindTask,a6)
                move.l  d0,a1
                tst.l   (_OldTaskName,a5)
                beq     .FreeBuffers
                SYSLOG  <"Restore taskname: '%s'">,(_OldTaskName,a5)
                jsr     (_LVOForbid,a6)
                move.l  (_OldTaskName,a5),(LN_NAME,a1)
                jsr     (_LVOPermit,a6)

.FreeBuffers    lea     (_RememberKey,a5),a0
                move.l  (_PToolBase,a5),a6
                SYSLOG  <"Free all the allocated memory.">
                jsr     (_LVOPTFreeRemember,a6)

.FreeRDArgs     move.l  (_RDArgsStruct,a5),d1
                move.l  (_DOSBase,a5),a6
                SYSLOG  <"FreeArgs()">
                jsr     (_LVOFreeArgs,a6)

.CloseCatalog   tst.l   (_LocaleBase,a5)
                beq     .CloseOwnDevLib
                move.l  (_MessageCatalog,a5),a0
                move.l  (_LocaleBase,a5),a6
                SYSLOG  <"CloseCatalog()">
                jsr     (_LVOCloseCatalog,a6)

.CloseOwnDevLib move.l  (_OwnDevUnitBase,a5),a1
                move.l  ($4).W,a6
                SYSLOG  <"Close OwnDevUnit.Library">
                jsr     (_LVOCloseLibrary,a6)

        IFNE DEBUG
                bsr     _UnInitSysLog
        ENDC

.ClosePToolLib  move.l  (_PToolBase,a5),a1
                move.l  ($4).W,a6
                jsr     (_LVOCloseLibrary,a6)

.Exit           unlk    a5
                moveq   #RETURN_OK,d0
                rts


;***************************************************************************
;*                                                                         *
;* SEKTION: Unterroutinen                                                  *
;*                                                                         *
;***************************************************************************

;------------------------------- Getty-Routinen ---------------------------
_MakeLogin:
                ; Fhrt den Login-Vorgang durch und liefert den Namen
                ; zurck, wenn das Passwort gltig war.
                ;
                ; Parameter: keine
                ; Result   : D0=&LoginName / NULL

                SYSLOG  <"Welcome to MakeLogin()">

                PUSHM   d2-d4/a2-a3/a6
                link    a4,#-64

                moveq   #3,d3                   ; Login-Versuche!
                move.l  (Timeout,a5),d2

.LoginLoop      move.l  #60,(Timeout,a5)
                lea     (LogInPrompt,PC),a0
                moveq   #LogInPrompt_len,d0
                SYSLOG  <"MakeLogin: Write login prompt.">
                bsr     _WriteSerial
                lea     (ReturnStr,PC),a0
                moveq   #1,d0
                SYSLOG  <"MakeLogin: Wait for return.">
                move.b  #$FF,(ModemEcho,a5)
                bsr     _WaitForString
                clr.b   (ModemEcho,a5)
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                bne     .LoginValid
                moveq   #0,d0
                bra     .exit
.LoginValid     cmp.b   #$0D,(a0)               ; leere Eingabe -> nochmal!
                beq     .LoginLoop
                clr.b   (31,a0)                 ; String darf nicht lnger
                clr.b   (-1,a0,d0.l)            ; als 32 Byte sein!
                lea     (-64,a4),a1             ; Login-Namen retten
                bsr     _FlushCopy

                lea     (-64,a4),a0
                move.l  (_PasswdBuffer,a5),a1
                addq.w  #4,a1
                lea     (ConfigEntryBuf,a5),a2
                bsr     _ParseLine
                move.l  d0,d4
                beq     .GetPassword

                move.l  (cl_Password,a2),a0
                SYSLOG  <"MakeLogin: The password for this user is '%s'.">,a0
                cmp.b   #'*',(a0)               ; Wenn Passwort gleich "*",
                bne     .GetPassword            ; die Abfrage berspringen
                tst.b   (1,a0)
                beq     .LoginOkay

.GetPassword    lea     (PasswordPrompt,PC),a0
                moveq   #PasswordPrompt_len,d0
                SYSLOG  <"MakeLogin: Ask caller for password.">
                bsr     _WriteSerial
                lea     (ReturnStr,PC),a0
                moveq   #1,d0
                SYSLOG  <"MakeLogin: wait for return">
                bsr     _WaitForString
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                bne     .PasswdValid
                moveq   #0,d0
                bra     .exit
.PasswdValid    clr.b   (31,a0)
                clr.b   (-1,a0,d0.l)
                lea     (-32,a4),a1             ; Passwort retten
                bsr     _FlushCopy

                tst.l   d4
                beq     .LoginFailed

                SYSLOG  <"MakeLogin: Check provided password for validity.">
                lea     (-32,a4),a0
                move.l  (cl_Password,a2),a1
                bsr     _Len
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOCmpStrings,a6)
                tst.l   d0
                bne     .LoginOkay

.LoginFailed    lea     (LoginFailedMsg,PC),a0
                moveq   #-1,d0
                bsr     _WriteSerial
                moveq   #MSG_LOG_LOGINFAILED,d0
                pea     (-32,a4)
                pea     (-64,a4)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                add.w   #12,SP
                subq.l  #1,d3
                SYSLOG  <"MakeLogin: Login failed. %ld attempts left">,d3
                tst.l   d3
                bne     .LoginLoop
                moveq   #0,d0
                bra     .exit

.LoginOkay      move.l  (cl_User,a2),d0
.exit           move.l  d2,(Timeout,a5)
                unlk    a4
                POPM
                SYSLOG  <"You're leaving MakeLogin()">
                rts


_GetCall:
                ; Nimmt einen Anruf entgegen
                ;
                ; Parameter: keine
                ; Result   : TRUE/FALSE

                PUSHM   d7/a6
                moveq   #0,d7

                cmp.l   #-1,(Wire7,a5)          ; Auf dem Nullmodemkabel ver-
                beq.b   .Connected              ; gessen wir das!

                cmp.l   #-1,(Answer,a5)         ; Im Answermode vergessen
                beq.b   .SkipRingCheck          ; wir das!

                cmp.l   #-1,(ImmediateAnswer,a5)
                beq.b   .WaitOnRing             ; Reicht ein "RING"?

                move.l  (_InternalBuffer,a5),a0
                moveq   #-1,d0
                bsr     _ReadSerial

.WaitOnRing     move.l  #10,(Timeout,a5)
                lea     (RingStr,PC),a0
                moveq   #4,d0                   ; "RING"
                bsr     _WaitForString
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit

.SkipRingCheck  move.l  #1,(Answer,a5)
                bsr     _MakeConnect
                move.l  d0,d2
                bne.b   .Connected
                moveq   #MSG_LOG_CONNECTFAILED,d0
                move.l  (OwnDevUnit,a5),-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP
                bra     .HangUp

.Connected      tst.l   (LogConnect,a5)
                beq.b   .SkipLogEntry
                moveq   #MSG_LOG_CONNECT,d0
                pea     (ConnectMsg,a5)
                move.l  (OwnDevUnit,a5),-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                add.w   #12,SP

.SkipLogEntry   move.l  #-1,(CarrierUp,a5)
                move.l  #-1,d7

                cmp.l   #-2,d2
                bne.b   .DataConnect
                lea     (FaxName,PC),a0
                move.l  (_PasswdBuffer,a5),a1
                addq.w  #4,a1
                lea     (ConfigEntryBuf,a5),a2
                bsr     _ParseLine
                tst.l   d0
                beq     .HangUp
                bsr     _StartMailer
                bra     .HangUp

.DataConnect    move.l  (_LoginText,a5),d0
                bne.b   .SendLoginText
                move.l  (_StdLoginText,a5),d0
                beq.b   .NoLoginText
.SendLoginText  move.l  d0,a0
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOLoadFile,a6)
                tst.l   d0
                beq.b   .NoLoginText
                move.l  d0,a2
                move.l  d0,a0
                move.l  (a0)+,d0
                beq.b   .FreeTxtBuffer
                bsr     _ConvertModemText
                movem.l d0-d1/a0-a1,-(SP)
                moveq   #1*50,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)
                movem.l (SP)+,d0-d1/a0-a1
                bsr     _WriteSerial
.FreeTxtBuffer  move.l  a2,a1
                move.l  ($4).w,a6
                jsr     (_LVOFreeVec,a6)

.NoLoginText    bsr     _MakeLogin
                tst.l   d0
                beq.b   .HangUp

                bsr     _StartMailer

.HangUp         clr.l   (CarrierUp,a5)
                bsr     _OffHook
                tst.l   d0
                bne.b   .exit
                moveq   #MSG_LOG_HANGUPFAILED,d0
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #4,SP

.exit           move.l  (_DOSBase,a5),a6
                move.l  #2*50,d1
                jsr     (_LVODelay,a6)

                move.l  d7,d0
                POPM
                rts


_StartMailer:
                ; Fhrt das im Passwd-File angegebene Kommando aus
                ;
                ; Parameter: keine
                ; Result   : keins

                PUSHM   d0-d7/a0-a4/a6

                lea     (ConfigEntryBuf,a5),a1
                move.l  (cl_CommandToRun,a1),a1
                cmp.b   #'*',(a1)
                beq     .StartShell

                lea     (CallTemplate,PC),a0
                movem.l (OwnDevUnit,a5),d0-d1
                exg.l   d0,d1
                movem.l d0-d1,-(SP)
                lea     (ConfigEntryBuf,a5),a1
                move.l  (cl_CommandToRun,a1),a1
                pea     (a1)
                move.l  SP,a1
                lea     (.CopyChar,PC),a2
                move.l  (_InternalBuffer,a5),a3
                move.l  ($4).w,a6
                jsr     (_LVORawDoFmt,a6)
                add.w   #12,SP

                moveq   #MSG_LOG_LOGGEDIN,d0
                move.l  (_InternalBuffer,a5),-(SP)
                lea     (ConfigEntryBuf,a5),a0
                move.l  (cl_User,a0),-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                add.w   #12,SP

                lea     (ConfigEntryBuf,a5),a2
                move.l  (cl_HomeDir,a2),a0
                tst.b   (a0)
                bne.b   .DirValid
                lea     (DefaultDir,PC),a0
.DirValid       move.l  a0,d1
                move.l  #ACCESS_READ,d2
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOLock,a6)
                move.l  d0,d1
                jsr     (_LVOCurrentDir,a6)
                move.l  d0,-(SP)

                bsr     _SetLocalVars

                move.l  (_InternalBuffer,a5),d1
                moveq   #0,d2
                moveq   #0,d3
                jsr     (_LVOExecute,a6)

                bsr     _UnsetLocalVars

                move.l  (SP)+,d1
                jsr     (_LVOCurrentDir,a6)
                move.l  d0,d1
                jsr     (_LVOUnLock,a6)

                POPM
                rts

.CopyChar       move.b  d0,(a3)+
                rts


.StartShell             ; Redirect STDIN and STDOUT to the serial.device
                        ; before starting the program and start a process
                        ; to pipe the charcters.

                sub.w   #44,SP
                move.l  SP,a3
                lea     (16,a3),a4

                move.l  ($4).w,a6
                sub.l   a1,a1
                jsr     (_LVOFindTask,a6)
                lea     (32,a3),a0
                bsr     _LongToStr

                lea     (PipeName,PC),a0
                move.l  a3,a1
...             move.b  (a0)+,(a1)+
                bne.b   ...
                lea     (32,a3),a0
                subq.w  #1,a1
...             move.b  (a0)+,(a1)+
                bne.b   ...
                subq.w  #1,a1
                move.b  #'_',(a1)+
                move.b  #'i',(a1)+
                move.b  #'n',(a1)+
                clr.b   (a1)+
                SYSLOG  <"StartShell: Input-pipe `%s'">,a3

                lea     (PipeName,PC),a0
                move.l  a4,a1
...             move.b  (a0)+,(a1)+
                bne.b   ...
                lea     (32,a3),a0
                subq.w  #1,a1
...             move.b  (a0)+,(a1)+
                bne.b   ...
                subq.w  #1,a1
                move.b  #'_',(a1)+
                move.b  #'o',(a1)+
                move.b  #'u',(a1)+
                move.b  #'t',(a1)+
                clr.b   (a1)
                SYSLOG  <"StartShell: Output-pipe: `%s'">,a4

                move.l  a3,a0
                move.l  a4,a1
                bsr     _InstallPipingProcess
                tst.l   d0
                bne     .ExecShell
                SYSLOG  <"StartShell: Failed installing the piping process!">
                add.w   #44,SP
                bra     .ShellExit

.ExecShell      lea     (ShellTemplate,PC),a0
                pea     (a4)
                pea     (a3)
                lea     (ConfigEntryBuf,a5),a1
                move.l  (cl_CommandToRun,a1),a1
                pea     (1,a1)
                move.l  SP,a1
                lea     (.CopyChar,PC),a2
                move.l  (_InternalBuffer,a5),a3
                move.l  ($4).w,a6
                jsr     (_LVORawDoFmt,a6)
                add.w   #12,SP
                add.w   #44,SP

                SYSLOG  <"StartShell: Still missing the log-entry here!">

                lea     (ConfigEntryBuf,a5),a2
                move.l  (cl_HomeDir,a2),a0
                tst.b   (a0)
                bne.b   .DirValid2
                lea     (DefaultDir,PC),a0
.DirValid2      move.l  a0,d1
                move.l  #ACCESS_READ,d2
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOLock,a6)
                move.l  d0,d1
                jsr     (_LVOCurrentDir,a6)
                move.l  d0,d5
                bsr     _SetLocalVars

                move.l  (_InternalBuffer,a5),d1
                SYSLOG  <"StartShell: Executing `%s'">,d1
                moveq   #0,d2
                moveq   #0,d3
                jsr     (_LVOExecute,a6)

                bsr     _RemovePipingProcess

                bsr     _UnsetLocalVars
                move.l  d5,d1
                jsr     (_LVOCurrentDir,a6)
                move.l  d0,d1
                jsr     (_LVOUnLock,a6)

.ShellExit      POPM
                rts


_SetLocalVars:
                ; Setzt die lokalen Variablen auf die entsprechenden Werte.
                ;
                ; Parameter: keine
                ; Result   : keins

                PUSHM   d0-d5/a0-a3

                lea     (ConfigEntryBuf,a5),a2
                moveq   #7-1,d5                ; Passwd-Zeile setzen
                lea     (_User,a5),a3
.loop           move.l  (a3),d1
                beq.b   .SkipSetVar
                move.l  d1,a0
                tst.b   (a0)
                beq.b   .SkipSetVar
                move.l  (a2),d2
                moveq   #-1,d3
                move.l  #GVF_LOCAL_ONLY,d4
                jsr     (_LVOSetVar,a6)         ; DOS-Variablen setzen
.SkipSetVar     addq.w  #4,a2
                addq.w  #4,a3
                dbra    d5,.loop

                POPM
                rts


_UnsetLocalVars:
                ; Lscht die lokalen Variablen
                ;
                ; Parameter: keine
                ; Result   : keins

                PUSHM   d0-d5/a0-a3

                lea     (ConfigEntryBuf,a5),a2
                moveq   #7-1,d5
                lea     (_User,a5),a3
.loop           move.l  (a3),d1
                beq.b   .SkipDelVar
                move.l  d1,a0
                tst.b   (a0)
                beq.b   .SkipDelVar
                move.l  #GVF_LOCAL_ONLY,d2
                jsr     (_LVODeleteVar,a6)      ; DOS-Variablen lschen
.SkipDelVar     addq.w  #4,a3
                dbra    d5,.loop

                POPM
                rts


_BorrowDevice:
                ; Gibt das Device kurzfristig frei und wartet darauf, da
                ; es wieder frei wird.
                ;
                ; Parameter: keine
                ; Result   : D0=Result von OpenDevice()

                SYSLOG  <"Welcome to BorrowDevice()">

                PUSHM   a2/a6

                move.l  (_SerialIOReq,a5),a1
                move.l  ($4).w,a6
                SYSLOG  <"BorrowDevice: CloseDevice(0x%lx)">,a1
                jsr     (_LVOCloseDevice,a6)

                movem.l (OwnDevUnit,a5),a0/d0
                SYSLOG  <"BorrowDevice: FreeDevUnit('%s', %ld)">,a0,d0
                move.l  (_OwnDevUnitBase,a5),a6
                jsr     (_LVOFreeDevUnit,a6)

                SYSLOG  <"BorrowDevice: Delay 2 seconds.">
                moveq   #2*50,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)

                SYSLOG  <"BorrowDevice: Check the new owner of the new device">
                movem.l (OwnDevUnit,a5),d0/a0
                move.l  (_OwnDevUnitBase,a5),a6
                jsr     (_LVOOwnerDevUnit,a6)
                move.l  d0,a0
                cmp.b   #ODUERR_LEADCHAR,(a0)
                bne.b   .NoUNKNOWN
                addq.w  #1,a0
.NoUNKNOWN

                move.l  d0,-(SP)
                moveq   #MSG_LOG_BORROWDEVICE,d0
                move.l  (OwnDevUnit,a5),-(SP)
                move.l  (4+OwnDevUnit,a5),-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP
                addq.w  #8,SP

                SYSLOG  <"BorrowDevice: Delay 2 seconds.">
                move.l  #2*50,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)

                move.l  (MySignalBit,a5),d1
                movem.l (OwnDevUnit,a5),d0/a0
                lea     (TaskName,a5),a1
                move.l  (_OwnDevUnitBase,a5),a6
                SYSLOG  <"BorrowDevice: Call LockDevUnit() to get my device back.">
                jsr     (_LVOLockDevUnit,a6)
                tst.l   d0
                bne     .exit

                SYSLOG  <"BorrowDevice: Got it.">
                moveq   #MSG_LOG_GOTDEVBACK,d0
                move.l  (OwnDevUnit,a5),-(SP)
                move.l  (4+OwnDevUnit,a5),-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP
                addq.w  #4,SP

                SYSLOG  <"BorrowDevice: open the device again.">
                move.l  (_SerialIOReq,a5),a1
                move.b  #PARAMETERS,(IO_SERFLAGS,a1)
                move.l  (4+OwnDevUnit,a5),a0
                move.l  (OwnDevUnit,a5),d0
                moveq   #0,d1
                move.l  ($4).w,a6
                jsr     (_LVOOpenDevice,a6)

.exit           POPM
                SYSLOG  <"You're leaving BorrowDevice()">
                rts

;------------------------------- Piping-Process ---------------------------
_InstallPipingProcess:
                ; Erzeugt einen Task der whrend das aufgerufende
                ; Tool luft einkommende Daten vom Device auf
                ; den Standard-Input piped und umgekehrt.
                ;
                ; Parameter: a0=&PipeName_in A1=&Pipename_out
                ; Result   : D0=success

                SYSLOG  <"Welcome to InstallPipingProcess()">

                PUSHM   d1-d7/a0-a4/a6

                move.l  a0,a3
                move.l  a1,a4

                move.l  #PipingProc_len,d0
                move.l  #MEMF_ANY|MEMF_CLEAR,d1
                move.l  ($4).w,a6
                jsr     (_LVOAllocVec,a6)
                SYSLOG  <"InstallPipingProcess: copy task to 0x%lx">,d0
                move.l  d0,(_PipingTaskAdr,a5)
                beq     .exit

                lea     (_PipingProc,PC),a0
                move.l  d0,a2
                move.l  #PipingProcC_len-1,d0
...             move.b  (a0)+,(a2)+
                dbra    d0,...
                move.l  a5,(a2)+

                move.l  SP,d7
                move.l  #TAG_DONE,-(SP)

                move.l  a3,d1
                SYSLOG  <"InstallPipingProcess: input-pipe `%s'">,d1
                move.l  #MODE_NEWFILE,d2
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOOpen,a6)
                move.l  d0,-(SP)
                beq     .ErrorExit
                SYSLOG  <"InstallPipingProcess: input-handle 0x%lx">,d0
                move.l  #NP_Output,-(SP)
                move.l  a4,d1
                move.l  #MODE_OLDFILE,d2
                jsr     (_LVOOpen,a6)
                move.l  d0,-(SP)
                beq     .ErrorExit2
                SYSLOG  <"InstallPipingProcess: output-handle 0x%lx">,d0
                move.l  #NP_Input,-(SP)
                move.l  (_PipingTaskAdr,a5),-(SP)
                move.l  #NP_Entry,-(SP)
                pea     (.PipingProcName,PC)
                move.l  #NP_Name,-(SP)
;                move.l  (MyPri,a5),-(SP)
                move.l  #-1,-(SP)
                move.l  #NP_Priority,-(SP)
                move.l  SP,d1
                jsr     (_LVOCreateNewProc,a6)
                move.l  d0,(_PipingTask,a5)
                beq     .ErrorExit3
.leave          move.l  d7,SP
                SYSLOG  <"InstallPipingProcess: Taskaddress: 0x%lx">,d0
.exit           POPM
                SYSLOG  <"You're leaving InstallPipingProcess()">
                rts

.ErrorExit3     add.w   #3*2*4,SP       ; Remove Pri, Name, Entry
                move.l  (SP)+,d1
                jsr     (_LVOClose,a6)
                addq.w  #4,SP           ; Remove Tag

.ErrorExit2     add.w   #2*2*4,SP       ; Remove failed handle + Tag
                move.l  (SP)+,d1
                jsr     (_LVOClose,a6)

.ErrorExit      move.l  (_PipingTaskAdr,a5),a1
                move.l  ($4).w,a6
                jsr     (_LVOFreeVec,a6)
                clr.l   (_PipingTaskAdr,a5)
                moveq   #0,d0
                bra     .leave

.PipingProcName CSTR    "AGetty Piping Process"


_PipingProc:
                PUSHM   d0-d7/a0-a6
                move.l  (.ProcessBase,PC),a5

                lea     (.ReadPort,PC),a4
                move.l  ($4).w,a6
                jsr     (_LVOCreateMsgPort,a6)
                SYSLOG  <"Piping: MessagePort 0x%lx">,d0
                move.l  d0,(a4)+
                beq     .exit
                move.l  d0,a2
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                SYSLOG  <"Piping: IORequest 0x%lx">,d0
                move.l  d0,(a4)+
                beq     .RemReadIOReq
                move.l  d0,a3
                move.l  (_SerialIOReq,a5),a0
                move.l  d0,a1
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCopyMem,a6)
                move.l  a2,(MN_REPLYPORT,a3)

                jsr     (_LVOCreateMsgPort,a6)
                SYSLOG  <"Piping: MessagePort 0x%lx">,d0
                move.l  d0,(a4)+
                beq     .RemWriteMP
                move.l  d0,a2
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                SYSLOG  <"Piping: IORequest 0x%lx">,d0
                move.l  d0,(a4)+
                beq     .RemWriteIOReq
                move.l  d0,a3
                move.l  (_SerialIOReq,a5),a0
                move.l  d0,a1
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCopyMem,a6)
                move.l  a2,(MN_REPLYPORT,a3)

                jsr     (_LVOCreateMsgPort,a6)
                SYSLOG  <"Piping: MessagePort 0x%lx">,d0
                move.l  d0,(a4)+
                beq     .RemPktMP

                move.l  #DOS_STDPKT,d1
                moveq   #0,d2
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOAllocDosObject,a6)
                SYSLOG  <"Piping: DosPacket 0x%lx">,d0
                move.l  d0,(a4)+
                beq     .FreeDosObj

                jsr     (_LVOInput,a6)
                SYSLOG  <"Piping: Input 0x%lx">,d0
                move.l  d0,(a4)+
                jsr     (_LVOOutput,a6)
                SYSLOG  <"Piping: Output 0x%lx">,d0
                move.l  d0,(a4)+

                lea     (.ReadBuf,PC),a2
                move.l  #260,d0
                move.l  #MEMF_ANY,d1
                move.l  ($4).w,a6
                jsr     (_LVOAllocVec,a6)
                SYSLOG  <"Piping: Readbuffer 0x%lx">,d0
                move.l  d0,(a2)+
                beq     .FreeBuffers
                add.l   #256,d0
                move.l  d0,(a2)+

                SYSLOG  <"PipingProcess works!">

.MainLoop       lea     (.SerFlag,PC),a0
                tst.b   (a0)
                bne     .SkipSerReadSetup
                moveq   #0,d0
                moveq   #0,d1
                move.l  (.ReadPort,PC),a1
                move.b  (MP_SIGBIT,a1),d0
                bset    d0,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOCheckSignal,a6)

                move.l  (.ReadIOReq,PC),a1
                move.l  #1,(IO_LENGTH,a1)
                lea     (.ReadBuf,PC),a0
                move.l  a0,(IO_DATA,a1)
                move.w  #CMD_READ,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                SYSLOG  <"Piping: Setup new serial read request">
                jsr     (_LVOSendIO,a6)
                lea     (.SerFlag,PC),a0
                move.b  #-1,(a0)

.SkipSerReadSetup:
                lea     (.ReadFlag,PC),a0
                tst.b   (a0)
                bne     .SkipReadSetup
                moveq   #0,d0
                moveq   #0,d1
                move.l  (.PktPort,PC),a1
                move.b  (MP_SIGBIT,a1),d0
                bset    d0,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOCheckSignal,a6)

                move.l  (.DosPkt,PC),a0
                move.l  #ACTION_READ,(dp_Type,a0)
                move.l  (.Input,PC),a1
                add.l   a1,a1
                add.l   a1,a1
                move.l  (fh_Arg1,a1),(dp_Arg1,a0)
                lea     (.PktBuf,PC),a2
                move.l  a2,(dp_Arg2,a0)
                move.l  #1,(dp_Arg3,a0)
                move.l  a0,d1
                move.l  (fh_Type,a1),d2
                move.l  (.PktPort,PC),d3
                SYSLOG  <"Piping: Setup new pipe read request">
                jsr     (_LVOSendPkt,a6)
                lea     (.ReadFlag,PC),a0
                move.b  #-1,(a0)

.SkipReadSetup:
.WaitLoop       moveq   #0,d2
                move.l  #SIGBREAKF_CTRL_C,d0
                move.l  (.PktPort,PC),a1
                move.b  (MP_SIGBIT,a1),d2
                bset    d2,d0
                move.l  (.ReadPort,PC),a1
                move.b  (MP_SIGBIT,a1),d2
                bset    d2,d0
                move.l  ($4).w,a6
                SYSLOG  <"Piping: Wait for signals: 0x%lx">,d0
                jsr     (_LVOWait,a6)
                SYSLOG  <"Piping: Got signals: 0x%lx">,d0
                btst    #SIGBREAKB_CTRL_C,d0
                bne     .ReceivedBreak
                btst    d2,d0
                bne     .ReceivedChar

.InputEvent     SYSLOG  <"Piping: InputEvent">
                move.l  (.PktPort,PC),a0
                move.l  ($4).w,a6
                jsr     (_LVOWaitPort,a6)
                move.l  (.PktPort,PC),a0
                jsr     (_LVOGetMsg,a6)

                move.l  (.DosPkt,PC),a0
                SYSLOG  <"Piping: dp_Res = %ld">,(dp_Res1,a0)
                tst.l   (dp_Res1,a0)
                beq     .ReadError
                bmi     .ReadError

                move.l  (.WriteIOReq,PC),a1
                move.l  #1,(IO_LENGTH,a1)
                lea     (.PktBuf,PC),a0
                move.l  a0,(IO_DATA,a1)
                move.w  #CMD_WRITE,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                SYSLOG  <"Piping: Send `%s' to serial.device">,a0
                jsr     (_LVODoIO,a6)

.ReadError      SYSLOG  <"Piping: Clear read flag">
                lea     (.ReadFlag,PC),a0
                clr.b   (a0)
                bra     .MainLoop

.ReceivedChar   SYSLOG  <"Piping: Serial-Event!!">
                bclr    d2,d0
                beq.b   .NoReadSigLeft
                SYSLOG  <"Piping: SignalTask again: 0x%lx">,d0
                move.l  ($4).w,a6
                move.l  (ThisTask,a6),a1
                jsr     (_LVOSignal,a6)

.NoReadSigLeft  move.l  (.ReadIOReq,PC),a1
                move.l  ($4).W,a6
                jsr     (_LVOWaitIO,a6)
                lea     (.SerFlag,PC),a0
                clr.b   (a0)
                SYSLOG  <"Piping: Clear serial read flag">

                move.l  (.ReadIOReq,PC),a1
                move.w  #SDCMD_QUERY,(IO_COMMAND,a1)
                jsr     (_LVODoIO,a6)
                move.l  (.ReadIOReq,PC),a1
                SYSLOG  <"Piping: Still %ld chars waiting in serial buffer">,(IO_ACTUAL,a1)
                move.l  (IO_ACTUAL,a1),d3
                beq.b   .PipeChars

                move.l  (.ReadIOReq,PC),a1
                move.l  d3,(IO_LENGTH,a1)
                and.l   #$000000FF,d3           ; maximal 255 Zeichen
                lea     (.ReadBuf+1,PC),a0
                move.l  a0,(IO_DATA,a1)
                move.w  #CMD_READ,(IO_COMMAND,a1)
                jsr     (_LVODoIO,a6)

.PipeChars      move.l  (_DOSBase,a5),a6
                move.l  (.Output,PC),d1
                lea     (.ReadBuf,PC),a0
                move.l  a0,d2
                addq.l  #1,d3
                SYSLOG  <"Piping: Send `%s' to stdout">,a0
                jsr     (_LVOWrite,a6)
                bra     .MainLoop

.ReceivedBreak  SYSLOG  <"Piping: Got a CTRL-C!">
                move.l  ($4).w,a6
                lea     (.SerFlag,PC),a0
                tst.b   (a0)
                beq.b   .KillPktReq
                move.l  (.ReadIOReq,PC),a1
                jsr     (_LVOAbortIO,a6)
                move.l  (.ReadIOReq,PC),a1
                jsr     (_LVOWaitIO,a6)

.FreeBuffers    move.l  (.ReadBuf,PC),d0
                beq.b   .KillPktReq
                move.l  d0,a1
                jsr     (_LVOFreeVec,a6)

.KillPktReq     lea     (.ReadFlag,PC),a0
                tst.b   (a0)
                beq.b   .FreeDosObj
                move.l  (.PktPort,PC),a0
                jsr     (_LVOWaitPort,a6)
                move.l  (.PktPort,PC),a0
                jsr     (_LVOGetMsg,a6)

.FreeDosObj     move.l  #DOS_STDPKT,d1
                move.l  (.DosPkt,PC),d2
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOFreeDosObject,a6)

.RemPktMP       move.l  (.PktPort,PC),a0
                move.l  ($4).w,a6
                jsr     (_LVODeleteMsgPort,a6)

.RemWriteIOReq  move.l  (.WriteIOReq,PC),a0
                jsr     (_LVODeleteIORequest,a6)

.RemWriteMP     move.l  (.WritePort,PC),a0
                jsr     (_LVODeleteMsgPort,a6)

.RemReadIOReq   move.l  (.ReadIOReq,PC),a0
                jsr     (_LVODeleteIORequest,a6)

.ReadMP         move.l  (.ReadPort,PC),a0
                jsr     (_LVODeleteMsgPort,a6)

.exit           POPM
                rts

PipingProcC_len equ     (*-_PipingProc)

.ProcessBase    ds.l    1
.ReadPort       ds.l    1
.ReadIOReq      ds.l    1
.WritePort      ds.l    1
.WriteIOReq     ds.l    1
.PktPort        ds.l    1
.DosPkt         ds.l    1
.Input          ds.l    1
.Output         ds.l    1
.ReadBuf        ds.l    1
.PktBuf         ds.l    1
.ReadFlag       ds.b    1
.SerFlag        ds.b    1

PipingProc_len  equ     (*-_PipingProc)



_RemovePipingProcess:
                ; Signalisiert dem zweiten Task, da er die Arbeit ein-
                ; stellen soll.
                ;
                ; Parameter: keine
                ; Result   : keine

                SYSLOG  <"Welcome to RemovePipingProcess()">

                PUSHM   d0-d1/a0-a1/a6
                move.l  (_PipingTask,a5),d0
                beq     .exit
                SYSLOG  <"RemovePipingProcess: sending break to subtask">
                move.l  d0,a1
                move.l  #SIGBREAKF_CTRL_C,d0
                move.l  ($4).w,a6
                jsr     (_LVOSignal,a6)
                clr.l   (_PipingTask,a5)

                SYSLOG  <"RemovePipingProcess: delay one second">
                move.l  (_DOSBase,a5),a6
                move.l  #1*50,d1
                jsr     (_LVODelay,a6)

                SYSLOG  <"RemovePipingProcess: deallocate subtask-memory">
                move.l  (_PipingTaskAdr,a5),a1
                move.l  ($4).w,a6
                jsr     (_LVOFreeVec,a6)
                clr.l   (_PipingTaskAdr,a5)

.exit           POPM
                SYSLOG  <"You're leaving RemovePipingProcess()">
                rts


;------------------------------- Modem-Routinen ---------------------------
_OffHook:
                ; Diese Funktion dient zum Auflegem
                ;
                ; Parameter: keine
                ; Result   : D0=TRUE/FALSE

                PUSHM   d1-d2/a0-a1/a6

                cmp.l   #-1,(Wire7,a5)          ; Auf Nullmodem brauchen wir
                beq     .finished               ; nicht auflegen.

                lea     (ReturnStr,PC),a0       ; Eventuell "angebrochene"
                moveq   #1,d0                   ; Kommandos lschen, "ATA"
                bsr     _WriteSerial            ; abrechen!!
                moveq   #20,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)

                bsr     _CheckCarrier
                tst.l   d0
                beq.b   .finished

                lea     (BreakStr,PC),a0
                moveq   #3,d0
                bsr     _WriteSerial
                moveq   #50,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)

                lea     (OffHook,PC),a0
                moveq   #-1,d0
                bsr     _WriteSerial
                moveq   #100,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)

                move.l  (_InternalBuffer,a5),a0
                moveq   #-1,d0
                bsr     _ReadSerial

                bsr     _CheckCarrier
                tst.l   d0
                beq.b   .finished


; Wenn das Auflegen ber "+++ATH" nicht funktioniert hat, dann droppen wir
; einfach mal fr 2 Sekunden den DTR.

                move.l  (_SerialIOReq,a5),a1
                move.l  ($4).w,a6
                jsr     (_LVOCloseDevice,a6)

                moveq   #2*50,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)

                move.l  (_SerialIOReq,a5),a1
                move.b  #PARAMETERS,(IO_SERFLAGS,a1)
                move.l  (4+OwnDevUnit,a5),a0
                move.l  (OwnDevUnit,a5),d0
                moveq   #0,d1
                move.l  ($4).w,a6
                jsr     (_LVOOpenDevice,a6)
                tst.l   d0
                beq.b   .OpenOkay
                moveq   #0,d0
                bra.b   .leave

.OpenOkay       bsr     _ResetSerialDevice

.finished       moveq  #-1,d0
.leave          POPM
                rts


_InitModem:
                ; Initialisiert Modem und Serial-Device neu
                ;
                ; Parameter: keine
                ; Result   : D0=TRUE/FALSE

                SYSLOG  <"Welcome to ModemInit()">

                PUSHM   d1-d2/a0-a1/a6
                moveq   #-1,d2

                bsr     _ResetSerialDevice

                move.l  (Wire7,a5),d0           ; Bei Nullmodemkabel gibts
                bne     .SkipModemInit          ; kein Modem. :-)
                move.l  (_ModemInit,a5),d0
                bne     .ModemInit
                move.l  (_StdModemInit,a5),d0
                beq     .SkipModemInit

.ModemInit      move.l  d0,-(SP)
                move.l  (_DOSBase,a5),a6        ; Den Delay auch nur aus-
                move.l  (DelayBefInit,a5),d1    ; fhren, wenn auch ein
                beq     .SendCMD                ; Modemkommando anliegt.
                SYSLOG  <"ModemInit: Delay %ld/50 seconds.">,d1
                jsr     (_LVODelay,a6)
.SendCMD        move.l  (SP)+,a0
                SYSLOG  <"ModemInit: Send command '%s'.">,a0
                bsr     _SendModemCMD

                move.l  d0,d2
                bne     .SkipModemInit
                moveq   #MSG_LOG_MODEMINITFAILED,d0
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #4,SP

.SkipModemInit  move.l  d2,d0
                POPM
                SYSLOG  <"You're leaving ModemInit()">
                rts



_ResetSerialDevice:
                ; Resettet die Parameter des Serial.Device
                ;
                ; Parameter: keine
                ; Result   : keine

                pea     (a6)

                move.l  (_SerialIOReq,a5),a1
                move.w  #CMD_FLUSH,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)

                move.l  (_SerialIOReq,a5),a1
                move.w  #CMD_CLEAR,(IO_COMMAND,a1)
                jsr     (_LVODoIO,a6)

                move.l  (_SerialIOReq,a5),a1
                move.l  #SER_DEFAULT_CTLCHAR,(IO_CTLCHAR,a1)
                move.l  #BUFFERSIZE,(IO_RBUFLEN,a1)
                clr.l   (IO_EXTFLAGS,a1)                ; Serielle Parameter
                move.l  #19200,d0
                tst.l   (_StandardBaud,a5)
                beq     .SkipStandard
                move.l  (_StandardBaud,a5),d0
.SkipStandard   tst.l   (_Baud,a5)
                beq     .SkipCMDLine
                move.l  (_Baud,a5),a0
                move.l  (a0),d0
.SkipCMDLine    move.l  d0,(IO_BAUD,a1)
                move.l  #500000,(IO_BRKTIME,a1)
                clr.l   (IO_TERMARRAY,a1)
                clr.l   (IO_TERMARRAY+4,a1)
                move.w  #$0808,(IO_READLEN,a1)
                move.b  #1,(IO_STOPBITS,a1)
                move.b  #PARAMETERS,(IO_SERFLAGS,a1)
                move.w  #SDCMD_SETPARAMS,(IO_COMMAND,a1)
                jsr     (_LVODoIO,a6)

                move.l  (SP)+,a6
                rts

;------------------------------ Connect herstellen ------------------------
_MakeConnect:
                ; Stellt den Connect her
                ;
                ; Parameter: keine
                ; Result   : D0: -1 = connect
                ;                -2 = faxconnect
                ;                 0 = no carrier
                ; Kommentar: einfachere Version, die nur ein "ATA" schickt
                ;            und kein Connectpattern untersttzt.

                PUSHM   d1-d2/a1-a3/a6
                move.l  a0,a3
                move.l  (Timeout,a5),d2
                move.l  #60,(Timeout,a5)

                lea     (StdAnswerString,PC),a0
                move.l  (_AnswerCommand,a5),d0
                bne.b   .UseConfig
                move.l  (_StdAnswCommand,a5),d0
                bne.b   .UseConfig
                bra.b   .AnswerRing             ; Wenn kein Kommando ange-
.UseConfig      move.l  d0,a0                   ; geben wurde, "ATA" be-
.AnswerRing     moveq   #-1,d0                  ; nutzen.
                bsr     _WriteSerial
                moveq   #20,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)
                moveq   #-1,d0                  ; Das "RING" loeschen.
                move.l  (_InternalBuffer,a5),a0
                bsr     _ReadSerial
                lea     (ReturnStr,PC),a0
                moveq   #1,d0
                bsr     _WriteSerial
                move.l  (_InternalBuffer,a5),a0
                moveq   #1,d0
                bsr     _ReadSerial             ; Return lschen
...             lea     (ReturnStr,PC),a0
                moveq   #2,d0
                bsr     _WaitForString
                cmp.l   #2,d0                   ; Leerzeilen lschen
                beq.b   ...
                tst.l   d0
                beq.b   .Error
                cmp.l   #-1,d0
                beq.b   .Error

                clr.b   (a0,d0.l)
                move.l  a0,a2

                lea     (ConnectStr,PC),a1
                moveq   #ConnectStr_len,d0
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOCmpStrings,a6)
                tst.l   d0                      ; -1 fr DataConnect ist
                bne     .CopyConnectString      ; ja bereits in D0

                move.l  a2,a0
                lea     (FaxConnectStr,PC),a1
                moveq   #FaxConnectStr_len,d0
                jsr     (_LVOCmpStrings,a6)
                tst.l   d0                      ; 0 fr NO CARRIER ist
                beq     .exit                   ; ja bereits in D0
                moveq   #-2,d0

.CopyConnectString:
                lea     (ConnectMsg,a5),a1
...             cmp.b   #$0D,(a0)
                beq.b   .NoConnectStr
                move.b  (a0)+,(a1)+
                bra.b   ...
.NoConnectStr   clr.b   (a1)

.exit           move.l  d2,(Timeout,a5)
                POPM
                rts
.Error          moveq   #0,d0
                bra.b   .exit


_CheckCarrier:
                ; Testet ob der Carrier noch besteht
                ;
                ; Parameter: keine
                ; Result   : D0=TRUE/FALSE
                bsr     _GetSerialStatus
                btst.l  #5,d0
                beq     .Carrier
.NoCarrier      moveq   #0,d0
                rts
.Carrier        moveq   #-1,d0
                rts

_SendModemCMD:
                ; Geht in den KommandoModus und schickt ein AT-Kommando ans
                ; Modem
                ;
                ; Parameter: a0=&CMDString
                ; Result   : D0=TRUE/FALSE  (OKAY/ERROR)
                ; Kommentar: Der String mu mit Null abgeschlossen sein.
                ;            Timeout: 5 Sekunden!
                ;            Nach jedem "\r" wird ein Return gesendet und
                ;            die Reaktions des Modems abgewartet. Nur bei
                ;            "OK" wird weitergeparsed.

                PUSHM   d1-d3/a0-a2/a6
                move.l  (Timeout,a5),d3

                cmp.l   #-1,(Wire7,a5)          ; Auf Nullmodem brauchen wir
                beq     .finished               ; keine Modem-Kommandos

                move.l  #5,(Timeout,a5)
                move.l  a0,a2

                lea     (ReturnStr,PC),a0       ; Eventuell "angebrochene"
                moveq   #1,d0                   ; Kommandos lschen, "ATA"
                bsr     _WriteSerial            ; abrechen!!
                moveq   #20,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)

                bsr     _CheckCarrier
                tst.l   d0
                beq.b   .ParseLoop

                lea     (BreakStr,PC),a0
                moveq   #3,d0
                bsr     _WriteSerial
                moveq   #50,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)

.ParseLoop      cmp.b   #'\',(a2)
                beq.b   .ModemCommand
                tst.b   (a2)
                beq.b   .finished
.NoModemCommand bsr     .SendChar
                bra.b   .ParseLoop
.ModemCommand   cmp.b   #'r',(1,a2)
                bne.b   .NoModemCommand
                bsr     .SendReturn
                tst.l   d0
                beq.b   .leave
                bra.b   .ParseLoop

.finished       moveq   #-1,d0
.leave          move.l  d3,(Timeout,a5)
                POPM
                rts

.SendReturn     addq.w  #2,a2
                moveq   #10,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODelay,a6)
                move.l  (_InternalBuffer,a5),a0
                moveq   #-1,d0
                bsr     _ReadSerial
                lea     (ReturnStr,PC),a0
                moveq   #1,d0
                bsr     _WriteSerial
                move.l  (_InternalBuffer,a5),a0
                moveq   #1,d0
                bsr     _ReadSerial     ; Return lschen
...             lea     (ReturnStr,PC),a0
                moveq   #2,d0
                bsr     _WaitForString
                cmp.l   #2,d0           ; Leerzeilen berspringen
                beq.b   ...
                tst.l   d0
                beq.b   .Error
                cmp.l   #-1,d0
                bne.b   .NoTimeOut
                moveq   #0,d0
                bra.b   .Error
.NoTimeOut      lea     (-4,a0,d0.l),a0
                moveq   #0,d0
                cmp.b   #'O',(a0)+
                bne     .Error
                cmp.b   #'K',(a0)+
                bne     .Error
                moveq   #-1,d0
.Error          rts

.SendChar       moveq   #1,d0
                move.l  a2,a0
                bsr     _WriteSerial
                addq.w  #1,a2
                rts




_WaitForString:
                ; Wartet darauf, da ein bestimmter Text am Serial.Device
                ; ankommt.
                ;
                ; Parameter: a0=&SearchString d0=Len
                ; Result   : A0=&Buffer D0=ReadLen
                ; Kommentar: Eine Stringlnge von Null schliet das Null-
                ;            Byte mit ein, eine Lnge von -1 liet bis genau
                ;            auf das Nullbyte.
                ;            Ist die Readlnge gleich 0 trat ein Timeout
                ;            auf, ist sie gleich -1 wurde der Read unter-
                ;            brochen.

                PUSHM   d1-d4/a1-a4/a6

                tst.l   d0
                bne.b   .DontReadAll
                bsr     _Len
                addq.l  #1,d0
                bra.b   .LengthValid

.DontReadAll    cmp.l   #-1,d0
                bne.b   .LengthValid
                bsr     _Len
                tst.l   d0
                beq     .exit

.LengthValid    move.l  a0,a4
                move.l  d0,d4

                move.l  (_InternalBuffer,a5),a2
                moveq   #0,d2
                move.l  a4,a3
                move.l  d4,d3

.Loop           bsr     _SerialBufferLen
                tst.l   d0
                bne     .CharsAvailable
                bsr     _WaitOnSerialPort
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit
.CharsAvailable move.l  (_ReadIOReq,a5),a1
                move.w  #CMD_READ,(IO_COMMAND,a1)
                move.l  #1,(IO_LENGTH,a1)
                move.l  a2,(IO_DATA,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)

                tst.b   (ModemEcho,a5)
                beq.b   .noecho                 ; send character back
                move.l  (_WriteIOReq,a5),a1     ; if echo is turned on
                move.l  #1,(IO_LENGTH,a1)
                move.l  a2,(IO_DATA,a1)
                move.w  #CMD_WRITE,(IO_COMMAND,a1)
                jsr     (_LVODoIO,a6)
.noecho
                addq.l  #1,d2
                cmp.b   (a3)+,(a2)+
                beq.b   .RightCharacter
                move.l  a4,a3
                move.l  d4,d3
                move.b  (-1,a2),d0
                cmp.b   (a3),d0
                bne.b   .Loop
                addq.w  #1,a3

.RightCharacter subq.l  #1,d3
                beq     .finished
                bra     .Loop

.finished       move.l  d2,d0
.exit           move.l  (_InternalBuffer,a5),a0
                POPM
                rts


_GetSerialStatus:
                ; Aktualisiert die Statusfelder in der IORequest-Struktur
                ;
                ; Parameter: keine
                ; Result   : D0.w=Status
                PUSHM   d1/a0-a1/a6
                move.l  (_SerialIOReq,a5),a1
                move.w  #SDCMD_QUERY,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)
                move.l  (_SerialIOReq,a5),a1
                moveq   #0,d0
                move.w  (IO_STATUS,a1),d0
                POPM
                rts

_SerialBufferLen:
                ; Diese Routine ermittelt die Anzahl der Zeichen, die sich
                ; im seriellen ReadPuffer befinden.
                ;
                ; Parameter: keine
                ; Result   : D0=Amount of Characters
                pea     (a1)
                bsr     _GetSerialStatus
                move.l  (_SerialIOReq,a5),a1
                move.l  (IO_ACTUAL,a1),d0
                move.l  (SP)+,a1
                rts


_WaitOnSerialPort:
                ; Wartet darauf, dass am seriellen Port ein Zeichen
                ; eintrifft. Erhaelt das Programm ein CTRL/C oder ein
                ; TIMEOUT, wird das Warten abgebrochen!
                ;
                ; Parameter: keine
                ; Result   : D0=Amount of Chars
                PUSHM   d1-d2/a0-a3/a6

                move.l  (_AlarmIOReq,a5),a1             ; Timeout-Uhr in
                move.w  #TR_ADDREQUEST,(IO_COMMAND,a1)  ; Gang setzen!
                move.l  (Timeout,a5),(IO_SIZE+TV_SECS,a1)
                move.l  #5,(IO_SIZE+TV_MICRO,a1) ; Wenn ein Request mit
                move.l  ($4).w,a6                ; weniger als 5 Microsecs
                jsr     (_LVOSendIO,a6)          ; abgeschickt wird, strzt
                                                 ; die Maschine ab!!!
.Loop           move.l  #SIGBREAKF_CTRL_C,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOCheckSignal,a6)
                tst.l   d0
                bne     .Break

                tst.l   (Wire7,a5)
                bne.b   .SkipCarrierCheck
                tst.l   (CarrierUp,a5)
                beq.b   .SkipCarrierCheck
                bsr     _CheckCarrier
                tst.l   d0
                beq.b   .CarrierLost
.SkipCarrierCheck
                moveq   #12,d1
                jsr     (_LVODelay,a6)

                move.l  (_AlarmIOReq,a5),a1
                move.l  ($4).w,a6
                jsr     (_LVOCheckIO,a6)
                tst.l   d0
                bne.b   .Timeout

.ContinueLoop   bsr     _SerialBufferLen
                tst.l   d0
                beq.b   .Loop


.AbortTimerIO   move.l  d0,-(SP)
                move.l  (_AlarmIOReq,a5),a1
                move.l  ($4).w,a6
                jsr     (_LVOAbortIO,a6)
                move.l  (_AlarmIOReq,a5),a1
                jsr     (_LVOWaitIO,a6)

                move.l  (SP)+,d0
.exit           POPM
                rts
.Break          moveq   #MSG_LOG_BREAK,d0
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #4,SP
                moveq   #-1,d0
                bra     .AbortTimerIO

.Timeout
;                 moveq   #MSG_LOG_TIMEOUT,d0
;                 move.l  (Timeout,a5),-(SP)
;                 pea     (PrgName,PC)
;                 move.l  SP,a1
;                 bsr     _MakeLogEntry
;                 addq.w  #8,SP

                moveq   #0,d0
                bra     .AbortTimerIO

.CarrierLost    moveq   #MSG_LOG_CARRIERLOST,d0
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #4,SP
                moveq   #0,d0
                bra     .AbortTimerIO


;------------------------ Serielle Lese-/Schreibzugriffe ------------------
_ReadSerial:
                ; Liet Zeichen aus dem seriellen Puffer
                ;
                ; Parameter: A0=&Buffer D0=Len
                ; Result   : Anzahl der gelesenen Zeichen (0=Timeout, -1=Break)
                ; Kommentar: Wird als Lnge eine 0 bergeben, wird ein
                ;            String gelesen, bei einer -1 der ganze Puffer.
                ;            ACHTUNG: Im -1 Modus gibt es kein Timeout!
                ;            Wenn keine Zeichen anliegen gibt die Routine
                ;            eine Null zurck.

                PUSHM   d1-d2/a0-a3/a6

                cmp.l   #-1,d0
                bne     .DontReadAll
                bsr     _SerialBufferLen
                tst.l   d0
                beq     .exit

.DontReadAll    tst.l   d0
                bne     .KnownLength

                move.l  a0,a3                   ; Read until 0-Byte!!
                move.l  (_ReadIOReq,a5),a2
                moveq   #0,d2

.ReadLoop       bsr     _SerialBufferLen
                tst.l   d0
                bne     .CharsAvailable
                bsr     _WaitOnSerialPort
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit
.CharsAvailable move.l  a2,a1
                move.l  #1,(IO_LENGTH,a1)
                move.l  a3,(IO_DATA,a1)
                move.w  #CMD_READ,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)
                addq.l  #1,d2
                tst.b   (a3)+
                bne     .ReadLoop
                move.l  d2,d0
                bra     .exit

.KnownLength    move.l  (_ReadIOReq,a5),a2
                move.l  a2,a1
                move.l  d0,(IO_LENGTH,a1)
                move.l  a0,(IO_DATA,a1)
                move.w  #CMD_READ,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)
.finished       move.l  (IO_ACTUAL,a2),d0

.exit           POPM
                rts


_WriteSerial:
                ; Schreibt Zeichen in den seriellen Puffer
                ;
                ; Parameter: A0=&Buffer D0=Len
                ; Result   : keins
                ; Kommentar: Ist Len gleich -1, wird bis zum ersten 0-Byte
                ;            gesendet, ist Len gleich 0, wird das 0-Byte
                ;            mitgeschickt!

                PUSHM   d0-d1/a0-a1/a6

                tst.l   d0
                bne     .NoStringSend
                bsr     _Len
                addq.l  #1,d0
                bra     .KnownLength

.NoStringSend   cmp.l   #-1,d0
                bne     .KnownLength
                bsr     _Len

.KnownLength    move.l  (_WriteIOReq,a5),a1
                move.l  d0,(IO_LENGTH,a1)
                move.l  a0,(IO_DATA,a1)
                move.w  #CMD_WRITE,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)

.exit           POPM
                rts

;----------------------------- Logfile-Handling ---------------------------
_WriteLog:
                ; Erstellt einen Logeintrag und gibt ihn auf ein FileHandle
                ; aus.
                ;
                ; Parameter: A0=&String A1=&DataFlow D0=FileHandle
                ; Result   : keins
                ; Kommentar: Ist D0.L=0, wird das FileHandle mittels
                ;            Output() ermittelt.

                PUSHM   d0-d2/a0-a2/a6
                link    a4,#-(LEN_DATSTRING*2)
                move.l  d0,d2

                lea     (DateTimeBuf,a5),a2
                move.l  a2,d1
                move.l  (_DOSBase,a5),a6
                jsr     (_LVODateStamp,a6)
                lea     (-(LEN_DATSTRING*2),a4),a0
                lea     (-(LEN_DATSTRING*1),a4),a1
                move.b  #FORMAT_DOS,(dat_Format,a2)
                clr.b   (dat_Flags,a2)
                clr.l   (dat_StrDay,a2)
                move.l  a0,(dat_StrDate,a2)
                move.l  a1,(dat_StrTime,a2)
                move.l  a2,d1
                jsr     (_LVODateToStr,a6)
                tst.l   d0
                beq.b   .exit
                lea     (.LogString,PC),a0
                lea     (dat_StrDate,a2),a1
                move.l  d2,d0
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOPrintF,a6)
                tst.l   d0
                beq.b   .exit

                movem.l (LEN_DATSTRING*2+4,SP),saveregs
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOPrintF,a6)

.exit           unlk    a4
                POPM
                rts

.LogString      CSTR    "(%s/%s) "
                EVEN


_MakeLogEntry:
                ; Erstellt einen Logeintrag und gibt ihn ins Logfile und auf
                ; den Bildschirm aus.
                ;
                ; Parameter: A1=&DataFlow D0=StringNumber
                ; Result   : keins

                PUSHM   d0-d5/a0-a3/a6
                bsr     _GetString
                move.l  d0,a2
                move.l  a1,a3

                tst.l   (_Logfile,a5)
                beq.b   .NoFileIO

                moveq   #0,d0
                moveq   #0,d1
                lea     (LogfileLock,PC),a0
                lea     (TaskName,a5),a1
                move.l  (_OwnDevUnitBase,a5),a6
                jsr     (_LVOAttemptDevUnit,a6)
                tst.l   d0
                bne.b   .NoFileIO

                move.l  (_Logfile,a5),d1
                move.l  #MODE_READWRITE,d2
                move.l  (_DOSBase,a5),a6
                jsr     (_LVOOpen,a6)
                ERRORHANDLE d4,0,(.FreeLock,PC)

                move.l  d0,d1
                moveq   #0,d2
                move.l  #OFFSET_END,d3
                jsr     (_LVOSeek,a6)

                move.l  d4,d0
                move.l  a2,a0
                move.l  a3,a1
                bsr     _WriteLog

                move.l  d4,d1
                jsr     (_LVOClose,a6)

.FreeLock       moveq   #0,d0
                lea     (LogfileLock,PC),a0
                move.l  (_OwnDevUnitBase,a5),a6
                jsr     (_LVOFreeDevUnit,a6)

.NoFileIO       moveq   #0,d0
                move.l  a2,a0
                move.l  a3,a1
                bsr     _WriteLog

.exit           POPM
                rts


;------------------------------- Lokalisierung ----------------------------
_GetString:
                ; Holt ber die Locale.Library einen String. Sollte Locale
                ; nicht verfgbar sein, wird der interne Catalog benutzt.
                ;
                ; Parameter: D0=Stringnumber
                ; Result   : D0=&String
                ; Kommentar: Kann GetString() den String nicht finden, wird
                ;            ein Zeiger auf eine Fehlermeldung zurck-
                ;            gegeben.
                ;            Die Routine benutzt nicht die Originaltabellen,
                ;            die von CatComp erzeugt werden!!

                PUSHM   d1-d2/a0-a1/a6
                move.l  d0,d2

                tst.l   (_MessageCatalog,a5)
                beq.b   .UseInternCat
                tst.l   (_LocaleBase,a5)
                beq.b   .UseInternCat

                move.l  (_MessageCatalog,a5),a0
                sub.l   a1,a1
                move.l  (_LocaleBase,a5),a6
                jsr     (_LVOGetCatalogStr,a6)
                tst.l   d0
                bne.b   .exit

                move.l  d2,d0
.UseInternCat   lea     (CatCompArray,PC),a0
...             cmp.l   (as_ID,a0),d0
                bmi.b   .ErrorExit
                beq.b   .FoundString
                addq.w  #AppString_SIZEOF,a0
                bra.b   ...

.ErrorExit      lea     (LocaleError,PC),a0
                move.l  a0,d0
                bra.b   .exit

.FoundString    add.w   (as_Str,a0),a0
                addq.w  #4,a0
                move.l  a0,d0

.exit           POPM
                rts

;----------------------------- Fehler-Behandlung --------------------------
_ErrorHandle:
                ; Erzeut ber ErrorHandleDos() die Fehlermeldung.
                ;
                ; Parameter: A1=&DataFlow D0=Errornumber
                ; Result   : keins

                PUSHM   d0-d1/a0-a1/a6
                moveq   #0,d1
                lea     (PrgName,PC),a0
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOErrorHandleDos,a6)
                POPM
                rts

_NoPToolLibrary:
                ; Gibt einen Fehler aus, falls die PTool.Library nicht ge-
                ; ffnet werden konnte.
                ;
                ; Parameter: keine
                ; Result   : keins

                lea     (dosname,PC),a1
                move.l  ($4).W,a6
                jsr     (_LVOOldOpenLibrary,a6)
                tst.l   d0
                beq.b   .exit
                move.l  d0,a6
                jsr     (_LVOOutput,a6)
                move.l  d0,d1
                beq.b   .CloseDosLib
                lea     (NoPToolError,PC),a0
                move.l  a0,d2
                moveq   #NoPToolError_len,d3
                jsr     (_LVOWrite,a6)
.CloseDosLib    move.l  a6,a1
                move.l  ($4).W,a6
                jmp     (_LVOCloseLibrary,a6)
.exit           rts


;---------------------------- Timer.Device-Handling -----------------------
_InitTimerDevice:
                ; Initialisiert das Timer.Device, sowie zahlreiche
                ; IORequest-Strukturen.
                ;
                ; Parameter: A0=&Buffer [2*LONG]  D0=Unit
                ; Result   : D0=TRUE/FALSE

                PUSHM   d1-d2/a0-a2/a6
                move.l  d0,d2
                move.l  a0,a2

                move.l  ($4).w,a6
                jsr     (_LVOCreateMsgPort,a6)
                move.l  d0,(4,a2)
                beq.b   .exit

                move.l  d0,a0
                moveq   #IOTV_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                move.l  d0,(a2)
                bne.b   .GoOn1
                pea     (.exit,PC)
                move.l  a2,a0
                bra     _CloseTimerDevice

.GoOn1          move.l  d0,a1
                lea     (timerdev,PC),a0
                moveq   #UNIT_VBLANK,d0
                moveq   #0,d1
                jsr     (_LVOOpenDevice,a6)
                tst.l   d0
                beq.b   .DevOkay
                moveq   #0,d0
                pea     (.exit,PC)
                move.l  a2,a0
                bra     _CloseTimerDevice

.DevOkay        moveq   #-1,d0
.exit           POPM
                rts

_CloseTimerDevice:
                ; Schliet das Timer.Device, und gibt zahlreiche
                ; IORequest-Strukturen frei.
                ;
                ; Parameter: a0=&Buffer [2*LONG]
                ; Result   : keins
                ; Kommentar: Interne Funktion!

                PUSHM   d0-d1/a0-a2/a6
                move.l  a0,a2

                move.l  ($4).w,a6
                move.l  (a2),d0
                beq.b   .GoOn1
                move.l  d0,a1
                tst.l   (IO_DEVICE,a1)
                beq.b   .GoOn2
                jsr     (_LVOCloseDevice,a6)

.GoOn2          move.l  (a2),a0
                jsr     (_LVODeleteIORequest,a6)

.GoOn1          move.l  (4,a2),d0
                beq.b   .exit
                move.l  d0,a0
                jsr     (_LVODeleteMsgPort,a6)

.exit           POPM
                rts


;-------------------------------- Allgemeines -----------------------------
_FlushCopy
                ; Kopiert "lesbare" Zeichen aus einem String in einen
                ; Puffer.
                ;
                ; Parameter: A0=&Source A1=&Targetpuffer
                ; Result   : -

                SYSLOG  <"Welcome to FlushCopy()">

                PUSHM   d0/a0-a2
                lea     (FlushTab,PC),a2
                moveq   #0,d0

...             move.b  (a0)+,d0
                beq.b   .exit
                tst.b   (a2,d0.w)
                beq.b   ...
                move.b  d0,(a1)+
                bra.b   ...

.exit           clr.b   (a1)+
                POPM
                SYSLOG  <"The string without control characters is '%s'">,a1
                rts


_ConvertModemText:
                ; Konvertiert die "normalen" 0x0A in 0x0D0A, wie sie bei
                ; der DF blich sind. Dies wird eigentlich nur fr den
                ; LoginText gebraucht.
                ;
                ; Parameter: A0=Buffer D0=Length
                ; Result   : A0=TargetBuffer D0=NewLength (0: Error!)
                ; Kommentar: Verndert den InternalBuffer

                PUSHM   d1/a1
                tst.l   d0
                bne.b   .LenOkay
                bsr     _Len
                tst.l   d0
                beq.b   .exit
.LenOkay        moveq   #0,d1
                move.l  (_InternalBuffer,a5),a1

                cmp.b   #$0A,(a0)               ; Sonderfall: Erste Zeichen
                beq.b   .SkipPreCmp             ; im Puffer ein $0A!!

.Loop           cmp.b   #$0A,(a0)
                beq.b   .FoundCR
.Copy           move.b  (a0)+,(a1)+
                addq.l  #1,d1
                subq.l  #1,d0
                beq.b   .exit
                cmp.l   #BUFFERSIZE-2,d1
                bmi.b   .Loop
                moveq   #0,d1

.exit           clr.b   (a1)+
                move.l  (_InternalBuffer,a5),a0
                move.l  d1,d0
                POPM
                rts

.FoundCR        cmp.b   #$0D,(-1,a0)
                beq.b   .SkipInsert
.SkipPreCmp     cmp.b   #$0D,(1,a0)
                beq.b   .SkipInsert
                move.b  #$0D,(a1)+
                addq.l  #1,d1
.SkipInsert     bra.b   .Copy



_LongToStr:
                ; Wandelt ein Langwort in einen ASCII-String.
                ;
                ; Parameter: D0.L=Date A0=&Buffer
                ; Result   : keins
                ; Kommentar: Der Puffer mu genau 9 Byte gro sein.

                PUSHM   d0-d1/a0-a3/a6
                move.l  a0,a3
                lea     (.Convert,PC),a0
                move.l  d0,-(SP)
                move.l  SP,a1
                lea     (.CopyChar,PC),a2
                move.l  ($4).w,a6
                jsr     (_LVORawDoFmt,a6)
                addq.w  #4,SP
                POPM
                rts

.CopyChar       move.b  d0,(a3)+
                rts
.Convert        CSTR    "%lx"
                EVEN


_MakeProcessName:
                ; Erstellt den Namen des Prozesses
                ;
                ; Parameter: A0=Buffer
                ; Result   : keins

                PUSHM   a2-a3/a6
                move.l  a0,a3
                lea     (.ProzessName,PC),a0
                movem.l (OwnDevUnit,a5),d0-d1
                exg.l   d0,d1
                movem.l d0-d1,-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                lea     (.CopyChar,PC),a2
                move.l  ($4).w,a6
                jsr     (_LVORawDoFmt,a6)
                add.w   #4,SP
                add.w   #8,SP
                POPM
                rts

.CopyChar       move.b  d0,(a3)+
                rts
.ProzessName    CSTR    "%s_%s_%lu"
                EVEN


_Len:
                ; Bestimmt die Lnge eines mit Null abgeschlossenen Textes
                ;
                ; Parameter: A0=&Textpuffer
                ; Result   : D0=Lnge ohne das Nullbytes
                ; Kommentar: Verndert keine Register ausser D0!!!

                pea     (a0)            ; Register retten
.loop           tst.b   (a0)+
                bne.b   .loop           ; aktuelles Zeichen gleich Null?
                move.l  a0,d0
                move.l  (SP)+,a0
                sub.l   a0,d0           ; Lnge zurckgeben
                subq.l  #1,d0
                rts

;---------------------------------- Parsing -------------------------------
_ParseLine:
                ; Parsed eine einzelne Configzeile aus dem Passwdfile
                ;
                ; Parameter: A0=&Name A1=&ConfigBuffer A2=&Array [7 Long]
                ; Result   : D0=TRUE/FALSE
                ; Kommentar: Die einzelnen Teile mssen durch "," abge-
                ;            trennt sein!

                PUSHM   d1-d2/a0-a2

                moveq   #7-1,d0                 ; Array vorher lschen!
...             clr.l   (a2)+
                dbra    d0,...
                movem.l (SP),saveregs

                bsr     _FindNameInConfig
                tst.l   d0
                beq.b   .exit

                move.l  d0,a0
                lea     (ParseLineBuffer,a5),a1
                move.l  a1,(a2)+
                moveq   #6-1,d2
                moveq   #0,d0
                bra.b   .FirstRun

.SplitLoop      move.b  (a0)+,(a1)+
.FirstRun       tst.b   (a0)
                beq.b   .exit
                cmp.b   #$0A,(a0)
                beq.b   .exit
                cmp.b   #',',(a0)
                bne.b   .SplitLoop
                addq.w  #1,a0
                clr.b   (a1)+
                move.l  a1,(a2)+
                dbra    d2,.FirstRun

...             tst.b   (a0)
                beq.b   .finished
                cmp.b   #$0A,(a0)
                beq.b   .finished
                move.b  (a0)+,(a1)+
                bra.b   ...

.finished       clr.b   (a1)
                moveq   #-1,d0

.exit           POPM
                rts


_FindNameInConfig:
                ; Sucht den einem System zugehrigen Eintrag
                ;
                ; Parameter: A0=&Name A1=&ConfigBuffer
                ; Result   : D0=&Line / NULL

                PUSHM   d1-d2/a0-a1/a6
                moveq   #0,d2
                bsr     _Len
                move.l  d0,d1
.FindNameLoop   move.l  d1,d0
                move.l  (_PToolBase,a5),a6
                jsr     (_LVOCmpStrings,a6)
                tst.l   d0
                bne.b   .FoundName
.NextLineLoop   tst.b   (a1)
                beq.b   .exit
                cmp.b   #$0A,(a1)+
                bne.b   .NextLineLoop
                bra.b   .FindNameLoop
.FoundName      cmp.b   #',',(a1,d1.l)
                bne.b   .NextLineLoop
                move.l  a1,d2
.exit           move.l  d2,d0
                POPM
                rts


;***************************************************************************
;*                                                                         *
;* SEKTION: Daten                                                          *
;*                                                                         *
;***************************************************************************

;-------------------------------------- Libraries ---------
ptoollibname    CSTR    "ptool.library"
dosname         CSTR    "dos.library"
owndevname      CSTR    "OwnDevUnit.library"
serialdev       CSTR    "serial.device"
timerdev        CSTR    "timer.device"
CatalogName     CSTR    "AGetty.catalog"

                CLRFO
_PToolBase      FO.l    1
_DOSBase        FO.l    1
_OwnDevUnitBase FO.l    1
_LocaleBase     FO.l    1
_MessageCatalog FO.l    1
_RememberKey    FO.l    1
_PipingTask     FO.l    1
_PipingTaskAdr  FO.l    1
OwnDevUnit      FO.l    2
MySignalBit     FO.l    1
TaskName        FO.l   50
_OldTaskName    FO.l    1
_SerialMP       FO.l    1
_ReadMP         FO.l    1
_WriteMP        FO.l    1
_SerialIOReq    FO.l    1
_ReadIOReq      FO.l    1
_WriteIOReq     FO.l    1
_InternalBuffer FO.l    1
_AlarmIOReq     FO.l    2
_PasswdBuffer   FO.l    1
DateTimeBuf     FO.b    dat_SIZEOF
ParseLineBuffer FO.b    256
ConfigEntryBuf  FO.b    cl_SizeOf
ConnectMsg      FO.b    50
                ALIGNFO 0,2
OldPri          FO.l    1

;-------------------------------------- Kommandozeile -----
RDArgsTemplate  CSTR    "DEVICE,UNIT/N,BAUD/N,-MI=MODEMINIT,-AC=ANSWERCOMMAND,-LIT=LOGINTEXT,PRI/N,-A=ANSWER/S,-S=SINGLERUN/S,-H=HARDWIRE/S,QUIT/S"

_RDArgsStruct   FO.l    1

Quit            FO.l    1
Wire7           FO.l    1
SingleRun       FO.l    1
Answer          FO.l    1
_Pri            FO.l    1
_LoginText      FO.l    1
_AnswerCommand  FO.l    1
_ModemInit      FO.l    1
_Baud           FO.l    1
_Unit           FO.l    1               ; Array fr RDArgs() verkertherum
_Device         FO.l    1               ; aufbauen

;-------------------------------------- AGetty-Config -----
ConfigFile      CSTR    "UULIB:AGetty.config"
Logfile_KW      CSTR    "LOGFILE"
MyPri_KW        CSTR    "MYPRI"
StandardDev_KW  CSTR    "STANDARDDEVICE"
StandardBaud_KW CSTR    "STANDARDBAUD"
Bruteforce_KW   CSTR    "BRUTEFORCE"
ModemInit_KW    CSTR    "MODEMINIT"
LoginText_KW    CSTR    "LOGINTEXT"
PasswdFile_KW   CSTR    "PASSWD"
ImmediateAns_KW CSTR    "IMMEDIATEANSWER"
User_KW         CSTR    "USER"
Password_KW     CSTR    "PASSWORD"
UserID_KW       CSTR    "USERID"
GroupID_KW      CSTR    "GROUPID"
FingerInfo_KW   CSTR    "FINGER-INFO"
HomeDir_KW      CSTR    "HOME-DIR"
CommandToRun_KW CSTR    "COMMAND-TO-RUN"
DelayBefInit_KW CSTR    "DELAYBEFOREINIT"
AnswCommand_KW  CSTR    "ANSWERCOMMAND"
DeviceName_KW   CSTR    "DEVICENAME"
DeviceUnit_KW   CSTR    "DEVICEUNIT"
ConnectMsg_KW   CSTR    "CONNECT"
BaudRate_KW     CSTR    "BAUD"
LogConnect_KW   CSTR    "LOGCONNECT"

                CNOP    0,4
ConfigTagList   dc.l    IC_String,Logfile_KW
                dc.l    IC_Number,MyPri_KW
                dc.l    IC_String,StandardDev_KW
                dc.l    IC_Number,StandardBaud_KW
                dc.l    IC_YesNo,Bruteforce_KW
                dc.l    IC_String,ModemInit_KW
                dc.l    IC_String,LoginText_KW
                dc.l    IC_String,PasswdFile_KW
                dc.l    IC_YesNo,ImmediateAns_KW
                dc.l    IC_String,User_KW
                dc.l    IC_String,Password_KW
                dc.l    IC_String,UserID_KW
                dc.l    IC_String,GroupID_KW
                dc.l    IC_String,FingerInfo_KW
                dc.l    IC_String,HomeDir_KW
                dc.l    IC_String,CommandToRun_KW
                dc.l    IC_String,AnswCommand_KW
                dc.l    IC_String,DeviceName_KW
                dc.l    IC_String,DeviceUnit_KW
                dc.l    IC_String,ConnectMsg_KW
                dc.l    IC_String,BaudRate_KW
                dc.l    IC_Number,DelayBefInit_KW
                dc.l    IC_YesNo,LogConnect_KW
                dc.l    TAG_DONE

LogConnect      FO.l    1
DelayBefInit    FO.l    1
_BaudRate       FO.l    1
_ConnectMsg     FO.l    1
_DeviceUnit     FO.l    1
_DeviceName     FO.l    1
_StdAnswCommand FO.l    1
_CommandToRun   FO.l    1
_HomeDir        FO.l    1
_FingerInfo     FO.l    1
_GroupID        FO.l    1
_UserID         FO.l    1
_Password       FO.l    1
_User           FO.l    1
ImmediateAnswer FO.l    1
_PasswdFile     FO.l    1
_StdLoginText   FO.l    1
_StdModemInit   FO.l    1
Bruteforce      FO.l    1
_StandardBaud   FO.l    1
_StandardDev    FO.l    1               ; Array fr InterpretConfigFile()
MyPri           FO.l    1               ; verkehrtherum anlegen
_Logfile        FO.l    1

;-------------------------------------- Fehlermeldungen ---
                        dc.b    "$VER: "
        IFNE DEBUG
NoPToolError            dc.b    "AssemblerGetty_debug v"
        ELSE
NoPToolError            dc.b    "AssemblerGetty v"
        ENDC
                        REVISION
                        dc.b    " ("
                        REVDATE
                        dc.b    ")"
                        dc.b    0
                        dc.b    " requires at least PTool.Library v2.xx!!",$0A,$0A
NoPToolError_len        equ     (*-NoPToolError)

        IFNE DEBUG
PrgName         CSTR    "AGetty_debug"
        ELSE
PrgName         CSTR    "AGetty"
        ENDC
ReadConfigError CSTR    "UULIB:AGetty.Config!"
AnyMemString    CSTR    "RAM"

;-------------------------------------- Modem -------------
LogInPrompt     dc.b    $0D,$0A,"login: "
LogInPrompt_len equ     (*-LogInPrompt)
PasswordPrompt  dc.b    $0D,$0A,"password: "
PasswordPrompt_len equ  (*-PasswordPrompt)
BreakStr        dc.b    "+++"
ReturnStr       dc.b    $0D,$0A
OkayString      dc.b    "OK"
ConnectStr      dc.b    "CONNECT"
ConnectStr_len  equ     (*-ConnectStr)
FaxConnectStr   dc.b    "+FCON"
FaxConnectStr_len equ   (*-FaxConnectStr)
StdAnswerString CSTR    "ATA"
RingStr         CSTR    "RING"
OffHook         CSTR    "ATH",$0D
LoginFailedMsg  CSTR    $0D,$0A,$0A,"Sorry, this attempt was incorrect! Please try again...",$0D,$0A

RingingBuffer   FO.b    16
ModemEcho       FO.b    1
                ALIGNFO 0,2
CarrierUp       FO.l    1
Timeout         FO.l    1
StackSIZE       FOVAL

;-------------------------------------- Diverses ----------
FaxName         CSTR    "fax"
DefaultDir      CSTR    "UUCP:"
LogfileLock     CSTR    "LOG-UPDATE.LOCK"
PipeName        CSTR    "PIPE:"
CallTemplate    CSTR    "%s -Getty -DEVICE %s -UNIT %lu"
ShellTemplate   CSTR    "%s <%s >%s"


FlushTab        dc.b    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                dc.b    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                dc.b    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
                dc.b    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
                dc.b    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
                dc.b    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
                dc.b    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
                dc.b    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0
                dc.b    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                dc.b    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                dc.b    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                dc.b    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
                dc.b    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
                dc.b    1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1
                dc.b    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
                dc.b    1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1

;-------------------------------------- Locale ------------
                INCLUDE "A:Sources/AssemblerGetty/AGetty_Locale.i"
                IFND MODIFIED_LOCALE
                FAIL
                ENDC

LocaleError     CSTR    "Something went wrong with AGetties localisation! Please report!!",$0A

                END

