/*
 * Simple line printer daemon
 *
 * ATTENTION: NO SECURITY FEATURES IMPLEMENTED
 *
 * (C) 1994 by Juergen Schubert
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <clib/netlib_protos.h>
#include <sys/syslog.h>
#include <proto/dos.h>


/*********** Macros, Defines ******************************/
// #define DEBUG        /* Include debugging info? */

#define ack()   write(sock, sp, 1);

#define BUFSIZE 512
#define IOSIZE  8*1024  /* has proven to be the best value */
#define true 1
#define false 0

/*********** global Variables *****************************/
const char *ver = "$VER: LPD 0.3 (14.11.94)";
char *sp = "";


int main(int argc, char **argv)
{
        char *buffer, device[BUFSIZE], from[BUFSIZE], *cp;
        long size, i;
        int log = false, outhandle;
        unsigned int amt, j;

        int sock = init_inet_daemon();

        if (!(buffer = malloc(IOSIZE)))
                return 10;

        if (argc > 1)
                log = true;

        openlog("lpd", LOG_PID, LOG_LPR);

        ack();

/******************* Read output device name ***********************/
        cp = buffer;
        do {
                if ((size = read(sock, cp, 1)) != 1) {
                        if (size < 0)
                                return 10;
                }
        } while (*cp++ != '\n');
        *--cp = '\0';
        cp = buffer;

        ack();

        if (cp[0] != '\2') {
                syslog(LOG_ERR, "Request not supported : %d", cp[0]);
                write(sock, &cp[0], 1);
                return 5;
        }

        strcpy(device, &cp[1]);

        if (!strcmp(device, "lpr"))     /* Remote host tries to print to default printer */
                strcpy(device, "par:"); /* Default printing to par: */

        if (log)
                syslog(LOG_INFO, "Remote host tries to print to %s", device);

/******************* Read size and sending host  **********************/
        cp = buffer;
        do {
                if ((size = read(sock, cp, 1)) != 1) {
                        if (size < 0)
                                return 10;
                }
        } while (*cp++ != '\n');
        *--cp = '\0';
        cp = buffer;

        ack();

        switch (*cp++) {        /*\1 = Cleanup not possible */
        case '\2':              /* read cf file */
        case '\3':              /* read df file */
                size = 0;
                while (*cp >= '0' && *cp <= '9')
                        size = size * 10 + (*cp++ - '0');
                if (*cp++ != ' ')
                        break;

                strcpy(from, cp + 6);
                break;

        default:
                syslog(LOG_ERR, "Cleanup not supported");
                return 5;
        }

        if (log)
                syslog(LOG_INFO, "Host %s is printing %ld bytes.", from, size);

/****************** Get the job ************************************/

        outhandle = open(device, O_APPEND | O_CREAT | O_WRONLY, S_IREAD | S_IWRITE);
        if (!outhandle) {
                write(sock, &cp[0], 1);
                syslog(LOG_ERR, "Can't get an output handle for %s", device);
                return 10;
        }

        i = 0;
        while (i < size) {
                if (i + IOSIZE > size)  /* Determine number of byte that are left to read. */
                        amt = size - i;
                else
                        amt = IOSIZE;

//                Delay(10);
                j = read(sock, buffer, amt);
#ifdef DEBUG
                syslog(LOG_INFO, "Read %u byte from remote host", j);
#endif
                if (j >= 0) {
                        write(outhandle, buffer, j);
                        i += j;
#ifdef DEBUG
                        syslog(LOG_INFO, "Wrote %u byte to printer", j);
#endif
                }
                else {
                        syslog(LOG_ERR, "Lost connection");
                        break;
                }
        }

        ack();
        ack();
        close(outhandle);

        if (log)
                if (i = size)
                        syslog(LOG_INFO, "Printing successfully finished");
                else
                        syslog(LOG_ERR, "Printing unsuccessfully finished");

        close(sock);
        exit(0);
}
