/*
 *      $Filename: LharcUUCP.c $
 *      $Revision: 1.15 $
 *      $Date: 1995/02/21 05:45:01 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: LharcUUCP.c,v 1.15 1995/02/21 05:45:01 simons Exp simons $
 *
 */


/************************************* includes ***********/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifndef SUNOS4
#       include <dos.h>
#else
#       include <sys/param.h>
#       include <unistd.h>
#endif
#ifdef AMIGA
#       include <proto/dos.h>
#       include <libraries/netsupport.h>
#       include <proto/netsupport.h>
#endif

#include "protos.h"
#include "LharcUUCP_rev.h"

/************************************* defines ************/
#define BUFFERSIZE 256
#define LHARC "LhA m"           /* archiver command, moving enabled */
#define LHARCPOSTFIX ".lha"
#define LHARCUUCPMANAGER "postmaster"
#define LHARCUUCPDAEMON "lharcuucp-daemon"
#ifdef AMIGA
#define JOKER "#?"
#define PREVIOUSDIR "/"
#elif defined(SUNOS4)
#define JOKER ""  /* This is an internal trick. The routines scanning
                   * directories don't handle real pattern matching
                   * but only the case: "data<joker>". Thus, we can
                   * do it with a simple strncmp() call. Hence we don't
                   * a joker under UNIX and don't need to patch it out
                   * later. Please pay attention to this fact when
                   * changing the code.
                   */
#define PREVIOUSDIR ".."
#endif

#define PRGNAME "LharcUUCP"
#define USAGE VERS " -- written by Peter Simons <simons@peti.GUN.de>\n" \
              "Usage: LharcUUCP system [[system] [...]]\n"

/************************************* global variables ***/
static const char __RCSId[] = "$Id: LharcUUCP.c,v 1.15 1995/02/21 05:45:01 simons Exp simons $";
#ifdef AMIGA
static const char __DOSVer[] = VERSTAG;
char *homesystem, *spooldir;
#elif defined(SUNOS4)
char homesystem[MAXHOSTNAMELEN], spooldir[] = "/usr/spool/uucp/";
#endif

int break_detected = 0;

/************************************* program ************/
int main(int argc, char **argv)
{
        char *currentdir;

        if (argc < 2) {
                fprintf(stderr, USAGE);
                return 10;
        }

        if (atexit(FreeSystem)) {
                fprintf(stderr, "%s: Couldn't install my atexit() code!\n", PRGNAME);
                return 20;
        }

        if (!InitSystem()) {
                fprintf(stderr, "%s: Failed initializing system resources!\n", PRGNAME);
                return 20;
        }

#ifdef AMIGA
        if (!(homesystem = GetConfig(NULL, "HostName", NULL, NULL)))
                if (!(homesystem = GetConfig(NULL, "NodeName", NULL, NULL))) {
                        fprintf(stderr, "%s: Can't determine the name of this machine!\n", PRGNAME);
                        return 10;
                }
        homesystem = strdup(homesystem);
        spooldir = strdup(GetConfig(NULL, UUSPOOL, NULL, "UUSPOOL:"));
#elif defined(SUNOS4)
        if (gethostname(homesystem, MAXHOSTNAMELEN) == -1) {
                fprintf(stderr, "%s: Can't determine the name of this machine!\n", PRGNAME);
                return 10;
        }
#endif
        if (!(currentdir = getcwd(NULL, 4096))) {
                /* getcwd() is broken in the SAS/C implementation and won't
                 * allocate the buffer if the specified maximum size is smaller
                 * than the string length. This may no appear in other
                 * compilers, it shouldn't hurt either, though.
                 */
                fprintf(stderr, "%s: Can't determine the name of the current directory!\n", PRGNAME);
                return 10;
        }

        while (--argc && !break_detected)
                PackSpoolJobs(argv[argc]);

        chdir(currentdir);
        if (break_detected)
                fprintf(stderr, "*** Break has been detected!\n");

        return 0;
}


/************************************* subroutines ********/
void PostArchive(char *systemname, char *postfix, char grade)
{
        FILE *cfh, *xfh;
        char cfilename[64], xfilename[64], *trunc_sysname;
        char dremote[64];

        trunc_sysname = TruncName(systemname);

        sprintf(cfilename, "C.%s%c0000", trunc_sysname, grade);
        if (!(cfh = fopen(cfilename, "r"))) {
                /*
                 * When the fopen() call succedes, does the file already
                 * exist and we're finished.
                 */

                sprintf(xfilename, "D.%s%c%s.X", trunc_sysname, grade, SeqToName((unsigned int) GetSeq(1)));
                sprintf(dremote, "%s%s", TruncName(homesystem), SeqToName((unsigned int) GetSeq(1)));

                if (cfh = fopen(cfilename, "w")) {
                        fprintf(cfh, "S D.%s%s D.%s " LHARCUUCPDAEMON " - D.%s%s 0666\n",
                                systemname, postfix, dremote, systemname, postfix);
                        fprintf(cfh, "S %s X.%s%s " LHARCUUCPDAEMON " - %s 0666\n",
                                xfilename, TruncName(homesystem), SeqToName((unsigned int) GetSeq(1)), xfilename);
                        fclose(cfh);
                }

                LockFile(xfilename);
                if (xfh = fopen(xfilename, "w")) {
                        fprintf(xfh, "U " LHARCUUCPMANAGER " %s\nF D.%s\nC rlharc\n", homesystem, dremote);
                        fclose(xfh);
                }
                else
                        remove(cfilename);
                UnLockFile(xfilename);
        }
        else
                fclose(cfh);
}


void PackSpoolJobs(char *systemname)
{
#ifdef TAYLOR
        struct ScanDir *sd2;
        char *gradedir;
#endif
        struct ScanDir *sd;
        char currentdir[BUFFERSIZE], *cfile, *proclock;
        char owngrade, grade;

        if (strlen(systemname)+strlen(spooldir)+1 > BUFFERSIZE) {
                fprintf(stderr, "%s: Directoryname is too long for internal buffer.\n", PRGNAME);
                return;         /* Buffer overflow! */
        }


        /*
         * Make sure we're the only LharcUUCP for this system.
         */

        if (!(proclock = malloc(strlen("LharcUUCP") + strlen(systemname) + 5)))
                return;
        sprintf(proclock, "%s_%s", "LharcUUCP", systemname);
        LockFile(proclock);
        printf("%s: Processing queued jobs for >%s<.\n", "LharcUUCP", systemname);


        /*
         * Change into the appropiate directory, holding the spool jobs
         * for the specified system.
         */

        sprintf(currentdir, "%s%s", spooldir, systemname);
        if (chdir(currentdir)) {
                sprintf(currentdir, "%s%s", spooldir, TruncName(systemname));
                if (chdir(currentdir)) {
                        fprintf(stderr, "%s: System >%s< has no spool directory?\n", PRGNAME, systemname);
                        strcpy(currentdir, spooldir);
                        if (chdir(currentdir)) {
                                fprintf(stderr, "%s: Cd to >%s< failed.\n", spooldir, PRGNAME);
                                perror(PRGNAME);
                                UnLockFile(proclock);
                                free(proclock);
                                return;
                        }
                }
        }


        /*
         * Determine the configured grades for our process.
         */
        {
                char *string, buf[80];

                string = GetConfig(NULL, "BatchOnlyGrade", NULL, NULL);
                grade = (string && isalnum(string[0])) ? string[0] : 'z';

                sprintf(buf, "%s_%s", "BatchOnlyGrade", systemname);
                string = GetConfig(NULL, buf, NULL, NULL);
                grade = (string && isalnum(string[0])) ? string[0] : grade;

                string = GetConfig(NULL, "LharcBatchGrade", NULL, NULL);
                owngrade = (string && isalnum(string[0])) ? string[0] : 'A';
        }



#ifdef TAYLOR
        /*
         * Okay, we're doing a pretty trick when compiled for
         * TAYLOR-UUCP. This while-loop embraces the inner one
         * and change the current directory to the sub-dir for
         * each grade. Furthermore, we can do the grade checking
         * already in the outer loop.
         */

        if (!(sd2 = InitDirScan(NULL))) {
                fprintf(stderr, "%s: Directory scan for >%s< failed!\n", PRGNAME, currentdir);
                perror(PRGNAME);
                return;
        }

        while ((gradedir = ScanNextFile(sd2)) && !break_detected) {
                if (*gradedir > grade)
                        continue;       /* Grade is too low for us. */
                if (chdir(gradedir))
                        continue;       /* Wasn't a valid directory. Try next one. */
#endif

                /*
                 * Build a pattern matching 'C.system*', with the systemname
                 * truncated to 7 characters and scan the spooldirectory for
                 * queued jobs we can pack.
                 */
                {
                        char pattern[64];
                        sprintf(pattern, "C.%s"JOKER, TruncName(systemname));
                        sd = InitDirScan(pattern);
                }
                if (sd)
                        while ((cfile = ScanNextFile(sd)) && !break_detected)
                                LharcJob(systemname, cfile, grade, owngrade);
                else {
                        fprintf(stderr, "%s: Directory scan for >%s< failed!\n", PRGNAME, currentdir);
                        perror(PRGNAME);
                }
                EndDirScan(sd);
#ifdef TAYLOR
                if (chdir(PREVIOUSDIR))
                        break;          /* FATAL ERROR: Can't go back to spool. */
        }

        EndDirScan(sd2);
#endif

        UnLockFile(proclock);
        free(proclock);
}


void LharcJob(char *systemname, char *cfilename, char grade, char owngrade)
{
        FILE *cfilehandle;
        char command[2], from[256], to[256], from2[256], to2[256], lharc_cfilename[256];
        char cmdbuf[256], archivename[64];
        int success = 0, size;
        unsigned int maxbatchsize;

        sprintf(lharc_cfilename, "C.%s%c0000", TruncName(systemname), owngrade);
        printf("%s: processing <%s>\n", PRGNAME, cfilename);

        if (IsFileLocked(cfilename))
                return;         /* uucico is processing this job right now,
                                 * so we better leave it alone.
                                 */
        LockFile(cfilename);

        if(!IsValidJob(cfilename, systemname, grade)) {
                UnLockFile(cfilename);
                return;
        }

        LockFile(lharc_cfilename);      /* lock the command file which transfers the archive,
                                         * to make sure nobody does any tricks while we're
                                         * at the archive.
                                         */

        sprintf(archivename, "D.%s%s", systemname, LHARCPOSTFIX);
#ifdef AMIGA    /* Does not work under UNIX currently! */
        maxbatchsize = (unsigned int) atoi(GetConfig(NULL, "MaxBatchSize", NULL, "500"));
        if ((size = GetFileSize(archivename)) >= 0L)
                size /= 1024;
        else
                size = 0L;
        if (maxbatchsize && (GetFileSize(archivename) / 1024) > maxbatchsize) {
                /*
                 * Queue the existing job via uux
                 */
                if (SetVar("USER", LHARCUUCPDAEMON, -1, GVF_LOCAL_ONLY)) {
                        sprintf(cmdbuf, "%s %s \"%s!rlharc\"", GetConfig(NULL, UUX, NULL, "uux"), archivename, systemname);
                        if (!system(cmdbuf)) {
                                LockFile(archivename);
                                DeleteFile(archivename);
                                UnLockFile(archivename);
                        }
                        DeleteVar("USER", GVF_LOCAL_ONLY);
                }
                else {
                        fprintf(stderr, "%s: Couldn't set the local USER variable.\n", PRGNAME);
                        perror(PRGNAME);
                }
        }
#endif
        if (cfilehandle = fopen(cfilename, "r")) {
                fscanf(cfilehandle, "%s%s%s", command, from, to);
                if (GoNextLine(cfilehandle) != EOF) {
                        fscanf(cfilehandle, "%s%s%s", command, from2, to2);

                        LockFile(from);
                        LockFile(to);
                        LockFile(from2);
                        LockFile(to2);
                        if (!(rename(from, to)))
                                if (!(rename(from2, to2))) {
                                        success = 1;


                                        /* Free unused files a soon as
                                         * possible.
                                         */

                                        fclose(cfilehandle);
                                        remove(cfilename);
                                        UnLockFile(from);
                                        UnLockFile(from2);
                                        UnLockFile(cfilename);


                                        /* Batch the processed files.
                                         */

                                        sprintf(cmdbuf, "%s D.%s%s %s %s", LHARC, systemname, LHARCPOSTFIX, to, to2);
                                        LockFile(archivename);
                                        system(cmdbuf);
                                        UnLockFile(archivename);
                                        PostArchive(systemname, LHARCPOSTFIX, owngrade);
                                }
                                else
                                        rename(to, from);


                        /* When one of the 'renames' failed, we still
                         * have to close and unlock the files.
                         */

                        if (!success) {
                                fprintf(stderr, "%s: Failed renaming the file for archiving!\n", PRGNAME);
                                perror(PRGNAME);
                                UnLockFile(from);
                                UnLockFile(from2);
                                fclose(cfilehandle);
                                UnLockFile(cfilename);
                        }

                        UnLockFile(to);         /* These must be freed */
                        UnLockFile(to2);        /* in any case.        */
                }
        }
        UnLockFile(lharc_cfilename);
}


 /*
  * IsValidJob() checks the provided C.* file and tests if this job may
  * may be processed by LharcUUCP. Currently, we only process mail and
  * news. UUCP- and lharc-jobs are left alone.
  */

int IsValidJob(char *cfilename, char *systemname, char grade)
{

        /*
         * Test if the grade of the commandfile is lower or the same as
         * our max. grade.
         */

        if (cfilename[strlen(TruncName(systemname)) + 2] > grade)
                return 0;


        /*
         * Check the number part of the filename. C.*0000 are the C.*
         * files for the processed archives and won't be touched.
         */

        if (!strncmp(&cfilename[strlen(cfilename) - 4], "0000", 4))
                return 0;


        /*
         * Check the contents of the file. Neither target nor source file
         * may contain paths and we can't batch read-requests of course.
         * Furthermore, we don't process jobs which have been created by
         * a user named lharcuucp-daemon.
         */

        {
                FILE *fh;
                int success = 0;
                char from[256], to[256], user[256];

                if (fh = fopen(cfilename, "r")) {
                        while (1) {
                                if (fgetc(fh) == 'S') {
                                        fscanf(fh, "%s%s%s", from, to, user);
                                        if (IsPath(from) || IsPath(to))
                                                break;
                                        if (!stricmp(user, LHARCUUCPDAEMON))
                                                break;
                                        GoNextLine(fh);
                                        if (feof(fh)) {
                                                success = 1;
                                                break;
                                        }
                                }
                                else {
                                        GoNextLine(fh);
                                        if (feof(fh)) {
                                                success = 1;
                                                break;
                                        }
                                }
                        }
                        fclose(fh);
                }
                if (!success)
                        return 0;
        }

        return 1;
}

