/*
 *      $Filename: ScanDir.c $
 *      $Revision: 1.6 $
 *      $Date: 1995/02/21 04:58:02 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: ScanDir.c,v 1.6 1995/02/21 04:58:02 simons Exp simons $
 *
 */

/************************************* includes ***********/
#include <stdlib.h>
#include <string.h>
#ifdef AMIGA
#       include <dos/dos.h>
#       include <dos/exall.h>
#       include <proto/dos.h>
#endif
#ifdef SUNOS4
#	include <sys/stat.h>
#	include <dirent.h>
#       define TRUE  1
#       define FALSE 0
#endif

/************************************* defines ************/
char **ScanDir(char *, char *);
struct ScanDir {
        char            **sd_namearray;
        unsigned int    sd_namepointer;
};

#define MAX_FILE_NUM    4096    /* The maximum number of files we can handle */

/************************************* global variables ***/
static const char __RCSId[] = "$Id: ScanDir.c,v 1.6 1995/02/21 04:58:02 simons Exp simons $";

/************************************* subroutines ********/
struct ScanDir *InitDirScan(char *pattern)
{
        struct ScanDir *sd;

        if (!(sd = malloc(sizeof(struct ScanDir))))
                return NULL;
        sd->sd_namepointer = 0;

        if (sd->sd_namearray = ScanDir(NULL, pattern))
                return sd;
        else
                return NULL;
}

char *ScanNextFile(struct ScanDir *sd)
{
        char *name = sd->sd_namearray[sd->sd_namepointer++];
                return (name) ? name : NULL;
}

void EndDirScan(struct ScanDir *sd)
{
        unsigned int i;

        if (!sd->sd_namearray)
                return;

        for (i = 0; sd->sd_namearray[i]; i++)
                free(sd->sd_namearray[i]);
        free(sd->sd_namearray);
}



 /*
  * ScanDir() reads all entries in the specified directory and returns those
  * matching the provided pattern. A pattern of NULL means that all files
  * are returned. If the directory-parameter is NULL, the current directory
  * will be scanned.
  */

#ifdef AMIGA

char **ScanDir(char *directory, char *pattern)
{
        struct ExAllControl *eac;
        struct ExAllData *ead;
        BPTR dir_lock;
        char **filenames = NULL, *parsed_pattern = NULL, *buffer;
        unsigned int i = 0, cont;

        if (!(buffer = malloc(10 * 1024)))
                return NULL;

        if (!(filenames = malloc(sizeof(char *) * 4096))) {
                free(buffer);
                return NULL;
        }

        if (pattern)
                if (parsed_pattern = malloc(strlen(pattern) + 256)) {
                        if (ParsePatternNoCase(pattern, parsed_pattern, (long) strlen(pattern) + 256) == -1L) {
                                free(buffer);
                                free(filenames);
                                free(parsed_pattern);
                                return NULL;
                        }
                }
                else
                        return NULL;

        if (dir_lock = Lock(directory ? directory : "", ACCESS_READ)) {
                if (eac = AllocDosObject(DOS_EXALLCONTROL, NULL)) {
                        if (parsed_pattern)
                                eac->eac_MatchString = parsed_pattern;

                        do {
                                cont = ExAll(dir_lock, (struct ExAllData *) buffer, 10 * 1024, ED_NAME, eac);
                                ead = (struct ExAllData *) buffer;
                                if (eac->eac_Entries > 0) {
                                        do {
                                                filenames[i++] = strdup(ead->ed_Name);
                                                ead = ead->ed_Next;
                                        } while (ead);
                                }
                        } while (cont);
                        filenames[i] = NULL;

                        FreeDosObject(DOS_EXALLCONTROL, eac);
                }
                UnLock(dir_lock);
        }

        if (parsed_pattern)
                free(parsed_pattern);
        free(buffer);
        if (*filenames)
                return filenames;
        else {
                free(filenames);
                return NULL;
        }
}

#elif defined(SUNOS4)

char **ScanDir(char *directory, char *pattern)
{
        DIR *dfd;
        struct dirent *dptr;
        char **filenames = NULL;
        unsigned int counter = 0;

        if (dfd = opendir(directory ? directory : "")) {
                if (!(filenames = malloc(sizeof(char *) * MAX_FILE_NUM))) {
                        closedir(dfd);
                        return NULL;
                }
                filenames[counter] = NULL;
                while (dptr = readdir(dfd)) {
                        if (!pattern || !strncmp(pattern, dptr->d_name, strlen(pattern))) {
                                filenames[counter++] = strdup(dptr->d_name);
                                if (counter >= MAX_FILE_NUM)
                                        break;
                                }
                }
                filenames[counter] = NULL;
        }

        return filenames;
}

#endif
