/*
 *      $Filename: SystemDep.c $
 *      $Revision: 1.8 $
 *      $Date: 1995/02/21 05:45:01 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: SystemDep.c,v 1.8 1995/02/21 05:45:01 simons Exp simons $
 *
 */


/************************************* includes ***********/
#include <stdio.h>
#ifdef AMIGA
#       include <exec/exec.h>
#       include <dos/dos.h>
#       include <dos/dostags.h>
#       include <libraries/netsupport.h>
#       include <proto/exec.h>
#       include <proto/dos.h>
#       include <proto/netsupport.h>
#endif

#include "protos.h"

/************************************* global variables ***/
static const char __RCSId[] = "$Id: SystemDep.c,v 1.8 1995/02/21 05:45:01 simons Exp simons $";

#ifdef AMIGA
struct NetSupportLibrary *NetSupportBase = NULL;
#endif

/************************************* routines ***********/

 /*
  * Initialize and open all required system resources we'll
  * need for our program.
  */

int InitSystem(void)
{
#ifdef AMIGA
        if (!(NetSupportBase = (struct NetSupportLibrary *) OpenLibrary(NETSUPPORTNAME, 0L))) {
                fprintf(stderr, "FATAL ERROR: Failed opening %s!\n", NETSUPPORTNAME);
                return 0;
        }

        if (!(DOSBase = (struct DosLibrary *) OpenLibrary(DOSNAME, 0L))) {
                fprintf(stderr, "FATAL ERROR: Failed opening %s!\n", DOSNAME);
                return 0;
        }
#endif
        return 1;
}


 /* Free all previously allocated system resources. */

void FreeSystem(void)
{
#ifdef AMIGA
        if (NetSupportBase) {
                UnLockFiles();
                CloseLibrary((struct Library *) NetSupportBase);
        }

        if (DOSBase)
                CloseLibrary((struct Library *) DOSBase);
#endif
}


 /*
  * GetFileSize() returns the file lenth in bytes.
  * If an error occurs, -1L is returned.
  */

long GetFileSize(char *filename)
{
#ifdef AMIGA
        BPTR fh;
        struct FileInfoBlock __aligned fib;
        LONG filesize = -1L;

        if (fh = Lock(filename, ACCESS_READ)) {
                if (Examine(fh, &fib))
                        filesize = fib.fib_Size;
                UnLock(fh);
        }

        return filesize;

#elif defined(SUNOS4)
        return -1L;
#endif
}


#ifdef AMIGA

 /*
  * My Amiga C compiler allows to simply replace the routine that is
  * called whenever a CTRL-C interrupt is detected. You UNIX folks
  * have to install this trap handler using signal() in InitSystem().
  * Sorry. :-))
  */

void __regargs _CXBRK(void)
{
        break_detected = 1;
}


 /*
  * Use this Amiga specific routine to execute the command
  * rather than the ANSI system(). This routine will
  * surpress all output of the called program.
  */

int system(const char *command)
{
        return SystemTags(command, NP_ConsoleTask, NULL,
                                   SYS_Input, NULL,
                                   SYS_Output, NULL,
                                   TAG_DONE);
}


 /* Replace the UNIX function with an AmigaOS version. */

void perror(const char *prgname)
{
        extern int _OSERR;
        PrintFault((long) _OSERR, prgname);
}

#endif


#if defined(SUNOS4)
 /*
  * These are just dummy routines for testing purposes.
  * I will have to write them one day, though. :-)
  */
char *GetConfig(char *a, char *b, char *c, char *def)
{
        return def;
}

long LockFile(char *f)
{
        return 1;
}
long UnLockFile(char *f)
{
        return 1;
}
long UnLockFiles(void)
{
        return 1;
}
long LockFileAttempt(char *f)
{
        return 1;
}
long IsFileLocked(char *f)
{
        return 0;
}

unsigned long GetSeq(unsigned long a)
{
        return 0;
}
#endif
