/*
 *      $Filename: ListMail.c $
 *      $Revision: 3.4 $
 *      $Date: 1995/04/22 23:09:30 $
 *
 *      Copyright (C) 1993 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: ListMail.c,v 3.4 1995/04/22 23:09:30 simons Exp simons $
 */


/************************************* includes ***********/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <libraries/netsupport.h>
#include <exec/memory.h>
#include <dos/dosextens.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/netsupport.h>

#include "ListMail_rev.h"

/************************************* defines ************/
#define USAGE "ListMail -- written by Peter Simons <simons@peti.GUN.de>\n" \
              "Usage: ListMail <mailfile address_list_file\n"

#define STDBUFSIZE 1024

/************************************* prototypes *********/
LONG MySystem(STRPTR name, STRPTR args, STRPTR infile, STRPTR outfile, ULONG stack);
void FreeSystem(void);
char *LoadFile(char *);
int strbpl(char **pointers, int maxfiles, const char *names);
char *stristr(char *, char *);
void CheckDateString(STRPTR);
void ParseAddress(char *);

/************************************* global variables ***/
static const char __RCSId[] = "$Id: ListMail.c,v 3.4 1995/04/22 23:09:30 simons Exp simons $";
static const char __DOSVer[] = VERSTAG;
struct NetSupportLibrary *NetSupportBase;
char tempfile[32];

/************************************* program ************/

int main(int argc, char **argv)
{
        BPTR fh, input = Input();
        char *buffer, *mailbuf, *cmdbuf, *receipt_from, *receipt_id;
        char *receipt_subject, *receipt_to;
        LONG len, i, z, rc = 0, header = 1;
        struct DateTime dt;
        TEXT StrDate[LEN_DATSTRING], StrTime[LEN_DATSTRING];

        if (argc != 2 || !stricmp(argv[1], "?") || !stricmp(argv[1], "-h")) {
                Printf(USAGE);
                return 0;
        }

        if (atexit(FreeSystem))
                exit(20);

        if (!(NetSupportBase = (struct NetSupportLibrary *) OpenLibrary(NETSUPPORTNAME, 0L))) {
                Printf("FATAL ERROR: Can't open %s!\n", NETSUPPORTNAME);
                exit(20);
        }

        TempNameT(tempfile);
        if (!(receipt_from = AllocMemPooled(STDBUFSIZE*5, MEMF_ANY|MEMF_CLEAR))) {
                Printf("FATAL ERROR: Can't allocate my internal buffers!\n");
                exit(20);
        }
        mailbuf = receipt_from+STDBUFSIZE;
        receipt_id = mailbuf+STDBUFSIZE;
        receipt_to = receipt_id+STDBUFSIZE;
        receipt_subject = receipt_to+STDBUFSIZE;

        Flush(input);
        if (fh = Open(tempfile, MODE_NEWFILE)) {
                while (FGets(input, mailbuf, STDBUFSIZE-1)) {
                        if (header) {
                                if (*mailbuf == '\n')
                                        header = 0;
                                else {
                                        if (!(strnicmp("From:", mailbuf, 5)))
                                                strcpy(receipt_from, mailbuf);
                                        else if (!(strnicmp("Message-ID:", mailbuf, 11)))
                                                strcpy(receipt_id, mailbuf);
                                        else if (!(strnicmp("Subject:", mailbuf, 8)))
                                                strcpy(receipt_subject, mailbuf);
                                        else if (!(strnicmp("To:", mailbuf, 3)))
                                                strcpy(receipt_to, mailbuf);
                                }
                        }
                        if (FPuts(fh, mailbuf) == -1L) {
                                rc = 20;
                                break;
                        }
                }
                Close(fh);
                if (rc)
                        exit(rc);
        }

        rc = 20;
        LockFile(argv[1]);
        if (buffer = LoadFile(argv[1])) {
                len = IoErr();
		if (!(cmdbuf = AllocMemPooled(len+1, MEMF_ANY)))
		  exit(rc);
                for (i=0; i < len ; i++) {
			if (buffer[i]== '\n')
			  cmdbuf[i] = ' ';
			else
			  cmdbuf[i] = buffer[i];
		}
		cmdbuf[len] = '\n'; cmdbuf[len+1] = '\0';
		/*
		 * Test wether we have to send a receipt out.
		 */
		receipt_from = receipt_from+5; /* skip "From: " */
		while(isspace(*receipt_from))
		  receipt_from++;
		ParseAddress(receipt_from);
		i=0;
		while(i < len) {
			if (!strnicmp(receipt_from, &cmdbuf[i], strlen(receipt_from))) {
				z = 0;
				while(!isspace(cmdbuf[i])) {
					receipt_from[z++] = cmdbuf[i];
					cmdbuf[i++] = ' ';
				}
				break;
			}
			else {
				while(!isspace(cmdbuf[i]))
				  i++;
				if (!(i < len))
				  break;
				while(isspace(cmdbuf[i]))
				  i++;
			}
		}
		Printf("Addresses: %s", cmdbuf);

		/*
		 * Send mail out.
		 */
		rc = MySystem(GetConfig(NULL, RMAIL, NULL, RMAIL), cmdbuf, tempfile, NULL, 4096L);
	}
	FreeMemPooled(buffer); 	FreeMemPooled(cmdbuf);
        UnLockFile(argv[1]);

        if (receipt_from && (fh = POpen(GetConfig(NULL, SENDMAIL, NULL, SENDMAIL), MODE_PIPETO))) {
                Printf("Sending receipt to <%s>.\n", receipt_from);
                FPrintf(fh, "From: listserv-manager@%s", GetConfig(NULL, NODENAME, NULL, "foo"));
                FPrintf(fh, "%s (ListSERV Manager)\n", GetConfig(NULL, DOMAINNAME, NULL, ".bar"));
                FPrintf(fh, "To: %s\n", receipt_from);
                FPrintf(fh, "Subject: mailing list receipt\n");
                FPrintf(fh, "\nYour mail arrived here on %s and was delivered to the mailinglist\nat ",
                        GetConfig(NULL, NODENAME, NULL, "foo"));

                DateStamp((struct DateStamp *) &dt);
                dt.dat_Format = FORMAT_DOS;
                dt.dat_Flags = 0;
                dt.dat_StrDay = NULL;
                dt.dat_StrDate = StrDate;
                dt.dat_StrTime = StrTime;

                if (DateToStr(&dt)) {
                        CheckDateString(StrDate);       /* remove trailing blanks */
                        CheckDateString(StrTime);
                        FPrintf(fh, "%s %s %s.\n", StrDate, StrTime, GetConfig(NULL, TIMEZONE, NULL, "GMT"));
                }
                else
                        FPrintf(fh, "[error building date string]!\n");

                if (*receipt_to)
                        FPrintf(fh, "\n\n      %s", receipt_to);
                else
                        FPrintf(fh, "\n\n      To: <not available>\n");
                if (*receipt_subject)
                        FPrintf(fh, "      %s", receipt_subject);
                else
                        FPrintf(fh, "      Subject: <not available>\n");
                if (*receipt_id)
                        FPrintf(fh, "      %s", receipt_id);
                else
                        FPrintf(fh, "      Message-Id: <not available>\n");

                PClose(fh);
        }
        else
                Printf("No receipt was sent!\n");

        return rc;
}



 /*
  * LoadFile() -- determines the length of a given file, allocates the
  * appropiate buffer and actually loads the file. If everything works,
  * the address of the buffer is returned and the length of the file is
  * available via IoErr().
  *
  * ATTENTION: Two zero-byte are appended to the loaded file, to enable the
  * usage of standard C string routines on the contents and to make my
  * strbpl() work.
  *
  * The buffer should be freed after usage using FreeMemPooled().
  */

char *LoadFile(char *filename)
{
        BPTR fh, fh2;
        struct FileInfoBlock *fib;
        char *memblock, *filebuffer = NULL;
        BOOL isfile = FALSE;
        int readlen;

        if (fh = Lock(filename, ACCESS_READ)) {
                if (fib = AllocMemPooled(sizeof(struct FileInfoBlock), 0L)) {
                        if (Examine(fh, fib)) {
                                if (fh2 = OpenFromLock(fh)) {
                                        isfile = TRUE;
                                        if (memblock = AllocMemPooled(fib->fib_Size + 2, 0L)) {
                                                if ((readlen = Read(fh2, memblock, fib->fib_Size)) != -1L) {
                                                        memblock[readlen] = '\0';
							memblock[readlen+1] = '\0';
                                                        filebuffer = memblock;
                                                        SetIoErr(readlen);
                                                }
                                                else
                                                        FreeMemPooled(memblock);
                                        }
                                        Close(fh2);
                                }
                        }
                        FreeMemPooled(fib);
                }
                if (!(isfile))
                        UnLock(fh);
        }

        return filebuffer;
}


 /*
  * Finds a provided part in a substring if available. Unlike, the
  * usual ANSI strstr(), this routine is not case sensitive.
  */

char *stristr(char *line, char *substring)
{
        int i, cmplen = strlen(substring), end = strlen(line)-cmplen;

        for (i = 0; i <= end; i++)
                if (strnicmp(&line[i], substring, cmplen) == NULL)
                        return &line[i];
        return NULL;
}


 /* This routine removes trailing spaces from the generated date
  * string. These blanks are added in order make all date strings
  * the same length. However, this is not required for our
  * application.
  */

void CheckDateString(STRPTR datestring)
{
        while(*datestring)
                datestring++;

        while(*(datestring-1) == ' ')
                *(--datestring) = '\0';
}


 /* Free all previously allocated system resources. */

void FreeSystem(void)
{
        DeleteFile(tempfile);
        UnLockFiles();
        if (NetSupportBase)
                CloseLibrary((struct Library *) NetSupportBase);
}


/*
 * LONG MySystem(STRPTR name, STRPTR args, STRPTR infile, STRPTR outfile);
 *
 * This routine calls a resident program and hands
 * the specified parameters over. It is able to
 * process more than 512 byte of commandline length.
 */

LONG MySystem(STRPTR name, STRPTR args, STRPTR infile, STRPTR outfile, ULONG stack)
{
        LONG rc = -1;
	struct Segment *segment;
	BPTR oldin, oldout, tmp = NULL, tmp2 = NULL;
	char prgname[64];

	if (!(tmp = Open((infile) ? infile : (STRPTR) "CONSOLE:", MODE_OLDFILE)))
	  return -1;
	if (!(tmp2 = Open((outfile) ? outfile : (STRPTR) "CONSOLE:", MODE_NEWFILE))) {
		Close(tmp);
		return -1;
	}
	oldin = SelectInput(tmp);
	oldout = SelectOutput(tmp2);

	GetProgramName(prgname, sizeof(prgname));
	Forbid();
        if ((segment = FindSegment(name, NULL, FALSE)) && segment->seg_UC >= 0) {
		segment->seg_UC++;
		Permit();
		SetProgramName(name);
		rc = RunCommand(segment->seg_Seg, stack, args, strlen(args));
		SetProgramName(prgname);
		segment->seg_UC--;

        }
        else
	  Permit();

	SelectInput(oldin);
	SelectOutput(oldout);
	Close(tmp);
	Close(tmp2);

        return rc;
}


void ParseAddress(char *line)
{
        int brackets, i, z;

        while(isspace(*line))
                line++;

        for (i = 0, z = 0, brackets = 0; line[i] != '\0'; i++) {
                if (line[i] == '(')
                        brackets++;
                else if (line[i] == ')')
                        brackets--;
                else if (line[i] == '<' && brackets == 0) {
                        i++;
                        while (1) {             /* Forever! */
                                if (line[i] == '>' && (line[i+1] == '\0' || isspace(line[i+1])))
                                        break;
                                line[z++] = line[i++];
                        }
                        line[z] = '\0';
                        break;
                }
        }
        if (z == 0) {   /* No <address> occured. */
                i = 0;
                while (!isspace(line[i]))
                        i++;
                line[i] = '\0';
        }
}
