/*
 *      $Filename: main.c $
 *      $Revision: 3.1 $
 *      $Date: 1994/06/22 15:02:13 $
 *
 *      Copyright (C) 1993 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *      $Id: ListSERV.c,v 3.1 1994/06/22 15:02:13 simons Exp simons $
 *
 */


/************************************* includes ***********/
#include <exec/execbase.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <dos.h>

#include "listserv.h"
#include "protos.h"
#include "ListSERV_rev.h"

/************************************* defines ************/
#define STARTUP "Amiga " VERS " -- written by by Peter Simons <simons@peti.GUN.de>\n"
#define USAGE   "USAGE: ListSERV <mailfile [default_list]\n"
#define PRGNAME argv[0]

/************************************* prototypes *********/
void CleanUp(void);

/************************************* global variables ***/
static char rcssid[] = "$Id: ListSERV.c,v 3.1 1994/06/22 15:02:13 simons Exp simons $";
static char dosid[] = VERSTAG;
struct NetSupportLibrary *NetSupportBase;
BPTR mailer;
char *def_listname, *tmpbuf;

/************************************* program ************/
int main(int argc, char **argv)
{
        extern struct ExecBase *SysBase;
        char *from = NULL, *buf, *grp;
        int gotcommand = 0, garbage = 0;

        if (SysBase->LibNode.lib_Version < 37)
                return 10;

        Printf(STARTUP);
        if (argc > 2 || (argc == 2 && (!strcmp(argv[1], "?") || !stricmp(argv[1], "-h")))) {
                Printf(USAGE);
                return 0;
        }
        if (!(NetSupportBase = (struct NetSupportLibrary *) OpenLibrary(NETSUPPORTNAME, 0L)))
                return 10;

        if (atexit(CleanUp)) {
                CleanUp();
                return 10;
        }

        /* init some variables */
        if (!(tmpbuf = AllocMemPooled(BUFSIZE*2+64, 0L))) {
                Printf("%s: Can't allocate my internal buffers!\n", PRGNAME);
                return 10;
        }
        buf = tmpbuf + BUFSIZE;
        grp = buf + BUFSIZE;

        if (argc == 2 && argv[1])
                def_listname = argv[1];


#ifdef EVALUATION
        /*
         * Check number of lists
         *
         * Only necessary in the evaluation version!
         */
        {
                DIR *listdir;
                int counter = 0;

                if ((listdir = opendir(LISTDIR)) == NULL) {
                        Printf("%s: Can't access %s!\n", PRGNAME, LISTDIR);
                        return 20;
                }
                while (readdir(listdir))
                        counter++;
                closedir(listdir);
                if (counter > 2) {      /* allow only two lists */
                        Printf("This is only an evaluation version of %s!\nPlease register if you need more than 2 mailing lists!\n", PRGNAME);
                        return 20;
                }
        }
#endif  /* EVALUATION */

        /* Parse header for "from". */
        Flush(Input());
        while (FGets(Input(), buf, BUFSIZE-1) && buf[0] != '\n') {
                /* drop trailing newline */
                buf[strlen(buf)-1] = '\0';

                /* get the From: line so we can return the answer */
                if (!strnicmp(buf, "From: ",6))
                        if (from = strdup(&buf[6]))
                                ParseAddress(from);
        }


        /* Check "from" validity! */

        if (!from) {
                MakeLogEntry("ListSERV", MLE_ERROR, "Received mail without 'From:'-line!");
                return 5;
        }
        if (IsBanned(from, NULL)) {
                MakeLogEntry("ListSERV", MLE_INFO, BANNED_MSG, from);
                return 0;
        }


        /* Process mail body. */

        while (FGets(Input(), buf, BUFSIZE-1)) {
                /* drop trailing newline */
                buf[strlen(buf)-1] = '\0';

                /* check for empty lines */
                if (!(*buf))
                        continue;

                /* log the request */
                MakeLogEntry("ListSERV", MLE_INFO, "%s: %s", from, buf);

                if (!strnicmp(buf, "list ",5)
                || !strnicmp(buf, "delete-all ",11)
                || !strnicmp(buf, "del-all ",8)
                || !strnicmp(buf, "unsubscribe-all ",16)
                || !stricmp(buf, "list")
                || !stricmp(buf, "delete-all")
                || !stricmp(buf, "del-all")
                || !stricmp(buf, "unsubscribe-all"))
                        {
                        gotcommand++;
                        listsearch(from, buf);
                        continue;
                        }

                if (!strnicmp(buf,"add", 3)
                || !strnicmp(buf,"subscribe", 9)
                || !strnicmp(buf,"sub", 3))
                        {
                        gotcommand++;
                        subscription(from,buf,1);
                        continue;
                        }

                if (!strnicmp(buf,"delete", 6)
                || !strnicmp(buf,"del", 3)
                || !strnicmp(buf,"unsubscribe", 11))
                        {
                        gotcommand++;
                        subscription(from,buf,0);
                        continue;
                        }

                if (!stricmp(buf,"index")
                || !stricmp(buf,"longindex"))
                        {
                        gotcommand++;
                        sendindex(from,buf,!stricmp(buf,"longindex"));
                        continue;
                        }

                if (!strnicmp(buf, "faq ", 4))
                        {
                        gotcommand++;
                        sendfaq(from, buf);
                        continue;
                        }

                if (!stricmp(buf, "faq"))
                        if (def_listname) {
                                gotcommand++;
                                sendfaq(from, buf);
                                continue;
                        }
                        else {
                                gotcommand++;
                                faqindex(from, buf);
                                continue;
                        }

                if (!stricmp(buf,"help"))
                        if (def_listname) {
                                gotcommand++;
                                listhelp(from, def_listname, buf);
                                continue;
                        }
                        else {
                                gotcommand++;
                                sendhelp(from,buf);
                                continue;
                        }

                if (!strnicmp(buf,"help ",5))
                        {
                        gotcommand++;
                        sscanf(buf,"%*s%s", grp);
                        listhelp(from, grp, buf);
                        continue;
                        }

                /* Flush message if > 5 lines of garbage */
                if (++garbage > 5) {
                        MakeLogEntry("ListSERV", MLE_ERROR, "Garbled message, flushing...");
                        break;
                        }
                }

        if (!gotcommand) {
                MakeLogEntry("ListSERV", MLE_ERROR, "%s: indecipherable mail --> sending HELP back", from);
                if (def_listname)
                        listhelp(from, def_listname, "indecipherable");
                else
                        sendhelp(from, "indecipherable");
        }

        return 0;
}

void callmailer(char *carboncopy, char *toaddr, char *request)
{
        strcpy(tmpbuf, GetConfig(NULL, SENDMAIL, NULL, SENDMAIL));

        strcat(tmpbuf, " -f " LISTSERVMANAGER " -R \"Mailing List Processor\"");
        if (carboncopy && *carboncopy) {
                strcat(tmpbuf, " -c ");
                strcat(tmpbuf, carboncopy);
        }
        if (!(mailer = POpen(tmpbuf, MODE_PIPETO))) {
                perror(tmpbuf);
                exit(10);
        }
        if (def_listname)
                FPrintf(mailer, "Reply-To: %s-request@", def_listname);
        else
                FPrintf(mailer, "Reply-To: " LISTSERVADDR "@");
        FPrintf(mailer, GetConfig(NULL, NODENAME, NULL, "site"));
        FPrintf(mailer, GetConfig(NULL, DOMAINNAME, NULL, ".domain"));
        FPrintf(mailer, "\n");
        FPrintf(mailer, "To: %s\n", toaddr);
        FPrintf(mailer, "Subject: Re: your ListSERV request \"%s\"\n", request);
#ifdef EVALUATION
        FPrintf(mailer, "X-ListSERV: Peti's Amiga " VERS " -- UNREGISTERED\n");
#else
        FPrintf(mailer, "X-ListSERV: Peti's Amiga " VERS "\n");
#endif
        FPrintf(mailer, "\n");
}

void mailcat(char *mfname, char *prefix)
{
        BPTR mf;

        LockFile(mfname);
        if (!(mf = Open(mfname, MODE_OLDFILE))) {
                UnLockFile(mfname);
                perror(mfname);
                FPrintf(mailer, "This should have been the contents of\n");
                FPrintf(mailer, "file '%s' but it's missing!\n",mfname);
                return;
        }

        /* copy the message file into the mailer */
        while (FGets(mf, tmpbuf, BUFSIZE)) {
                if (prefix && *prefix)
                        FPuts(mailer, prefix);
                FPuts(mailer, tmpbuf);
        }
        Close(mf);
        UnLockFile(mfname);
        return;
}


void ParseAddress(char *line)
{
        int brackets, i, z;

        while(isspace(*line))
                line++;

        for (i = 0, z = 0, brackets = 0; line[i] != '\0'; i++) {
                if (line[i] == '(')
                        brackets++;
                else if (line[i] == ')')
                        brackets--;
                else if (line[i] == '<' && brackets == 0) {
                        i++;
                        while (1) {             /* Forever! */
                                if (line[i] == '>' && (line[i+1] == '\0' || isspace(line[i+1])))
                                        break;
                                line[z++] = line[i++];
                        }
                        line[z] = '\0';
                        break;
                }
        }
        if (z == 0) {   /* No <address> occured. */
                i = 0;
                while (!isspace(line[i]))
                        i++;
                line[i] = '\0';
        }
}


void CleanUp(void)              /* is called by atexit() automatically */
{
        UnLockFiles();
        CloseLibrary((struct Library *) NetSupportBase);
}


 /* Replace the UNIX function with an AmigaOS version. */

void perror(const char *dummy)
{
        extern int _OSERR;
        PrintFault(_OSERR, "ListSERV");
}


