/*
 * $Filename: MemControl.c $
 * $Revision: 1.2 $
 * $Date: 1993/12/18 19:07:56 $
 *
 * written in 1993 by Peter Simons <simons@peti.GUN.de>
 *
 * This program is released under the concept of Public Domain, meaning
 * that it is free of any restrictions or copyrights. You are welcome to
 * spread, use or modify it as you like!
 *
 */

/************************************* switches ***********/
#define __USE_SYSBASE

/************************************* includes ***********/
#include <stdlib.h>
#include <string.h>

#include <exec/execbase.h>
#include <exec/memory.h>
#include <proto/exec.h>
#include <proto/dos.h>

/************************************* defines ************/
#define USAGE "MemoryControl -- written by Peter Simons <simons@peti.GUN.de>\n" \
              "Usage: MemoryControl [[address pri] ...]\n"
#define TEMPLATE "dummy/m"
#define PRGNAME "MemControl"

/************************************* prototypes *********/
void PrintMemList(void);
int SetMemPri(struct MemHeader *, int);
void SortList(struct List *list);
void SwapNodes(struct Node *);

/************************************* global variables ***/
static const char __DOSVer[] = "\0$VER: MemControl v1.2 (25.12.93)";
struct ExecBase *SysBase;
struct DosLibrary *DOSBase;

/************************************* program ************/

int __saveds __asm main(register __d0 int arglen, register __a0 char *arg)
{
        struct RDArgs *RDArgs;
        char **arguments;
        int rc = 0, newpri;
        struct MemHeader *memblock;

        arg[arglen - 1] = '\0';
        SysBase = (*(struct ExecBase **) 4);

        if (!(DOSBase = (struct DosLibrary *) OpenLibrary(DOSNAME, 36)))
                return 20;

        switch (arglen) {
        case 0:
                rc = 20;
                break;

        case 1:
                PrintMemList();
                break;

        default:
                if (!strcmp(arg, "?"))
                        Printf(USAGE);
                else if (RDArgs = ReadArgs(TEMPLATE, (LONG *) & arguments, 0L)) {
                        while (*arguments) {
                                if (!arguments[1]) {
                                        Printf(PRGNAME ": Commandline is not correct, missing argument!\n");
                                        rc = 10;
                                }
                                else {
                                        {
                                                char *param;
                                                if (!strncmp(arguments[0], "0x", 2))
                                                        param = &arguments[0][2];
                                                else if (!strncmp(arguments[0], "$", 1))
                                                        param = &arguments[0][1];
                                                else
                                                        param = arguments[0];
                                                stch_i(param, (int *) &memblock);
                                        }
                                        newpri = atoi(arguments[1]);
                                        if (newpri < -128 || newpri > 127) {
                                                Printf(PRGNAME ": Specified priority %ld is not valid!\n", (LONG) newpri);
                                                rc = 5;
                                        }
                                        else if (!SetMemPri(memblock, newpri)) {
                                                Printf(PRGNAME ": Couldn't find block 0x%08.lx in memory list!\n", memblock);
                                                rc = 5;
                                        }
                                }
                                arguments = &arguments[2];
                        }
                        FreeArgs(RDArgs);
                }
        }

        CloseLibrary((struct Library *) DOSBase);
        return rc;
}

/************************************* subroutines ********/
void PrintMemList(void)
{
        struct MemHeader *MemHeader = (struct MemHeader *) SysBase->MemList.lh_Head;

        Disable();
        while (*(ULONG *) MemHeader) {
                Printf("Node: 0x%08.lx   Name: '%20.20s'   Pri: %4.ld   Size: %ldk\n",
                       MemHeader, MemHeader->mh_Node.ln_Name,
                       (LONG) MemHeader->mh_Node.ln_Pri,
                       ((LONG) MemHeader->mh_Upper - (LONG) MemHeader->mh_Lower + 1)/1024);

                MemHeader = (struct MemHeader *) MemHeader->mh_Node.ln_Succ;
        }
        Enable();
}

int SetMemPri(struct MemHeader *MemBlock, int pri)
{
        struct MemHeader *MemHeader = (struct MemHeader *) SysBase->MemList.lh_Head;

        Disable();
        while (*(ULONG *) MemHeader) {
                if (MemHeader == MemBlock) {
                        MemHeader->mh_Node.ln_Pri = (BYTE) pri;
                        SortList(&(SysBase->MemList));
                        Enable();
                        return 1;
                }
                MemHeader = (struct MemHeader *) MemHeader->mh_Node.ln_Succ;
        }
        Enable();
        return 0;
}

void SortList(struct List *List)
{
        struct Node *node;
        int modified;

        Disable();
        do {
                modified = 0;
                node = List->lh_Head;

                while (*((ULONG *) node->ln_Succ)) {
                        if (node->ln_Succ->ln_Pri > node->ln_Pri) {
                                SwapNodes(node);
                                modified = 1;
                        }
                        node = node->ln_Succ;
                }
        } while (modified);
        Enable();

}

void SwapNodes(struct Node *Node)
{
        struct Node *Node2, *preNode, *succNode;

        Node2 = Node->ln_Succ;
        preNode = Node->ln_Pred;
        succNode = Node2->ln_Succ;

        preNode->ln_Succ = Node2;

        Node2->ln_Succ = Node;
        Node2->ln_Pred = preNode;

        Node->ln_Succ = succNode;
        Node->ln_Pred = Node2;

        succNode->ln_Pred = Node;
}
