echo;                           /* Execute to compile with SAS/C v6.x * /

SC Crossword.c LINK PARAMETERS=REGISTERS STRINGMERGE GST LIB:Includes.gst DEBUG=FF
delete Crossword.lnk Crossword.o
quit
*/

/*
 *      $Filename: Crossword.c $
 *      $Revision: 1.0 $
 *      $Date: 1994/01/26 $
 *
 *      This tools help to solve a special kind of crossword.
 *
 *      This is Public Domain!
 */

/**************************************************************************
 *                                                                        *
 * Sektion: Macros, Definitions, Includes, Structures                     *
 *                                                                        *
 **************************************************************************/

/************************************* Includes ***********/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/************************************* Defines ************/
#define WIDTH   12
#define HEIGHT  36

/************************************* Prototypes *********/
int FindWord(char *, int, int);
int FindWord_internal(char *, int, int, int, int);
void MarkChars(int, int, int, char *);

/************************************* Global Variables ***/
char Crossword[HEIGHT][WIDTH+1] = {
        "RAFSESLERTSV",
        "ECECNCAIEACE",
        "IKLHIHRCFSHR",
        "EEDNELVHUTWD",
        "BRSEBAEEEEIA",
        "OSALLMLNARMU",
        "RANLEMEBLSMU",
        "KNDSTSDODCKN",
        "EDLCTCECNHAG",
        "NLAHIHRKAEES",
        "KAEWMWLPSNFS",
        "AEUIRIAUDKEA",
        "EUFMEMUPLERE",
        "FFEMSMFPALHF",
        "EERESEKEWRIT",
        "RRERERANPINE",
        "XKTBRLERANTS",
        "GLOAFEFAAGEU",
        "AOEUNIEERNRE",
        "RBNCEBRUUEFA",
        "TEEHZZTBNBLH",
        "ERKWNWAEGIUE",
        "NKCAAEURSEEG",
        "LIESLRMEZHGV",
        "AEDSFGEFECEO",
        "UFLEPSLEISLR",
        "FEERRCKATTBD",
        "KRGKEHAKDFRE",
        "AZEALWERRAAR",
        "EAUEHIFEUHNF",
        "FNLFEMEDEADL",
        "EGFEUMROSAUU",
        "RESRFEIMERVE",
        "ENIEBREDROVG",
        "DECKFLUEGELE",
        "VORDERKANTEL"
};
char FoundChars[HEIGHT][WIDTH];

char *Words[] = {
        "ACKERSANDLAEUFER",
        "BAUCH",
        "BORKENKAEFER",
        "DECKFLUEGEL",
        "DRUESE",
        "EICHENBOCK",
        "EIER",
        "FELDSANDLAEUFER",
        "FLUEGELDECKEN",
        "FUEHLER",
        "GARTENLAUFKAEFER",
        "GEHAEUSE",
        "GELBRAND",
        "HAAR",
        "HAFTSCHEIBEN",
        "HINTERFLUEGEL",
        "KIEFERZANGEN",
        "LARVE",
        "LEDERLAUFKAEFER",
        "LEIB",
        "MITTELBEINE",
        "MODERKAEFER",
        "OBERKIEFER",
        "PAARUNGSZEIT",
        "PFLANZENFRESSER",
        "PUPPENRAEUBER",
        "SCHENKELRING",
        "SCHLAMMSCHWIMMER",
        "SCHNELLSCHWIMMER",
        "SCHWIMMKAEFER",
        "TASTER",
        "TAUMELKAEFER",
        "TOENE",
        "VERDAUUNGSSAEFTE",
        "VORDERBEINE",
        "VORDERFLUEGEL",
        "VORDERKANTE",
        "WALDSANDLAEUFER",
        "WASSERKAEFER",
        "ZWERGSCHWIMMER",
        NULL
};



/**************************************************************************
 *                                                                        *
 * Sektion: Hauptprogramm                                                 *
 *                                                                        *
 **************************************************************************/


int main(void)
{
        int i, z, word = 0, direction;

        /*
         * Print the crossword!
         */

        printf("Das Kreuzortrtsel sieht wie folgt aus:\n\n");
        for (i = 0; i < HEIGHT; i++) {
                for (z = 0; z < WIDTH; z++)
                        printf(" %c ", Crossword[i][z]);
                printf("\n");
        }
        printf("\nund wir suchen folgende Wrter:\n");
        for (i = 0; Words[i]; i++)
                printf("\t%s\n", Words[i]);
        printf("\n");


        /*
         * Solve it!
         */

        while (Words[word]) {
                for (i = 0; i < HEIGHT; i++)
                        for (z = 0; z < WIDTH; z++)
                                if ((direction = FindWord(Words[word], z, i)) != -1) {
                                        printf("Treffer, \"%s\": %d/%d, Richtung: %d\n", Words[word], z+1, i+1, direction);
                                        MarkChars(z, i, direction, Words[word]);
                                }
                word++;
        }

        /*
         * Print the the solution!
         */

        printf("\n");
        for (i = 0; i < HEIGHT; i++) {
                for (z = 0; z < WIDTH; z++)
                        if (FoundChars[i][z] == '\0')
                                printf(" %c ", Crossword[i][z]);
                        else
                                printf(" . ");
                printf("\n");
        }
        printf("\n\n");

        printf("\nLsungsbuchstaben:");
        for (i = 0; i < HEIGHT; i++)
                for (z = 0; z < WIDTH; z++)
                        if (FoundChars[i][z] == '\0'
                            && Crossword[i][z] != '.')
                                printf(" %c ", Crossword[i][z]);
        printf("\n");

        return 0;
}


/**************************************************************************
 *                                                                        *
 * Sektion: Unterroutinen                                                 *
 *                                                                        *
 **************************************************************************/

int FindWord(char *word, int width, int height)
{
        int wordlen = strlen(word);

        if (height+1 >= wordlen)
                if (FindWord_internal(word, width, height, 0, -1))
                        return 0;

        if (WIDTH-width+1 >= wordlen && height+1 >= wordlen)
                if (FindWord_internal(word, width, height, 1, -1))
                        return 1;

        if (WIDTH-width+1 >= wordlen)
                if (FindWord_internal(word, width, height, 1, 0))
                        return 2;

        if (WIDTH-width+1 >= wordlen && HEIGHT-height+1 >= wordlen)
                if (FindWord_internal(word, width, height, 1, 1))
                        return 3;

        if (HEIGHT-height+1 >= wordlen)
                if (FindWord_internal(word, width, height, 0, 1))
                        return 4;

        if (width+1 >= wordlen && HEIGHT-height+1 >= wordlen)
                if (FindWord_internal(word, width, height, -1, 1))
                        return 5;

        if (width+1 >= wordlen)
                if (FindWord_internal(word, width, height, -1, 0))
                        return 6;

        if (width+1 >= wordlen && height+1 >= wordlen)
                if (FindWord_internal(word, width, height, -1, -1))
                        return 7;

        return -1;
}


int FindWord_internal(char *word, int width, int height, int right, int down)
{
        int i, wordlen = strlen(word);

        for (i = 0; i < wordlen; i++) {
                if (word[i] != Crossword[height][width] && Crossword[height][width] != ' ')
                        return 0;
                width += right;
                height += down;
        }

        return 1;
}



void MarkChars(int width, int height, int direction, char *word)
{
        int right, down, wordlen = strlen(word);

        switch(direction)
        {
        case 0: right = 0, down = -1;
                break;
        case 1: right = 1, down = -1;
                break;
        case 2: right = 1, down = 0;
                break;
        case 3: right = 1, down = 1;
                break;
        case 4: right = 0, down = 1;
                break;
        case 5: right = -1, down = 1;
                break;
        case 6: right = -1, down = 0;
                break;
        case 7: right = -1, down = -1;
                break;
        }

        while (wordlen--) {
                FoundChars[height][width] = *word++;
                width += right;
                height += down;
        }
}

