echo; /*    Execute to compile with SAS/C v6.x

SC SignText.c DEBUG=FF LINK STRICT ANSI STARTUP=cres PARAMETERS=REGISTERS ; NOSTACKCHECK STRINGMERGE OPTIMIZE OPTIMIZERSIZE NOOPTIMIZERINLINE OPTIMIZERCOMPLEXITY=5 OPTIMIZERDEPTH=5 OPTIMIZERRECURDEPTH=5
delete SignText.lnk SignText.o
quit

*/

/**
 **     $Filename: SignText.c $
 **     $Revision: 0.1 $
 **     $Date: 1993/08/03 $
 **
 **     Signs a file using PGP.
 **
 **      Copyright 1993 Peter Simons, Germany
 **       All Rights Reserved
 **/



/**************************************************************************
 *                                                                        *
 * Sektion: Macros, Definitions, Includes, Structures                     *
 *                                                                        *
 **************************************************************************/

/************************************* Includes ***********/
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>


/************************************* Defines ************/
#define PGP "PGP -ftsa <%s >%s +clearsig +batchmode"

#define NO_ERROR        0
#define ERR_SRCFILE     1
#define ERR_TMPFILE     2

#define IOBUFSIZE       BUFSIZ*10

/************************************* Prototypes *********/
int CopyUntilEOF(FILE *in, FILE *out);


/**************************************************************************
 *                                                                        *
 * Sektion: Hauptprogramm                                                 *
 *                                                                        *
 **************************************************************************/

int main(int argc, char *argv[])
{
        FILE *srcfile, *tmpfile;
        int returncode = NO_ERROR;
        char cmd[81], *TMPFILE;

        if ((TMPFILE = tmpnam(NULL)) == NULL)
                exit(20);

        if (argc != 2 || !strcmp("?", argv[1]))
                fprintf(stderr, "USAGE: SignText <textfile>\n");
        else {
                if (srcfile = fopen(argv[1], "r")) {
                        if (tmpfile = fopen(TMPFILE, "w")) {

                        /* Das Sourcefile ins temporre File kopieren */

                                if (CopyUntilEOF(srcfile, tmpfile) == NULL)
                                        exit(ERR_TMPFILE);

                        /* Beide Files schlieen */

                                fclose(srcfile);
                                fclose(tmpfile);

                        /* Let PGP do the job */

                                sprintf(cmd, PGP, TMPFILE, argv[1]);
                                system(cmd);

                        /* Remove the temporary file */

                                remove(TMPFILE);
                        }
                        else {
                                fprintf(stderr, "Can't open temporary file!\n");
                                returncode = ERR_TMPFILE;
                        }
                }
                else {
                        /*
                         * Falls das ffnen des temporren Files nicht
                         * geklappt hat, ist das Sourcefile noch offen.
                         */

                        if (returncode == ERR_TMPFILE)
                                fclose(srcfile);

                        fprintf(stderr, "Can't open file for read access!\n");
                        returncode = ERR_SRCFILE;
                }
        }

        return returncode;
}

int CopyUntilEOF(FILE *in, FILE *out)
{
        char *buffer;
        int len;

        if ((buffer = malloc(IOBUFSIZE)) == NULL)
                return 0;

        if (in != stdin)
                lseek(fileno(in), ftell(in), 0);
        if (out != stdout)
                lseek(fileno(out), ftell(out), 0);

        while ((len = read(fileno(in), buffer, IOBUFSIZE)) > 0)
                if (write(fileno(out), buffer, len) == -1) {
                        free(buffer);
                        return 0;
                }

        free(buffer);
        return 1;
}


