/*
 *      $Filename: LogFiles.c $
 *      $Revision: 1.18 $
 *      $Date: 1994/10/25 13:55:19 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *      $Id: LogFiles.c,v 1.18 1994/10/25 13:55:19 simons Exp simons $
 *
 */


/**************************************************************************
 *                                                                        *
 * SEKTION: Labels, Macros, Switches, Structures                          *
 *                                                                        *
 **************************************************************************/

/************************************* Includes ***********/
#include <string.h>
#include <exec/types.h>
#include <proto/dos.h>
#include <proto/utility.h>

#include "libraries/netsupport.h"
#include "NetSupportLibrary.h"
#include "MemoryHandling.h"
#include "SupportRoutines.h"
#include "ConfigFiles.h"

/************************************* Defines ************/

/************************************* Prototypes *********/
STRPTR *DetermineLogfile(STRPTR, ULONG);
void RemTrailingBlanks(STRPTR);

static const char __RCSId[] = "$Id: LogFiles.c,v 1.18 1994/10/25 13:55:19 simons Exp simons $";

/**************************************************************************
 *                                                                        *
 * SEKTION: Public library functions                                      *
 *                                                                        *
 **************************************************************************/

/****** netsupport.library/MakeLogEntry ***********************************
*
*
*    NAME
*       MakeLogEntry -- generates an entry in the appropriate logfile
*
*    SYNOPSIS
*       MakeLogEntry(system, class, logmessage, ...)
*                    A0      D0     A1          A2
*
*       STRPTR  system;
*       ULONG   class;
*       STRPTR  logmessage;
*
*    FUNCTION
*       This routine determines the appropriate logfile for an entry,
*       using the specified system and class parameters (please refer to
*       the libraries user manual for further information). Then it
*       locks this file and writes the logmessage into it, prefaced with
*       the current date and time.
*
*    INPUTS
*       system - Pointer to a string, specifying the 'system', making the
*               log entry. The system should be a short, descriptive
*               name for your program, maybe the package it belongs to.
*               The user can use the system name to direct the log-entries
*               to certain logfiles, so better be verbose. :-)
*               You should document the system name your program uses!
*               If NULL is specified, the log-entry goes into the
*               default logfile. If no default is specified, "T:Logfile"
*               is used.
*
*       class - value, describing the kind of the logmessage
*
*       logmessage - pointer to a string containing the actual message
*               text. Please refer to RawDoFmt() for usage.
*
*    RESULT
*       none
*
*    CLASSES
*       MLE_DEFAULT - This class is used if no special class can be
*               determined (0L). This makes it easier to port older
*               sources to the new routine--however, this entry should
*               be avoided.
*
*       MLE_INFO - This class should be used for plain information logs,
*               what happened to what time.
*
*       MLE_DEBUG[1-9] - This class should be used for debugging output.
*               The number following the actual name is the debug level
*               of the log-entry. MLE_DEBUG9, for example, is a rather
*               unimportant debug message, while MLE_DEBUG0 is rather
*               elementary and important to find bugs.
*
*       MLE_ERROR - This class should be used to log occurring non-fatal
*               errors.
*
*       MLE_FATAL_ERROR - This class should be used to log fatal errors.
*               Fatal errors are very important and usually need
*               immediate fixing. So be careful using this class.
*
*    EXAMPLE ENTRY
*       (10-Nov-93/13:04:18) UUCP, Sending mail to foo@bar.UUCP
*       (10-Nov-93/13:04:56) TCP/IP, Connected wuarchive.wust.edu
*       (10-Nov-93/13:05:01) TCP/IP, Starting FTP request for foobar.lha
*
*    NOTE
*
*    SEE ALSO
*       RawDoFmt()
*
***************************************************************************
*
*/

void __asm MakeLogEntry(register __a0 STRPTR system,
                        register __d0 ULONG class,
                        register __a1 STRPTR logmessage)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        struct UtilityBase *UtilityBase = NetSupportBase->nsp_UtilityBase;
        BPTR fh;
        STRPTR *logfile;
        struct DateTime dt;
        TEXT StrDate[LEN_DATSTRING], StrTime[LEN_DATSTRING];

        if (!(logfile = DetermineLogfile(system, class)))
                return;

        DateStamp((struct DateStamp *) &dt);
        dt.dat_Format = FORMAT_DOS;
        dt.dat_Flags = 0;
        dt.dat_StrDay = NULL;
        dt.dat_StrDate = StrDate;
        dt.dat_StrTime = StrTime;

        if (DateToStr(&dt)) {
                RemTrailingBlanks(StrDate);       /* remove trailing blanks */
                RemTrailingBlanks(StrTime);
                while (*logfile) {
                        if (Strnicmp(*logfile, "stdout", 7)) {
                                if (fh = Open(*(logfile++), MODE_READWRITE)) {
                                        Seek(fh, 0L, OFFSET_END);
                                        FPrintf(fh, "(%s/%s) %s, %s\n", StrDate, StrTime, system, logmessage);
                                        Close(fh);
                                }
                        }
                        else {
                                        Printf("(%s/%s) %s, %s\n", StrDate, StrTime, system, logmessage);
                                        logfile++;
                        }
                }
        }

        if (logfile)
                NSPFreeMemPooled(logfile);
        NSPFreeMemPooled(logmessage);   /* free allocated memory */
}


/**************************************************************************
 *                                                                        *
 * SEKTION: Internal library functions                                    *
 *                                                                        *
 **************************************************************************/


STRPTR *DetermineLogfile(STRPTR system, ULONG class)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        STRPTR *logfile, tmpbuf, debugkey, config = GetNSPConfigName();
        LONG i, debuglevel;


        /* Check debuglevel. */

        if (class >= MLE_DEBUG1 && class <= MLE_DEBUG9) {
                debuglevel = class - (MLE_DEBUG1-1);
                if (!(debugkey = NSPAllocMemPooled(strlen(system)+8, 0L)))
                        return NULL;
                StringCopy(StringCopy(debugkey, system), "_debug");
                if (!(tmpbuf = GetConfig(config, debugkey, NULL, NULL)))
                        tmpbuf = GetConfig(config, "Debug", NULL, "0");

                NSPFreeMemPooled(debugkey);
                StrToLong(tmpbuf, &i);
                if (i < debuglevel)     /* Is <system>_debug higher than the message class? */
                        return NULL;
        }


        /* Obtain logfile(s) name(s). */

        if (!(tmpbuf = GetConfigEntry(config, system, NULL)))
                tmpbuf = GetConfig(config, "DefaultLog", NULL, "T:Logfile");


        /* Parse returned line */

        if (!(logfile = (STRPTR *) NSPAllocMemPooled(50*sizeof(STRPTR), 0L)))
                return NULL;


        i = 0;
        logfile[i++] = tmpbuf;
        do {
                if (*tmpbuf == ',') {
                        *(tmpbuf++) = '\0';
                        RemTrailingBlanks(logfile[i-1]);
                        while (*tmpbuf == ' ' || *tmpbuf == '\t')
                                tmpbuf++;
                        logfile[i++] = tmpbuf;
                }
                else
                        tmpbuf++;
        } while (*tmpbuf);

        logfile[i] = NULL;
        return logfile;
}


        /* This routine removes trailing spaces from the generated date
         * string. These blanks are added in order make all date strings
         * the same length. However, this is not required for our
         * application.
         */

void RemTrailingBlanks(STRPTR string)
{
        while(*string)
                string++;

        while(*(string-1) == ' ')
                *(--string) = '\0';
}
