*
*       $Filename: MemoryHandling.asm $
*       $Revision: 1.14 $
*       $Date: 1994/05/24 21:12:09 $
*
*       Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
*
*       This program is free software; you can redistribute it and/or
*       modify it under the terms of the GNU General Public License as
*       published by the Free Software Foundation; either version 2 of
*       the License, or (at your option) any later version.
*
*       This program is distributed in the hope that it will be useful,
*       but WITHOUT ANY WARRANTY; without even the implied warranty of
*       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*       General Public License for more details.
*
*       You should have received a copy of the GNU General Public License
*       along with this program; if not, write to the Free Software
*       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*
*       $Id: MemoryHandling.asm,v 1.14 1994/05/24 21:12:09 simons Exp simons $
*
*


;***************************************************************************
;*                                                                         *
;* SEKTION: Labels, Macros, Switches, Structures                           *
;*                                                                         *
;***************************************************************************

;-------------------------------------- Switches -----------
                MC68000
                NEWSYNTAX
                OBJFILE "MemoryHandling.o"
                INCDIR  "INCLUDE:"
                MULTIPASS
;               VERBOSEOPTIM
                DEBUG on
                SYM

;-------------------------------------- Labels ------------
                MACLIB  "lib:Offsets.mac"
                MACFILE "exec/exec.i"

                MACFILE "libraries/netsupport.i"
                MACFILE "NetSupportLibrary.i"

;-------------------------------------- Structures --------
ULMemoryList    CLRSO           ; internal structure of RememberList
nspml_NextBlock SO.L    1
nspml_Size      SO.L    1
nspml_Pooled    SO.L    1
nspml_SIZEOF    SOVAL

StaticBuffer            CLRSO           ; internal structure of a
nspsb_NextEntry         SO.L    1       ; StaticBuffer
nspsb_TaskID            SO.L    1
nspsb_InternalMemList   SO.L    1
nspsb_TaskMemList       SO.L    1
nspsb_GCEntryBuffer     SO.L    1
nspsb_LockedFileList    SO.L    1
nspsb_TempNameBuffer    SO.L    1
nspsb_NSPConfigName     SO.L    1
nspsb_SIZEOF            SOVAL

;-------------------------------------- Macros ------------

;-------------------------------------- Entry-points ------
                XDEF    _AllocMemPooled
                XDEF    _FreeMemPooled
                XDEF    _GetStaticBuffer
                XDEF    _FreeStaticBuffer
                XDEF    _NSPAllocMemPooled
                XDEF    _NSPFreeMemPooled

;-------------------------------------- References --------
                XREF    _AsmAllocPooled
                XREF    _AsmFreePooled


;***************************************************************************
;*                                                                         *
;* SEKTION: Public library functions                                       *
;*                                                                         *
;***************************************************************************


;****** netsupport.library/AllocMemPooled *********************************
;
;    NAME
;       AllocMemPooled -- allocate a block of memory out of the
;               libraries memory pool
;
;    SYNOPSIS
;       memblock = AllocMemPooled(blocksize, attributes);
;       D0                        D0         D1
;
;       void *  memblock;
;       ULONG   blocksize;
;       ULONG   attributes;
;
;    FUNCTION
;       This routines allocates a memory block of the requested size
;       using the libraries internal memory pool. This is a big
;       advantage when several smaller allocations have to be made,
;       because memory pooling reduces fragmentation significantly.
;
;       Using this routine you won't have to care about managing the
;       pool, the library will do for you. If the routine is started
;       under exec.library v39 or later, the exec pool routines will be
;       used. Under Kickstart 37, the library will handle the pool
;       itself.
;
;    INPUTS
;       blocksize = this is the size the requested block in byte
;
;       attributes = AllocMemPooled() understands exactly the same
;               attributes as the original exec routines. Please refer to
;               AllocMem() or exec/memory.h for further details.
;
;    RESULT
;       The routine returns either the address of the allocated block or
;       NULL, if the routine failed due to low-memory condition.
;
;    NOTE
;       The library tracks all allocations you do and frees them itself
;       when CloseLibrary() is called. However, it is strongly
;       recommended to free all memory IMMEDIATELY when it isn't needed
;       anymore. It is very bad programming style to rely on the library
;       freeing everything for you.
;
;       If you're calling the routine in another process, you created
;       using CreateNewProc() or a similar routine, you MUST open the
;       library again! Do not just pass the LibraryBase to the new
;       process or you'll break the memory-tracking mechanism, either
;       causing loss of memory or even a complete system crash.
;
;    SEE ALSO
;       FreeMemPooled()
;
;
;**************************************************************************
;
; Font-end to NSPAllocRemember() for public usage. The routine
; determines the address of the task's static buffer and provides
; remember key, size and attributes to the real routine.
;

_AllocMemPooled:
                move.l  d0,-(SP)
                bsr     _GetStaticBuffer        ; Get task's static buffer.
                tst.l   d0
                beq.b   .exit
                move.l  d0,a0
                move.l  (SP)+,d0
                lea     (nspsb_TaskMemList,a0),a0
                bra     _NSPAllocRemember

.exit           move.l  (SP)+,d0
                rts



;****** netsupport.library/FreeMemPooled **********************************
;
;
;    NAME
;       FreeMemPooled -- free a memory block previously allocated with
;                       AllocMemPooled()
;
;    SYNOPSIS
;       success = FreeMemPooled(memblock);
;       D0                      A1
;
;       LONG    success;
;       void *  memblock;
;
;    FUNCTION
;       This routines frees a memory block, allocated with AllocMemPooled()
;       earlier. Just the address is required, the library keeps track
;       of the blocksizes itself.
;
;    INPUTS
;       memblock = pointer the the memory block
;
;    RESULT
;       Either -1L for success or 0L for failure.
;
;    NOTE
;       The memory allocation tracking mechanism will catch an attempt
;       to free a memory block twice. However, please do NOT rely on
;       this feature!
;
;    SEE ALSO
;       AllocMemPooled()
;
;
;**************************************************************************
;
; Font-end to NSPFreeOneBlock() for public usage. The routine
; determines the address of the task's static buffer and provides
; remember key, size and attributes to the real routine.
;

_FreeMemPooled:
                ; Font-end to NSPFreeOneBlock() for public usage. The
                ; routine determines the address of the task's static
                ; buffer and provides remember key, and block address
                ; to the real routine.
                ;
                ; Parameter: A1=&MemoryBlock
                ; Result   : D0=TRUE/FALSE (success/failure)
                ; Comment  : see exec/memory{.h|.i}

                bsr     _GetStaticBuffer        ; Get task's static buffer.
                tst.l   d0
                beq.b   .exit
                move.l  d0,a0
                lea     (nspsb_TaskMemList,a0),a0
;               move.l  a1,a1
                bra     _NSPFreeOneBlock

.exit           rts



;***************************************************************************
;*                                                                         *
;* SEKTION: Internal library functions                                     *
;*                                                                         *
;***************************************************************************


;****i* MemoryHandling.asm/GetStaticBuffer ********************************
;
;    NAME
;       GetStaticBuffer -- find a task's static data buffer
;
;    SYNOPSIS
;       staticbuf = GetStaticBuffer(void);
;
;       struct StaticBuffer *staticbuf;
;
;    FUNCTION
;       Because routines in a shared library have to be re-entrant, it's
;       a little bit difficult to store certain data statically, as you
;       can't just use an entry in the library header.
;
;       GetStaticBuffer() initializes a special structure for each task
;       and returns a pointer, so the routine can store the required data
;       in this structure for later usage.
;
;    INPUTS
;       none
;
;    RESULT
;       GetStaticBuffer() returns a pointer to the task's individual
;       structure or NULL for failure.
;
;    EXAMPLE
;       Here's a short example to demonstrate the usage of
;       GetStaticBuffer. The following routine will count the number of
;       times it has been called and returns the integer. (Not very
;       useful, agreed. :->)
;
;               LONG CountMe(void)
;               {
;                       struct StaticBuffer *sb;
;
;                       if (sb = GetStaticBuffer())
;                               return ++(sb->nspsb_CountMe);
;                       else
;                             return 0L;
;               }
;
;    SEE ALSO
;       FreeStaticBuffer()
;
;**************************************************************************
;

_GetStaticBuffer:
                PUSHM   d1/a0-a3/a5-a6
                move.l  (_NetSupportBase,PC),a5
                sub.l   a3,a3

                move.l  (nsp_StaticBufferSemaphore,a5),a0
                move.l  ($4).w,a6                ; Make sure, we're the
                jsr     (_LVOObtainSemaphore,a6) ; task accessing the list
                                                 ; currently.
                sub.l   a1,a1
                jsr     (_LVOFindTask,a6)

                lea     (nsp_StaticBufferList,a5),a2
...             tst.l   (a2)
                beq.b   .CreateStaticBuffer
                move.l  (a2),a2
                cmp.l   (nspsb_TaskID,a2),d0
                beq.b   .FoundStaticBuffer
                bra.b   ...

.CreateStaticBuffer:
                moveq   #nspsb_SIZEOF,d0
                move.l  #MEMF_ANY|MEMF_CLEAR,d1
                jsr     (_LVOAllocMem,a6)
                move.l  d0,(a2)
                beq.b   .FreeSemaphore
                move.l  d0,a3

                sub.l   a1,a1
                jsr     (_LVOFindTask,a6)
                move.l  d0,(nspsb_TaskID,a3)
                bra.b   .FreeSemaphore

.FoundStaticBuffer:
                move.l  a2,a3

.FreeSemaphore  move.l  (nsp_StaticBufferSemaphore,a5),a0
                move.l  ($4).w,a6
                jsr     (_LVOReleaseSemaphore,a6)

                move.l  a3,d0
                POPM
                rts



;****i* MemoryHandling.asm/FreeStaticBuffer *******************************
;
;    NAME
;       FreeStaticBuffer -- deallocate a task's static buffer
;
;    SYNOPSIS
;       FreeStaticBuffer(void);
;
;    FUNCTION
;       This routine gets the tasks static buffer, frees all allocated
;       memory that has been tracked in the lists and  removes the static
;       buffer itself from the library internal list.
;
;       This routine may be modified as more entries are added to the
;       static buffer structure--currently holding only memory allocations.
;       However, it is possible to track opened files, and more...
;
;    INPUTS
;       none
;
;    RESULT
;       none
;
;    NOTE
;       This routine is usually called when the library is closed!
;
;    SEE ALSO
;       GetStaticBuffer()
;
;**************************************************************************
;

_FreeStaticBuffer:
                PUSHM   d0-d1/a0-a3/a5-a6
                move.l  (_NetSupportBase,PC),a5

                move.l  (nsp_StaticBufferSemaphore,a5),a0
                move.l  ($4).w,a6                ; Make sure, we're the
                jsr     (_LVOObtainSemaphore,a6) ; task accessing the list
                                                 ; currently.
                sub.l   a1,a1
                jsr     (_LVOFindTask,a6)

                lea     (nsp_StaticBufferList,a5),a2
...             tst.l   (a2)                    ; No static buffer was
                beq.b   .FreeSemaphore          ; created.
                move.l  a2,a3
                move.l  (a2),a2
                cmp.l   (nspsb_TaskID,a2),d0
                beq.b   .FoundStaticBuffer
                bra.b   ...

.FoundStaticBuffer:
                lea     (nspsb_InternalMemList,a2),a0
                bsr     _NSPFreeRemember

                lea     (nspsb_TaskMemList,a2),a0
                bsr     _NSPFreeRemember

                move.l  (nspsb_NextEntry,a2),(a3) ; Remove buffer from list!

                move.l  a2,a1
                moveq   #nspsb_SIZEOF,d0
                jsr     (_LVOFreeMem,a6)

.FreeSemaphore  move.l  (nsp_StaticBufferSemaphore,a5),a0
                move.l  ($4).w,a6
                jsr     (_LVOReleaseSemaphore,a6)

                POPM
                rts



;****i* MemoryHandling.asm/NSPAllocMemPooled ******************************
;
;    NAME
;       NSPAllocMemPooled -- allocate memory for library internal usage
;
;    SYNOPSIS
;       memblock = NSPAllocMemPooled(blocksize, attributes);
;       D0                           D0         D1
;
;       void *  memblock;
;       ULONG   blocksize;
;       ULONG   attributes;
;
;    FUNCTION
;       This routine does mainly the same as AllocMemPooled(). The only
;       difference is, that the allocated memory is tracked in a different
;       list. The advantage is, that FreeStaticBuffer() is able to
;       determine wether this was an internal required memoryblock or the
;       user has forgotten to free one of his allocations. Future versions
;       of the library might pop up a requester a la: "The following
;       allocated memoryblocks were not freed at run-time:...".
;
;       This might be useful for debugging purposes.
;
;    INPUTS
;       blocksize = this is the size the requested block in byte
;
;       attributes = AllocMemPooled() understands exactly the same
;               attributes as the original exec routines. Please refer to
;               AllocMem() or exec/memory.h for further details.
;
;    RESULT
;       The routine returns either the address of the allocated block or
;       NULL, if the routine failed due to low-memory condition.
;
;    NOTE
;
;    SEE ALSO
;       NSPFreeMemPooled(), AllocMemPooled(), FreeMemPooled()
;
;
;**************************************************************************
;
; Font-end to NSPAllocRemember() for internal usage. The routine
; determines the address of the task's static buffer and provides
; remember key, size and attributes to the real routine.
;

_NSPAllocMemPooled:
                move.l  d0,-(SP)
                bsr     _GetStaticBuffer        ; Get task's static buffer.
                tst.l   d0
                beq.b   .exit
                move.l  d0,a0
                move.l  (SP)+,d0
                lea     (nspsb_InternalMemList,a0),a0
                bra     _NSPAllocRemember

.exit           move.l  (SP)+,d0
                rts


;****i* netsupport.library/NSPFreeMemPooled *******************************
;
;
;    NAME
;       NSPFreeMemPooled -- free a memory block previously allocated
;                       with AllocMemPooled()
;
;    SYNOPSIS
;       success = NSPFreeMemPooled(memblock);
;       D0                      A1
;
;       LONG    success;
;       void *  memblock;
;
;    FUNCTION
;       This routine frees a block, allocated with NSPAllocMemPooled()
;       earlier. Just the address is required, the library keeps track
;       of the blocksizes itself.
;
;    INPUTS
;       memblock = pointer the the memory block
;
;    RESULT
;       Either -1L for success or 0L for failure.
;
;    NOTE
;       The memory allocation tracking mechanism will catch an attempt
;       to free a memory block twice. However, please do NOT rely on
;       this feature!
;
;    SEE ALSO
;       NSPAllocMemPooled(), AllocMemPooled(), FreeMemPooled()
;
;
;**************************************************************************
;
; Font-end to NSPFreeOneBlock() for internal usage. The routine
; determines the address of the task's static buffer and provides
; remember key, size and attributes to the real routine.
;


_NSPFreeMemPooled:
                bsr     _GetStaticBuffer        ; Get task's static buffer.
                tst.l   d0
                beq.b   .exit
                move.l  d0,a0
                lea     (nspsb_InternalMemList,a0),a0
;               move.l  a1,a1
                bra     _NSPFreeOneBlock

.exit           rts




_NSPAllocRemember:
                ; Allocates a memory block and adds it to a tracking list
                ;
                ; Parameter: A0=RememberKey D0=Size D1=Flags
                ; Result   : D0=Address / NULL
                ; Comment  : This routine may NOT be used by several tasks
                ;            at the same time!

                PUSHM   d2/a2-a4/a6
                add.l   #nspml_SIZEOF,d0        ; add structure overhead
                move.l  d0,d2
                move.l  a0,a2
                move.l  (_NetSupportBase,PC),a4

                and.l   #$FFFFFFFF^(MEMF_PUBLIC!MEMF_REVERSE),d1
                bne.b   .UseAllocMem

                bsr     _LockAnyMemPool
                move.l  (nsp_AnyMemPool,a4),a0
                move.l  d2,d0
                move.l  ($4).w,a6
                bsr     (_AsmAllocPooled)
                bsr     _FreeAnyMemPool
                tst.l   d0
                beq.b   .exit
                move.l  d0,a1
                move.l  (nsp_AnyMemPool,a4),(nspml_Pooled,a1)
                bra.b   .GotIt

.UseAllocMem    move.l  ($4).w,a6               ; No pool available?
                jsr     (_LVOAllocMem,a6)       ; Use AllocMem()!
                tst.l   d0
                beq.b   .exit

                move.l  d0,a1                   ; clear pool information
                clr.l   (nspml_Pooled,a1)
.GotIt          clr.l   (nspml_NextBlock,a1)    ; Initialize the listheader
                move.l  d2,(nspml_Size,a1)

...             tst.l   (nspml_NextBlock,a2)    ; Find last entry in the
                beq.b   .FoundLastEntry         ; existing list...
                move.l  (nspml_NextBlock,a2),a2
                bra.b   ...
.FoundLastEntry move.l a1,(a2)                  ; and append the block.

                lea     (nspml_SIZEOF,a1),a1    ; Return the address of the
                move.l  a1,d0                   ; the actual memory.

.exit           POPM
                rts



_NSPFreeRemember:
                ; Frees all memblocks tracked down in a memory list
                ;
                ; Parameter: A0=&RememberKey
                ; Result   : none
                ; Comment  : This routine may NOT be used by several tasks
                ;            at the same time!

                PUSHM   a2-a3/a6
                move.l  a0,a2
                move.l  ($4).w,a6

                tst.l   (a2)                    ; Memory list is empty!
                beq.b   .exit

                move.l  (nspml_NextBlock,a2),a0
                clr.l   (a2)                    ; Reset the list pointer
                move.l  a0,a2

.FreeLoop       cmp.l   #0,a2                   ; no block left
                beq.b   .exit
                move.l  a2,a3
                move.l  (nspml_NextBlock,a2),a2 ; get next block
                tst.l   (nspml_Pooled,a3)
                beq.b   .FreeMem

                bsr     _LockAnyMemPool
                move.l  a3,a1
                move.l  (nspml_Pooled,a1),a0
                move.l  (nspml_Size,a1),d0
                bsr     (_AsmFreePooled)
                bsr     _FreeAnyMemPool
                bra.b   .FreeLoop

.FreeMem        move.l  a3,a1
                move.l  (nspml_Size,a1),d0
                jsr     (_LVOFreeMem,a6)
                bra.b   .FreeLoop

.exit           POPM
                rts


_NSPFreeOneBlock:
                ; Frees one memory block and removes it from the list.
                ;
                ; Parameter: A0=&RememberKey A1=&Address
                ; Result   : TRUE/FALSE (success/failure)
                ; Comment  : This routine may NOT be used by several tasks
                ;            at the same time!

                PUSHM   a2-a3/a6
                move.l  ($4).w,a6
                move.l  (_NetSupportBase,PC),a3
                lea     (-(nspml_SIZEOF),a1),a1 ; Get address of block's
                                                ; memory list header
                moveq   #0,d0

...             tst.l   (nspml_NextBlock,a0)    ; Try to find the entry in
                beq.b   .exit                   ; the memory list and
                cmp.l   (nspml_NextBlock,a0),a1 ; return FAILURE if it
                beq.b   .FoundBlock             ; isn't found.
                move.l  (nspml_NextBlock,a0),a0
                bra.b   ...

.FoundBlock     move.l  (nspml_NextBlock,a1),(nspml_NextBlock,a0)
                tst.l   (nspml_Pooled,a1)
                bne.b   .FreePool
                move.l  (nspml_Size,a1),d0      ; Determine size and
                jsr     (_LVOFreeMem,a6)        ; free it.
                moveq   #-1,d0
                bra.b   .exit

.FreePool       bsr     _LockAnyMemPool
                move.l  (nspml_Pooled,a1),a0
                move.l  (nspml_Size,a1),d0
                bsr     (_AsmFreePooled)
                bsr     _FreeAnyMemPool
                moveq   #-1,d0

.exit           POPM
                rts


;***************************************************************************
;*                                                                         *
;* SEKTION: Datas                                                          *
;*                                                                         *
;***************************************************************************

_RCSId          CSTR    "$Id: MemoryHandling.asm,v 1.14 1994/05/24 21:12:09 simons Exp simons $"

                END
