*
*       $Filename: NetSupportLibrary.asm $
*       $Revision: 1.27 $
*       $Date: 1994/10/25 13:32:45 $
*
*       Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
*
*       This program is free software; you can redistribute it and/or
*       modify it under the terms of the GNU General Public License as
*       published by the Free Software Foundation; either version 2 of
*       the License, or (at your option) any later version.
*
*       This program is distributed in the hope that it will be useful,
*       but WITHOUT ANY WARRANTY; without even the implied warranty of
*       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*       General Public License for more details.
*
*       You should have received a copy of the GNU General Public License
*       along with this program; if not, write to the Free Software
*       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*
*       $Id: NetSupportLibrary.asm,v 1.27 1994/10/25 13:32:45 simons Exp simons $
*

;***************************************************************************
;*                                                                         *
;* SEKTION: Labels, Macros, Switches, Structures                           *
;*                                                                         *
;***************************************************************************

;-------------------------------------- Switches -----------
                MC68000
                INCDIR  "INCLUDE:"
                OBJFILE "NetSupportLibrary.o"
                NEWSYNTAX
                MULTIPASS
;               VERBOSEOPTIM
                DEBUG on
                SYM

;-------------------------------------- Labels ------------
                MACLIB  "lib:Offsets.mac"
                MACFILE "exec/exec.i"

                MACFILE "libraries/netsupport.i"
                MACFILE "MemoryHandling.i"
                MACFILE "TmpFiles.i"
                MACFILE "ConfigFiles.i"
                MACFILE "LogFiles.i"
                MACFILE "LockFiles.i"
                MACFILE "POpen.i"
                MACFILE "Residents.i"
                MACFILE "NetSupportLibrary_rev.i"

MY_PRI          equ     0
LOG_BUFFERSIZE  equ     4*1024
PUDDLE_SIZE     equ     20*1024
THRESH_SIZE     equ     10*1024

;-------------------------------------- Entry-Points ------
                XDEF    _NetSupportBase
                XDEF    _StuffChar
                XDEF    _LockAnyMemPool
                XDEF    _FreeAnyMemPool

;-------------------------------------- References --------
                XREF    _AsmCreatePool
                XREF    _AsmDeletePool

;-------------------------------------- Macros ------------


;***************************************************************************
;*                                                                         *
;* SEKTION: Executable part                                                *
;*                                                                         *
;***************************************************************************

                lea     (NetSupportName,PC),a1
                move.l  ($4).w,a6
                jsr     (_LVOOldOpenLibrary,a6)
                tst.l   d0
                beq.b   .exit
                move.l  d0,a5
                move.l  (nsp_DOSBase,a5),a6
                jsr     (_LVOOutput,a6)
                move.l  d0,d1
                beq.b   .CloseNetSupportLib
                lea     (LibraryID,PC),a0
                move.l  a0,d2
                moveq   #LibID_len,d3
                jsr     (_LVOWrite,a6)

.CloseNetSupportLib:
                move.l  a5,a1
                move.l  ($4).w,a6
                jsr     (_LVOCloseLibrary,a6)
.exit           moveq   #0,d0
                rts


;***************************************************************************
;*                                                                         *
;* SEKTION: Library-Header                                                 *
;*                                                                         *
;***************************************************************************

ResidentHeader  dc.w    RTC_MATCHWORD
                dc.l    ResidentHeader
                dc.l    EndOfCode
                dc.b    RTF_AUTOINIT
                dc.b    VERSION
                dc.b    NT_LIBRARY
                dc.b    MY_PRI
                dc.l    NetSupportName
                dc.l    LibraryID
                dc.l    Init

Init            dc.l    nsp_SIZEOF
                dc.l    FunctionTable
                dc.l    DataTable
                dc.l    _InitRoutine

DataTable       INITBYTE LN_TYPE,NT_LIBRARY
                INITLONG LN_NAME,NetSupportName
                INITBYTE LIB_FLAGS,LIBF_SUMUSED|LIBF_CHANGED
                INITWORD LIB_VERSION,VERSION
                INITWORD LIB_REVISION,REVISION
                INITLONG LIB_IDSTRING,LibraryID
                dc.l     0


;------------------------- Table of library functions ---------------------

FunctionTable   dc.l    _Open                   ; standard libraryfunctions
                dc.l    _Close
                dc.l    _Expunge
                dc.l    _Null

                dc.l    _AllocMemPooled
                dc.l    _FreeMemPooled
                dc.l    _GetStaticBuffer

                dc.l    _GetConfigEntry
                dc.l    _MakeLogEntryStub
                dc.l    _GetConfig

                dc.l    _LockFile
                dc.l    _UnLockFile
                dc.l    _UnLockFiles
                dc.l    _LockFileAttempt
                dc.l    _IsFileLocked

                dc.l    _SetConfigEntry

                dc.l    _TempName
                dc.l    _TempNameT
                dc.l    _GetSeq

                dc.l    _POpen
                dc.l    _PClose

                dc.l    _LoadResident
                dc.l    _ExpungeResident

                dc.l    _GetNSPConfigName

                dc.l    -1                      ; end of table


;***************************************************************************
;*                                                                         *
;* SEKTION: Library skeleton                                               *
;*                                                                         *
;***************************************************************************

_InitRoutine:
                ; This routine is called by the OS, when the library is
                ; opened the first time. It initializes the library base
                ; and opens required resources.
                ;
                ; Parameter: A0=Seglist A6=ExecBase D0=LibraryBase
                ; Result   : D0=LibraryBase
                ; Comment  : Fails under OS 36 or earlier.

                PUSHM   d1-d7/a0-a6
                move.l  d0,a1
                move.l  a0,(nsp_SegList,a1)
                move.l  a1,(_NetSupportBase)


                move.l  a1,a0
                bsr     _InitLibraryBase        ; returns either libbase
                                                ; or zero!
.exit           POPM
                rts


_Open:
                ; The OS jumps into this routine every time, a process
                ; opens our library. Therefore, we have to increase the
                ; user-counter and to delete the "delayed expunge" bit.
                ;
                ; Parameter: A6=LibraryBase
                ; Result   : D0=LibraryBase

                addq.w  #1,(LIB_OPENCNT,a6)
                bclr.b  #LIBB_DELEXP,(LIB_FLAGS,a6)
                move.l  a6,d0
                rts

_Close:
                ; This routine tries to close the library. When compiled
                ; in betamode, the library removes itself if no user-
                ; task is left.
                ;
                ; Parameter: A6=&LibraryBase
                ; Result   : D0=Null

                bsr     _FreeStaticBuffer

                subq.w  #1,(LIB_OPENCNT,a6)
                bne.b   .Close

        IFD SELFEXPUNGE
                bra.b   _Expunge                ; remove immediatly
        ELSE
                btst.b  #LIBB_DELEXP,(LIB_FLAGS,a6)
                bne.b   _Expunge
        ENDC

.Close          moveq   #0,d0
                rts


_Expunge:
                ; Removes the library from the system. This routine is
                ; called in low-memory situations.
                ;
                ; Parameter: A6=LibraryBase
                ; Result   : D0=SegList

                PUSHM   d1-d7/a0-a6

                bsr     _ExpungeResident        ; free all unneeded buffers

                tst.w   (LIB_OPENCNT,a6)
                beq.b   .expunge
                bset.b  #LIBB_DELEXP,(LIB_FLAGS,a6)
                bra.b   .exit

.expunge:       move.l  a6,a0
                bsr     _UnInitLibraryBase

                move.l  (nsp_SegList,a6),d2
                move.l  ($4).w,a5
                exg.l   a5,a6

                move.l  a5,a1
                jsr     (_LVORemove,a6)

                moveq   #0,d0
                move.l  a5,a1
                move.w  (LIB_NEGSIZE,a5),d0
                sub.w   d0,a1
                add.w   (LIB_POSSIZE,a5),d0
                jsr     (_LVOFreeMem,a6)
                move.l  d2,d0

.exit           POPM
                rts

_Null:
                ; Not called in the current release of the OS.
                ;
                ; Parameter: none
                ; Result   : D0=Null

                moveq   #0,d0
                rts


;***************************************************************************
;*                                                                         *
;* SEKTION: Internal Library functions                                     *
;*                                                                         *
;***************************************************************************

_InitLibraryBase:
                ; Initialize the LibraryBase
                ;
                ; Parameter: A0=LibraryBase
                ; Result   : D0=LibraryBase / NULL

                PUSHM   d1-d7/a0-a6
                move.l  a0,a5

                moveq   #37,d0
                lea     (dosname,PC),a1
                move.l  ($4).w,a6
                jsr     (_LVOOpenLibrary,a6)
                move.l  d0,(nsp_DOSBase,a5)
                beq     .ErrorExit

                moveq   #37,d0
                lea     (utilname,PC),a1
                jsr     (_LVOOpenLibrary,a6)
                move.l  d0,(nsp_UtilityBase,a5)
                beq     .ErrorExit

                lea     (StaticBufferSemaphore,PC),a0
                move.l  a0,(nsp_StaticBufferSemaphore,a5)
                jsr     (_LVOInitSemaphore,a6)

                lea     (AnyMemPoolSemaphore,PC),a0
                move.l  a0,(nsp_AnyMemPoolSemaphore,a5)
                jsr     (_LVOInitSemaphore,a6)

                lea     (ResidentListSemaphore,PC),a0
                move.l  a0,(nsp_ResidentListSemaphore,a5)
                jsr     (_LVOInitSemaphore,a6)

                lea     (MsgPortListSemaphore,PC),a0
                move.l  a0,(nsp_MsgPortListSemaphore,a5)
                jsr     (_LVOInitSemaphore,a6)

                clr.l   (nsp_StaticBufferList,a5)
                clr.l   (nsp_UniqueNumber,a5)

                lea     (nsp_ResModules,a5),a0
        OLDSYNTAX
                NEWLIST a0
        NEWSYNTAX
                lea.l   (nsp_MsgPortList,a5),a0
        OLDSYNTAX
                NEWLIST a0
        NEWSYNTAX

                moveq   #MEMF_ANY,d0
                move.l  #PUDDLE_SIZE,d1
                move.l  #THRESH_SIZE,d2
                jsr     (_AsmCreatePool)
                move.l  d0,(nsp_AnyMemPool,a5)
                beq.b   .ErrorExit

                move.l  a5,d0

.exit           POPM
                rts

.ErrorExit      bsr     _UnInitLibraryBase
                moveq   #0,d0
                bra.b   .exit


_UnInitLibraryBase:
                ; Free the resources, allocated for the LibraryBase
                ;
                ; Parameter: A0=LibraryBase
                ; Result   : none

                PUSHM   d0-d7/a0-a6
                move.l  ($4).w,a6
                move.l  a0,a5

                move.l  (nsp_AnyMemPool,a5),d0
                beq.b   .CloseUtilLib
                move.l  d0,a0
                jsr     (_AsmDeletePool)

.CloseUtilLib   tst.l   (nsp_UtilityBase,a5)
                beq.b   .CloseDosLib
                move.l  (nsp_UtilityBase,a5),a1
                jsr     (_LVOCloseLibrary,a6)

.CloseDosLib    tst.l   (nsp_DOSBase,a5)
                beq.b   .exit
                move.l  (nsp_DOSBase,a5),a1
                move.l  ($4).w,a6
                jsr     (_LVOCloseLibrary,a6)

.exit           POPM
                rts


_MakeLogEntryStub:
                ; Buids the log-entry and calls the real logging routine.
                ; This is necesarry, because SAS/C can't handle varags
                ; routines with registered parameters.
                ;
                ; Parameter: A1=&Logmessage A2=&Parameters[]
                ;            D0=&Logentry
                ; Result   : none
                ; Comment  : works with a fixed buffer size

                PUSHM   d0/a0-a3/a6

                move.l  a1,a3
                move.l  #LOG_BUFFERSIZE,d0
                moveq   #0,d1
                bsr     _NSPAllocMemPooled
                tst.l   d0
                beq.b   .error
                move.l  a3,a1

                move.l  d0,a3
                move.l  a1,a0
                move.l  a2,a1
                lea     (_StuffChar,PC),a2
                move.l  ($4).w,a6
                jsr     (_LVORawDoFmt,a6)

                move.l  a3,d1
                POPM
                move.l  d1,a1
                bra     _MakeLogEntry

.error          POPM
                rts


;****i* NetSupportLibrary.asm/StuffChar ***********************************
;
;    NAME
;       StuffChar -- routine to write one character into a buffer
;
;
;    SYNOPSIS
;       StuffChar(character, buffer);
;       D0        D0.b       A3
;
;       UBYTE   character;
;       STRPTR  buffer;
;
;    FUNCTION
;       This is a small assembler routine (two lines, in fact!) that is
;       suited to be called by RawDoFmt() and writes the provided
;       character into a buffer.
;
;    INPUTS
;       character = a simple byte value
;       buffer = pointer to a text buffer
;
;    RESULT
;       none
;
;    NOTE
;       StuffChar() increases the pointer to the buffer by one!! This
;       behavior is required by RawDoFmt().
;
;    SEE ALSO
;       RawDoFmt()
;
;**************************************************************************

_StuffChar      move.b  d0,(a3)+
                rts


;****i* NetSupportLibrary.asm/LockPoolList ********************************
;
;    NAME
;       LockAnyMemPool -- easy way of obtaining the AnyMemPoolSemaphore
;
;
;    SYNOPSIS
;       LockAnyMemPool();
;
;    FUNCTION
;       Tries to obtain the AnyMemPoolSemaphore. If someone else is using
;       it at the moment, the task is delayed until it becomes free. This
;       is vital for modifying linked lists.
;
;    INPUTS
;       none
;
;    RESULT
;       none
;
;    NOTE
;       All registers are preserved.
;
;    SEE ALSO
;       FreeAnyMemPool()
;
;**************************************************************************

_LockAnyMemPool
                PUSHM   d0-d1/a0-a1/a6
                move.l  (_NetSupportBase,PC),a0
                move.l  (nsp_AnyMemPoolSemaphore,a0),a0
                move.l  ($4).W,a6
                jsr     (_LVOObtainSemaphore,a6)
                POPM
                rts


;****i* NetSupportLibrary.asm/FreePoolList ********************************
;
;    NAME
;       FreeAnyMemPool -- easy way of releasing the AnyMemPoolSemaphore
;
;
;    SYNOPSIS
;       FreeAnyMemPool();
;
;    FUNCTION
;       Releases the AnyMemPoolSemaphore for usage by other tasks.
;
;    INPUTS
;       none
;
;    RESULT
;       none
;
;    NOTE
;       All registers are preserved.
;
;    SEE ALSO
;       LockAnyMemPool()
;
;**************************************************************************

_FreeAnyMemPool
                PUSHM   d0-d1/a0-a1/a6
                move.l  (_NetSupportBase,PC),a0
                move.l  (nsp_AnyMemPoolSemaphore,a0),a0
                move.l  ($4).W,a6
                jsr     (_LVOReleaseSemaphore,a6)
                POPM
                rts


;***************************************************************************
;*                                                                         *
;* SEKTION: Datas, Pointers                                                *
;*                                                                         *
;***************************************************************************

                CNOP    0,4
_NetSupportBase ds.l    1

StaticBufferSemaphore   ds.b SS_SIZE            ; signal semaphores to protect
AnyMemPoolSemaphore     ds.b SS_SIZE            ; linked lists from several accesses
ResidentListSemaphore   ds.b SS_SIZE            ; at the same time
MsgPortListSemaphore    ds.b SS_SIZE

NetSupportName  NETSUPPORTNAME
dosname         CSTR    "dos.library"
utilname        CSTR    "utility.library"
RCSId           CSTR    "$Id: NetSupportLibrary.asm,v 1.27 1994/10/25 13:32:45 simons Exp simons $"
                dc.b    0,"$VER: "      ; Kludge for 2.04 version
LibraryID       VSTRING
LibID_len       equ     (*-LibraryID)

EndOfCode:      END

