/*
 *      $Filename: Residents.c $
 *      $Revision: 1.7 $
 *      $Date: 1994/05/24 21:12:09 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *      $Id: Residents.c,v 1.7 1994/05/24 21:12:09 simons Exp simons $
 *
 */


/**************************************************************************
 *                                                                        *
 * SEKTION: Labels, Macros, Switches, Structures                          *
 *                                                                        *
 **************************************************************************/

/************************************* Includes ***********/
#include <exec/lists.h>
#include <exec/memory.h>
#include <dos/dos.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "libraries/netsupport.h"
#include "NetSupportLibrary.h"
#include "MemoryHandling.h"
#include "SupportRoutines.h"

/************************************* Defines ************/
struct NSPResModule {
        struct Node nsprm_node;
        struct DateStamp nsprm_datestamp;
        ULONG nsprm_filelength;
        STRPTR nsprm_filebuffer;
};

/************************************* Prototypes *********/

static const char __RCSId[] = "$Id: Residents.c,v 1.7 1994/05/24 21:12:09 simons Exp simons $";


/**************************************************************************
 *                                                                        *
 * SEKTION: Internal library functions                                    *
 *                                                                        *
 **************************************************************************/

/****** netsupport.library/LoadResident ***********************************
*
*    NAME
*       LoadResident -- loads a file and stores it resident, to buffer
*               further access
*
*    SYNOPSIS
*       buffer = LoadResident(filename)
*       D0                    A0
*
*       STRPTR  buffer;
*       STRPTR  filename;
*
*    FUNCTION
*       This routine loads the specified file and stores it in an
*       internal resident list. The next time this file is accessed,
*       the buffer will be returned immediately without disk access.
*
*       The routine tests wether the file on disk is newer than the
*       resident version and re-loads it in case it is.
*
*    INPUTS
*       filename - name and path of the file that should be loaded
*
*    RESULT
*       If the file couldn't be loaded for any reason, NULL is returned.
*       Otherwise the routine returns a pointer to the buffer holding the
*       file. The length of the file is returned by IoErr(). You must not
*       modify or free the returned buffer!
*
*    NOTE
*       The routine does NOT lock the file using LockFile()!
*       The loaded file is always null terminated and thus can be used
*       with string routines.
*
*    SEE ALSO
*       ExpungeResident()
*
***************************************************************************
*
*/

STRPTR __asm LoadResident(register __a0 STRPTR filename)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        struct FileInfoBlock __aligned fib;
        struct NSPResModule *resmodule = NULL;
        struct MinList *list = &(NetSupportBase->nsp_ResModules);
        BPTR lock, file;
        STRPTR name, buffer = NULL;

        if (!(name = AllocVec(256, 0L)))
                return NULL;

        ObtainSemaphore(NetSupportBase->nsp_ResidentListSemaphore);     /* lock list */

        if (lock = Lock(filename, ACCESS_READ)) {
                NameFromLock(lock, name, 256);
                if (Examine(lock, &fib)) {
                        if (resmodule = (struct NSPResModule *) FindName((struct List *) list, name)) {
                                FreeVec(name);
                                name = resmodule->nsprm_node.ln_Name;
                                if (!CompareDates(&(resmodule->nsprm_datestamp), &(fib.fib_Date)))
                                        buffer = resmodule->nsprm_filebuffer;
                                else {
                                        FreeVec(resmodule->nsprm_filebuffer);
                                        CopyMem((APTR) &(fib.fib_Date), (APTR) &(resmodule->nsprm_datestamp), sizeof(struct DateStamp));
                                        resmodule->nsprm_filelength = fib.fib_Size;
                                        if (buffer = AllocVec(fib.fib_Size+1, MEMF_ANY)) {
                                                resmodule->nsprm_filebuffer = buffer;
                                                if (file = OpenFromLock(lock)) {
                                                        lock = NULL;
                                                        Read(file, buffer, fib.fib_Size);
                                                        Close(file);
                                                        buffer[fib.fib_Size] = '\0';
                                                }
                                                else {
                                                        FreeVec(buffer);
                                                        buffer = NULL;
                                                }
                                        }
                                        if (!buffer)
                                                Remove((struct Node *) resmodule);
                                }
                        }
                        else {
                                if (resmodule = AllocVec(sizeof(struct NSPResModule), MEMF_ANY|MEMF_CLEAR)) {
                                        CopyMem((APTR) &(fib.fib_Date), (APTR) &(resmodule->nsprm_datestamp), sizeof(struct DateStamp));
                                        resmodule->nsprm_node.ln_Name = name;
                                        resmodule->nsprm_filelength = fib.fib_Size;
                                        if (buffer = AllocVec(fib.fib_Size+1, MEMF_ANY)) {
                                                if (file = OpenFromLock(lock)) {
                                                        lock = NULL;
                                                        Read(file, buffer, fib.fib_Size);
                                                        Close(file);
                                                        buffer[fib.fib_Size] = '\0';
                                                }
                                                else {
                                                        FreeVec(buffer);
                                                        buffer = NULL;
                                                }
                                                resmodule->nsprm_filebuffer = buffer;
                                                AddHead((struct List *) list, (struct Node *) resmodule);
                                        }
                                }
                        }
                }
                if (lock)
                        UnLock(lock);
        }

        ReleaseSemaphore(NetSupportBase->nsp_ResidentListSemaphore);     /* release list */

        if (buffer) {
                SetIoErr(fib.fib_Size);
                return buffer;
        }
        else {
                if (name)       FreeVec(name);
                if (resmodule)  FreeVec(resmodule);
                return NULL;
        }
}



/****** netsupport.library/ExpungeResident ********************************
*
*    NAME
*       ExpungeResident -- expunges all resident files and thus frees
*               memory
*
*    SYNOPSIS
*       ExpungeResident(void);
*
*    FUNCTION
*       This routine frees all allocated buffers and expunges all loaded
*       resident modules. This routine is usually called by the OS when
*       memory is lacking and should not be used by other programs.
*
*    INPUTS
*       none
*
*    RESULT
*       none
*
*    SEE ALSO
*       LoadResident()
*
***************************************************************************
*
*/

void __asm ExpungeResident(void)
{
        struct NSPResModule *resmodule;

        while (resmodule = (struct NSPResModule *) RemHead((struct List *) &(NetSupportBase->nsp_ResModules))) {
                FreeVec(resmodule->nsprm_filebuffer);
                FreeVec(resmodule->nsprm_node.ln_Name);
                FreeVec(resmodule);
        }
}

