/*
 *      $Filename: SupportRoutines.c $
 *      $Revision: 1.11 $
 *      $Date: 1994/05/24 21:12:09 $
 *
 *      Copyright (C) 1993,94 by Peter Simons <simons@peti.GUN.de>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License as
 *      published by the Free Software Foundation; either version 2 of
 *      the License, or (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *      General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *      $Id: SupportRoutines.c,v 1.11 1994/05/24 21:12:09 simons Exp simons $
 *
 */

/**************************************************************************
 *                                                                        *
 * SEKTION: Labels, Macros, Switches, Structures                          *
 *                                                                        *
 **************************************************************************/

/************************************* Includes ***********/
#include <stdarg.h>
#include <dos/dos.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include "libraries/netsupport.h"
#include "NetSupportLibrary.h"
#include "MemoryHandling.h"

/************************************* Defines ************/

/************************************* Prototypes *********/
static const char __RCSId[] = "$Id: SupportRoutines.c,v 1.11 1994/05/24 21:12:09 simons Exp simons $";


/**************************************************************************
 *                                                                        *
 * SEKTION: Internal library functions                                    *
 *                                                                        *
 **************************************************************************/


/****i* SupportRoutines.c/LoadFile ****************************************
*
*   NAME
*       LoadFile -- loads a file into a buffer
*
*   SYNOPSIS
*       buffer = LoadFile(filename);
*       D0                A0
*
*       STRPTR buffer;
*       STRPTR filename;
*
*   FUNCTION
*       LoadFile() -- determines the length of a given file, allocates
*       the appropriate buffer and actually loads the file.
*
*   INPUTS
*       filename - pointer to a zero-terminated name string. The string
*                       may contain an relative or absolute path.
*
*   RESULT
*       If everything works, the address of the buffer is returned and
*       the length of the file is available via IoErr().
*
*       The buffer should be free after usage using NSPFreeMemPooled(),
*       however the memory allocation tracking mechanism will take
*       care if you forget to free it.
*
*   NOTES
*       A zero-byte is appended to the loaded file, to enable the
*       usage of standard C string routines on the contents.
*
*   SEE ALSO
*
***************************************************************************
*
*/


STRPTR __asm LoadFile(register __a0 STRPTR filename)
{
        struct DosLibrary *DOSBase = NetSupportBase->nsp_DOSBase;
        BPTR fh, fh2;
        struct FileInfoBlock *fib;
        STRPTR memblock, filebuffer = NULL;
        BOOL isfile = FALSE;
        int readlen;

        if (fh = Lock(filename, ACCESS_READ)) {
                if (fib = NSPAllocMemPooled(sizeof(struct FileInfoBlock), 0L)) {
                        if (Examine(fh, fib)) {
                                if (fh2 = OpenFromLock(fh)) {
                                        isfile = TRUE;
                                        if (memblock = NSPAllocMemPooled(fib->fib_Size + 1, 0L)) {
                                                if ((readlen = Read(fh2, memblock, fib->fib_Size)) != -1L) {
                                                        memblock[readlen] = '\0';
                                                        filebuffer = memblock;
                                                        SetIoErr(readlen);
                                                }
                                                else
                                                        NSPFreeMemPooled(memblock);
                                        }
                                        Close(fh2);
                                }
                        }
                        NSPFreeMemPooled(fib);
                }
                if (!(isfile))
                        UnLock(fh);
        }

        return filebuffer;
}



/****i* SupportRoutines.c/StringCopy **************************************
*
*   NAME
*       StringCopy -- copies a zero-terminated string into a buffer
*
*   SYNOPSIS
*       end_of_string = StringCopy(buffer, string);
*       D0                         A0      A1
*
*       STRPTR end_of_string;
*       STRPTR buffer;
*       STRPTR string;
*
*   FUNCTION
*       This routine does the same as the ANSI-C strcpy() routine, but
*       this version is more suitable for a shared library as it doesn't
*       require any link libraries, etc...
*
*   INPUTS
*       buffer - Pointer to a textbuffer large enough to hold the
*                       string. No buffer overflow-checking is performed.
*
*       string - Pointer to a zero-terminated string to be copied.
*
*   RESULT
*       The resulting pointer points to the zero-byte of the copied string
*       in the destination pointer. Using this pointer, it is possible to
*       append several strings easily. See example.
*
*   EXAMPLE
*       Append two strings in a target buffer using StringCopy():
*
*               STRPTR buffer[256];
*               const char part1[] = "These two strings ";
*               const char part2[] = "belong together!";
*
*               StringCopy(StringCopy(buffer, part1), part2);
*               Printf("%s\n", buffer);
*
*   SEE ALSO
*
***************************************************************************
*
*/

STRPTR __asm StringCopy(register __a0 STRPTR to, register __a1 STRPTR from)
{
        while((*(to++) = *(from++)) != 0)
                ;

        return to-1;
}


/****i* SupportRoutines.c/StringCat ***************************************
*
*   NAME
*       StringCat -- append one string to the end of another one
*
*   SYNOPSIS
*       end_of_string = StringCat(part1, part2);
*       D0                        A0     A1
*
*       STRPTR end_of_string, part1, part2;
*
*   FUNCTION
*       This routine does the same as the ANSI-C strcat() routine, but
*       this version is more suitable for a shared library as it doesn't
*       require any link libraries, etc...
*
*   INPUTS
*       part1 - Pointer to a textbuffer large enough to hold both
*                       strings. No buffer overflow-checking is performed.
*
*       part2 - Pointer to a zero-terminated string to be appended at the
*                       end of >part1<.
*
*   RESULT
*       The resulting pointer points to the zero-byte of the copied string
*       in the destination pointer. Using this pointer, it is possible to
*       append several strings easily. See StringCopy() for a code example.
*
*   SEE ALSO
*
***************************************************************************
*
*/

STRPTR __asm StringCat(register __a0 STRPTR to, register __a1 STRPTR from)
{
        while(*to)
                to++;

        return StringCopy(to, from);
}


/****i* SupportRoutines.c/SPrint ******************************************
*
*   NAME
*       SPrint -- format data into a character stream.
*
*   SYNOPSIS
*       SPrint(buffer, fmt, ...);
*
*       STRPTR buffer;
*       STRPTR fmt;
*
*   FUNCTION
*       Perform "C"-language-like formatting of a data stream, outputting
*       the result a character at a time.  Where % formatting commands are
*       found in the FormatString, they will be replaced with the
*       corresponding element in the DataStream.  %% must be used in the
*       string if a % is desired in the output.
*
*   INPUTS
*       buffer -- Pointer to a buffer large enough enough to hold the
*               formatted string.
*
*       fmt -- Control string describing how the string and the parameters
*               shall be formatted. Please read the autodoc of RawDoFmt()
*               for further information.
*
*   RESULT
*       none
*
*   NOTES
*       This routine expects it's parameters on the stack. No
*       registerized version is available.
*
*   SEE ALSO
*       RawDoFmt()
*
***************************************************************************
*
*/

void SPrint(STRPTR buffer, STRPTR *fmt, ...)
{
        va_list ap;
        va_start(ap, fmt);
        RawDoFmt((UBYTE *) fmt, ap, StuffChar, buffer);
}

