;-------------------------------------- Switches -----------
                MC68000
                NEWSYNTAX
                OBJFILE "PTMFast.o"
                MULTIPASS
;               VERBOSEOPTIM


;-------------------------------------- Entry-Points -------
                XDEF    _PatternTest
                XDEF    _ParsePatternFast
                XDEF    _PatternMatchFast
                XDEF    _ParsePatternFastArray
                XDEF    _PatternMatchFastArray

;-------------------------------------- Macros -------------
PUSHM                           MACRO
                                IFGT    NARG-1
                                FAIL    !!!! TOO MANY ARGUMENTS TO PUSHM !!!!
                                ENDC
saveregs                        SETRL   \1
                                movem.l saveregs,-(sp)
                                ENDM
POPM                            MACRO
                                movem.l (sp)+,saveregs
                                ENDM


;----------------------------- PatternMatching Faaast ---------------------
_PatternTest:
                ; Funktion : Testet ein Pattern und gibt dessen Status zurck
                ;
                ; Parameter: A0=&Pattern D0=maximale Lnge+1
                ; Result   : D0=Status
                ; Kommentar:  0=Pattern okay
                ;             1=Or-Verknpfung okay
                ;            -1=Or-Verknpfung fehlerhaft
                ;            -2=Pattern zu lang

                move.l  d0,d1
                beq.b   .SkipLengthTest
                bsr     _Len
                cmp.l   d0,d1
                bmi.b   .LengthError
.SkipLengthTest cmp.b   #'(',(a0)
                bne.b   .PatternOkay
                cmp.b   #')',(-1,a0,d0.l)
                bne.b   .ArrayError
.ArrayOkay      moveq   #1,d0
                rts
.PatternOkay    moveq   #0,d0
                rts
.ArrayError     moveq   #-1,d0
                rts
.LengthError    moveq   #-2,d0
                rts



_PatternMatchFastArray:
                ; Funktion : Testet ob ein String auf ein Pattern zutrift
                ;
                ; Parameter: A0=&Text A1=&Orbuffer
                ; Result   : D0=0 (FALSE) / -1 (TRUE)
                ; Kommentar: Pattern mu erst mit _ParsePatternFastArray
                ;            behandelt werden!

                PUSHM   a2-a3
                moveq   #0,d0
                move.l  a1,a2
                move.l  a0,a3
.loop           tst.l   (a2)
                beq.b   .finished
                move.l  a3,a0
                move.l  (a2)+,a1
                bsr     _PatternMatchFast
                tst.l   d0
                beq.b   .loop
.finished       POPM
                rts


_ParsePatternFastArray:
                ; Funktion : Wandelt ein Pattern in das interne Format um
                ; Parameter: A0=&Pattern A1=&Patternbuffer A2=&Orbuffer
                ; Result   : keins
                ; Kommentar: Erkennt zustzlich noch () und | Operatoren

                PUSHM   a2-a5
                cmp.b   #'(',(a0)
                beq.b   .ParseArray
                move.l  a1,a2
                lea     (8,a2),a1
                move.l  a1,(a2)+
                clr.l   (a2)
                bsr     _ParsePatternFast
                bra.b   .exit

.ParseArray     addq.w  #1,a0
                link.w  a5,#-255
                lea     (-255,a5),a3
                cmp.b   #')',(a0)
                beq.b   .finished
.CopyLoop       move.b  (a0)+,(a3)+
                bne.b   .CopyLoop

                lea     (-255,a5),a3
                move.l  a2,a4
                move.l  a3,(a2)+
.PreProcLoop    cmp.b   #'|',(a3)
                beq.b   .FoundOr
                tst.b   (a3)+
                bne.b   .PreProcLoop
                clr.b   (-2,a3)
                bra.b   .GoOn
.FoundOr        clr.b   (a3)+
                move.l  a3,(a2)+
                bra.b   .PreProcLoop
.GoOn           clr.l   (a2)
                move.l  a1,a3

.ParseLoop      move.l  (a4),a0
                move.l  a3,(a4)+
                move.l  a3,a1
                bsr     _ParsePatternFast
                tst.l   (a4)
                beq.b   .finished
.FindEOFPattern tst.b   (a3)+
                bne.b   .FindEOFPattern
                cmp.b   #-1,(a3)
                bne.b   .FindEOFPattern
                addq.w  #1,a3
                bra.b   .ParseLoop

.finished       unlk    a5
.exit           POPM
                rts



_PatternMatchFast:
                ; Testet ob ein String auf ein Pattern zutrift
                ;
                ; Parameter: A0=&Text A1=&Patternbuffer
                ; Result   : D0=0 (FALSE) / -1 (TRUE)
                ; Kommentar: Pattern mu erst mit _ParsePatternFast
                ;            behandelt werden!

                move.l  d7,-(SP)
                addq.w  #1,a1                   ; Pad-Byte berspringen
                exg.l   a0,a1
                moveq   #0,d7                   ; D7=Result (DEFAULT=FALSE)
                tst.b   (a0)
                bne.b   .GoOn                   ; Patternlnge = 0 --> Fehler!
                cmp.b   #-1,(1,a0)
                beq.b   .finished
.GoOn           tst.b   (a1)
                beq.b   .finished               ; Textlnge    = 0 --> Fehler!

                tst.b   (a0)                    ; Erste Zeichen ein Pattern?
                beq.b   .StartPatternMatch      ; Ja  : In die Hauptschleife

                bsr     _Len                    ; Patternlnge berechnen
                move.l  d0,d1
                bsr     _PMCmpStrings           ; Stimmen die ersten Zeichen
                tst.l   d0                      ; des Patterns mit dem Text
                beq.b   .finished               ; berein?
                lea     (1,a0,d1.l),a0          ; Nein: --> Ende
                lea     (1,a1,d1.l),a1
                cmp.b   #-1,(a0)                ; Wenn dies das einzige Pattern
                bne.b   .PatternMatch           ; war, mu der Text ebenfalls
                tst.b   (-1,a1)                 ; zu Ende sein!
                beq.b   .PatternMatchEnd
                bra.b   .finished

.StartPatternMatch:
                addq.w  #1,a0                   ; Null berspringen

.PatternMatch   bsr     _Len
                cmp.b   #-1,(1,a0,d0.l)         ; letztes Pattern??
                beq.b   .MatchLastPattern
                move.l  d0,d1
                bsr.b   _PMFind
                tst.l   d0
                beq.b   .finished
                lea     (1,a0,d1.l),a0
                lea     (1,a1,d1.l),a1
                bra.b   .PatternMatch

.MatchLastPattern:
                tst.b   (a0)                    ; letzte Zeichen war ein Joker?
                beq.b   .PatternMatchEnd        ; Ja  : --> Fertig!
                move.l  d0,d1

                exg.l   a0,a1
                bsr     _Len                    ; Ende vom Text finden
                exg.l   a0,a1
                tst.l   d0
                beq.b   .finished
                lea     (a1,d0.l),a1
                sub.l   d1,a1

                move.l  d1,d0
                bsr     _PMCmpStrings           ; Stimmen die letzten Zeichen
                tst.l   d0                      ; des Patterns mit dem Text
                beq.b   .finished               ; berein?

.PatternMatchEnd:
                moveq   #-1,d7
.finished       move.l  d7,d0                   ; Rckgabewert holen
                move.l  (SP)+,d7
                rts                             ; Bye, bye!


_ParsePatternFast:
                ; Wandelt ein Pattern ins interne Format
                ;
                ; Parameter: A0=&Pattern A1=&Patternbuffer
                ; Result   : keine
                ; Kommentar: Routine erkennt nur "#?" und "?" als Patterns,
                ;            da sie in erster Linie schnell sein soll!! Dem
                ;            Patternpuffer mu ein Byte <> 0 vorrangehen!!

                move.b  #-1,(a1)+
.SplitLoop      cmp.b   #'#',(a0)       ; aktuelles Zeichen ein "#" ??
                beq.b   .FoundPattern   ; Ja: Verzweigen
.CopyChar       move.b  (a0)+,(a1)+     ; Nein: Zeichen kopieren
                bne.b   .SplitLoop      ; String zu Ende
                bra.b   .SplitEnd       ; Nchstes Zeichen testen

.FoundQM        move.b  d0,(a1)+        ; Fragezeichen schreiben
                bra.b   .SplitLoop


.FoundPattern   cmp.b   #'?',(1,a0)     ; Ist das folgende Zeichen ein "?"
                bne.b   .CopyChar       ; Nein: aktuelles Zeichen kopieren
                tst.b   (-1,a1)         ; Ja  : Pattern gefunden und Endmar-
                beq.b   .SkipPattern    ;       kierung im Array schreiben
                clr.b   (a1)+           ; Vorsicht: Zwei Joker nacheinander
.SkipPattern    addq.w  #2,a0           ;           werden bersprungen
.SkipQM         cmp.b   #'?',(a0)       ; Fragezeichen nach Jokern lschen
                bne.b   .SplitLoop
                addq.w  #1,a0
                bra.b   .SkipQM

.SplitEnd       move.b  #-1,(a1)        ; Endmarkierung: 0.b,-1.b
                rts



;------------------------------ Support-Routines --------------------------
_Len:
                ; Funktion : Bestimmt die Lnge eines mit Null
                ;            abgeschlossenen Textes
                ;
                ; Parameter: A0=&Textpuffer
                ; Result   : D0=Lnge ohne das Nullbytes
                ; Kommentar: Verndert keine Register ausser D0!!!

                pea     (a0)            ; Register retten
.loop           tst.b   (a0)+
                bne.b   .loop           ; aktuelles Zeichen gleich Null?
                move.l  a0,d0
                move.l  (SP)+,a0
                sub.l   a0,d0           ; Lnge zurckgeben
                subq.l  #1,d0
                rts

_PMFind:        movem.l d1-d2/a0-a2,-(SP)
                tst.l   d0
                beq.b   .exit
                move.l  d0,d2
                lea     (PMCmpTab,PC),a2
                moveq   #0,d1
.continue       moveq   #0,d0
                move.b  (a0),d1
                move.b  (a2,d1.w),d1
.loop           move.b  (a1)+,d0                ; erste Zeichen im Text finden
                beq.b   .exit
                move.b  (a2,d0.w),d0            ; Break when NULL is reached
                cmp.b   d0,d1
                bne.b   .loop

                subq.w  #1,a1
                move.l  d2,d0
                bsr.b   _PMCmpStrings           ; test if whole string is
                tst.l   d0                      ; found
                bne.b   .GotIt                  ; GotIt-> move address to D0
                addq.w  #1,a1
                bra.b   .continue               ; continue search

.exit           sub.l   a0,a0                   ; didn`t find anything
.GotIt          move.l  a0,d0
                movem.l (SP)+,d1-d2/a0-a2
                rts


_PMCmpStrings:  movem.l d1-d2/a0-a2,-(SP)
                move.l  d0,d2
                lea     (PMCmpTab,PC),a2
                subq.l  #1,d2
                moveq   #0,d0
                moveq   #0,d1
.CmpLoop        move.b  (a1)+,d1
                beq.b   .NotSame                ; String2 finished
                move.b  (a2,d1.w),d1
                move.b  (a0)+,d0
                move.b  (a2,d0.w),d0
                bne.b   .NoQM                   ; QM=QuestionMark :-))
                move.b  d1,d0
.NoQM           cmp.b   d0,d1
                dbne    d2,.CmpLoop
                move.w  d2,d0
                bmi.b   .Same
.NotSame        moveq   #0,d0
.Same           ext.l   d0
.exit           movem.l (SP)+,d1-d2/a0-a2
                rts




PMCmpTab        DC.B    $00,$01,$02,$03,$04,$05,$06,$07 ; tab for flushing
                DC.B    $08,$09,$0A,$0B,$0C,$0D,$0E,$0F ; the characters
                DC.B    $10,$11,$12,$13,$14,$15,$16,$17
                DC.B    $18,$19,$1A,$1B,$1C,$1D,$1E,$1F
                DC.B    $20,$21,$22,$23,$24,$25,$26,$27
                DC.B    $28,$29,$2A,$2B,$2C,$2D,$2E,$2F
                DC.B    $30,$31,$32,$33,$34,$35,$36,$37
                DC.B    $38,$39,$3A,$3B,$3C,$3D,$3E,$00 ; <-- '?'
                DC.B    $40,$61,$62,$63,$64,$65,$66,$67
                DC.B    $68,$69,$6A,$6B,$6C,$6D,$6E,$6F
                DC.B    $70,$71,$72,$73,$74,$75,$76,$77
                DC.B    $78,$79,$7A,$6B,$5C,$5D,$5E,$5F
                DC.B    $60,$61,$62,$63,$64,$65,$66,$67
                DC.B    $68,$69,$6A,$6B,$6C,$6D,$6E,$6F
                DC.B    $70,$71,$72,$73,$74,$75,$76,$77
                DC.B    $78,$79,$7A,$7B,$7C,$7D,$7E,$7F
                DC.B    $80,$81,$82,$83,$84,$85,$86,$87
                DC.B    $88,$89,$8A,$8B,$8C,$8D,$8E,$8F
                DC.B    $90,$91,$92,$93,$94,$95,$96,$97
                DC.B    $98,$99,$9A,$9B,$9C,$9D,$9E,$9F
                DC.B    $A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7
                DC.B    $A8,$A9,$AA,$AB,$AC,$AD,$AE,$AF
                DC.B    $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7
                DC.B    $B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
                DC.B    $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7
                DC.B    $C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
                DC.B    $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7
                DC.B    $D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
                DC.B    $E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7
                DC.B    $E8,$E9,$EA,$EB,$EC,$ED,$EE,$EF
                DC.B    $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7
                DC.B    $F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF


