;***************************************************************************
;*                                                                         *
;* SEKTION: Header                                                         *
;*                                                                         *
;***************************************************************************

;----------------------------------- Switches --------------
                MC68000
                NEWSYNTAX
                DEBUG on
                SYM

;----------------------------------- Labels ----------------
                INCDIR  "Include:"
                MACLIB  "LIB:Includes.preass"
                MACLIB  "LIB:Offsets.preass"
                MACFILE "libraries/PToolLibrary.i"
                MACFILE "PMail_Locale.i"

PARAMETERS      equ     SERF_SHARED|SERF_7WIRE|SERF_RAD_BOOGIE|SERF_XDISABLED
BUFFERSIZE      equ     20*1024

;----------------------------------- Macros ----------------

;----------------------------------- References ------------
                XREF    PrgName
                XREF    PublicMemStr
                XREF    _PToolBase
                XREF    _DOSBase
                XREF    _OwnDevUnitBase
                XREF    _AlarmIOReq
                XREF    _Len
                XREF    _ErrorHandle
                XREF    _MakeLogEntry
                XREF    _RememberKey
                XREF    SentBytesAll
                XREF    ReceivedBytesAll

                XDEF    _OffHook
                XDEF    _ReadIOReq
                XDEF    _WriteIOReq
                XDEF    _SerialIOReq
                XDEF    _InitSerial
                XDEF    _CloseSerial
                XDEF    _ReadSerial
                XDEF    _WriteSerial
                XDEF    _WaitForString
                XDEF    _Converse
                XDEF    _SendModemCMD
                XDEF    _CheckCarrier
                XDEF    _InternalBuffer
                XDEF    _MakeConnect
                XDEF    _MakeHandshake
                XDEF    CarrierUp
                XDEF    Timeout
                XDEF    _ResetSerialDevice

;***************************************************************************
;*                                                                         *
;* SEKTION: Unterroutinen                                                  *
;*                                                                         *
;***************************************************************************

;------------------------- Connect herstellen und abbrechen ---------------
_OffHook:
                ; Diese Funktion dient zum Auflegem
                ;
                ; Parameter: keine
                ; Result   : D0=TRUE/FALSE

                PUSHM   d1-d2/a0-a1/a6

                lea     (ReturnStr,PC),a0       ; Eventuell "angebrochene"
                moveq   #1,d0                   ; Kommandos lschen, "ATA"
                bsr     _WriteSerial            ; abrechen!!
                moveq   #20,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODelay,a6)

                bsr     _CheckCarrier
                tst.l   d0
                beq.b   .finished

                lea     (BreakStr,PC),a0
                moveq   #3,d0
                bsr     _WriteSerial
                moveq   #50,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODelay,a6)

                lea     (OffHook,PC),a0
                moveq   #-1,d0
                bsr     _WriteSerial
                moveq   #100,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODelay,a6)

                move.l  (_InternalBuffer,PC),a0
                moveq   #-1,d0
                bsr     _ReadSerial

                bsr     _CheckCarrier
                not.l   d0
                beq.b   .leave

.finished       moveq  #-1,d0
.leave          POPM
                rts


_MakeConnect:
                ; Stellt den Connect her
                ;
                ; Parameter: a0=&DialString a1=&ConnectStringPattern
                ; Result   : D0=TRUE/FALSE  (CONNECT/NO CARRIER)
                ; Kommentar: String wie bei SendModemCMD()

                PUSHM   d1/a0-a3/a6
                link.w  a4,#-256
                move.l  a0,a3

                move.l  a1,a0
                lea     (-256,a4),a1
                move.l  (_PToolBase,PC),a6
                jsr     (_LVOParsePatternFast,a6)

                lea     (RingingStr,PC),a0
                lea     (RingingBuffer,PC),a1
                jsr     (_LVOParsePatternFast,a6)

                bsr     _CheckCarrier
                tst.l   d0
                beq.b   .NoBreakNeeded
                lea     (BreakStr,PC),a0
                moveq   #3,d0
                bsr     _WriteSerial
                move.l  (_DOSBase,PC),a6
                moveq   #100,d1
                jsr     (_LVODelay,a6)
.NoBreakNeeded  move.l  a3,a0
                moveq   #-1,d0
                bsr     _WriteSerial
                lea     (ReturnStr,PC),a0
                moveq   #1,d0
                bsr     _WriteSerial

.RingingLoop    lea     (ReturnStr,PC),a0
                moveq   #2,d0
                bsr     _WaitForString
                tst.l   d0
                beq     .Error
                cmp.l   #-1,d0
                beq     .Error
                cmp.l   #2,d0
                beq.b   .RingingLoop            ; Leerzeilen abfangen!
                clr.b   (a0,d0.l)

                move.l  a0,a2                   ; Alles auer "RINGING" kann
                lea     (RingingBuffer,PC),a1   ; als Connectentscheidung
                move.l  (_PToolBase,PC),a6      ; genommen werden. (BUSY,
                jsr     (_LVOPatternMatchFast,a6) ; NO CARRIER, CONNECT)
                tst.l   d0
                bne.b   .RingingLoop

                move.l  a2,a0                           ; Connectstring mit
                lea     (-256,a4),a1                    ; dem vorgegebenen
                jsr     (_LVOPatternMatchFast,a6)       ; Pattern ver-
                                                        ; gleichen
.exit           unlk    a4
                POPM
                rts
.Error          moveq   #0,d0
                bra.b   .exit


;--------------------------- Serielle Supportroutinen ---------------------
_ResetSerialDevice:
                ; Resettet die Parameter des Serial.Device
                ;
                ; Parameter: keine
                ; Result   : keine
                ; Kommentar: Die verwendete Baudrate wird von InitSerial()
                ;            gesetzt!

                PUSHM   d0-d1/a0-a1/a6

                move.l  (_SerialIOReq,PC),a1
                move.w  #CMD_FLUSH,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)

                move.l  (_SerialIOReq,PC),a1
                move.w  #CMD_CLEAR,(IO_COMMAND,a1)
                jsr     (_LVODoIO,a6)

                move.l  (_SerialIOReq,PC),a1
                move.l  #SER_DEFAULT_CTLCHAR,(IO_CTLCHAR,a1)
                move.l  #BUFFERSIZE,(IO_RBUFLEN,a1)
                clr.l   (IO_EXTFLAGS,a1)                ; Serielle Parameter
                move.l  (SerBaud,PC),(IO_BAUD,a1)       ; einstellen
                move.l  #500000,(IO_BRKTIME,a1)
                clr.l   (IO_TERMARRAY,a1)
                clr.l   (IO_TERMARRAY+4,a1)
                move.w  #$0808,(IO_READLEN,a1)
                move.b  #1,(IO_STOPBITS,a1)
                move.b  #PARAMETERS,(IO_SERFLAGS,a1)
                move.w  #SDCMD_SETPARAMS,(IO_COMMAND,a1)
                jsr     (_LVODoIO,a6)

                POPM
                rts

_CheckCarrier:
                ; Testet ob der Carrier noch besteht
                ;
                ; Parameter: keine
                ; Result   : D0=TRUE/FALSE
                bsr     _GetSerialStatus
                btst.l  #5,d0
                beq     .Carrier
.NoCarrier      moveq   #0,d0
                rts
.Carrier        moveq   #-1,d0
                rts

_SendModemCMD:
                ; Geht in den KommandoModus und schickt ein AT-Kommando ans
                ; Modem
                ;
                ; Parameter: a0=&CMDString
                ; Result   : D0=TRUE/FALSE  (OKAY/ERROR)
                ; Kommentar: Der String mu mit Null abgeschlossen sein.
                ;            Timeout: 5 Sekunden!
                ;            Nach jedem "\r" wird ein Return gesendet und
                ;            die Reaktions des Modems abgewartet. Nur bei
                ;            "OK" wird weitergeparsed.

                PUSHM   d1-d3/a0-a2/a6
                move.l  (Timeout,PC),d3

                move.l  #5,(Timeout)
                move.l  a0,a2

                bsr     _CheckCarrier
                tst.l   d0
                beq.b   .NoBreakNeeded
                lea     (BreakStr,PC),a0
                moveq   #3,d0
                bsr     _WriteSerial
                moveq   #100,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODelay,a6)
.NoBreakNeeded  lea     (ReturnStr,PC),a0       ; Eventuell "angebrochene"
                moveq   #1,d0                   ; Kommandos lschen, "ATA"
                bsr     _WriteSerial            ; abrechen!!
                moveq   #50,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODelay,a6)

.ParseLoop      cmp.b   #'\',(a2)
                beq.b   .ModemCommand
                tst.b   (a2)
                beq.b   .finished
.NoModemCommand bsr     .SendChar
                bra.b   .ParseLoop
.ModemCommand   cmp.b   #'r',(1,a2)
                bne.b   .NoModemCommand
                bsr     .SendReturn
                tst.l   d0
                beq.b   .leave
                bra.b   .ParseLoop

.finished       moveq   #-1,d0
.leave          move.l  d3,(Timeout)
                POPM
                rts

.SendReturn     addq.w  #2,a2
                move.l  (_InternalBuffer,PC),a0
                moveq   #-1,d0
                bsr     _ReadSerial
                lea     (ReturnStr,PC),a0
                moveq   #1,d0
                bsr     _WriteSerial
                lea     (ReturnStr,PC),a0
                moveq   #2,d0
                bsr     _WaitForString
                tst.l   d0
                beq.b   .Error
                cmp.l   #-1,d0
                bne.b   .NoTimeOut
                moveq   #0,d0
                bra.b   .Error
.NoTimeOut      lea     (-4,a0,d0.l),a0
                moveq   #0,d0
                cmp.b   #'O',(a0)+
                bne     .Error
                cmp.b   #'K',(a0)+
                bne     .Error
                moveq   #-1,d0
.Error          rts

.SendChar       moveq   #1,d0
                move.l  a2,a0
                bsr     _WriteSerial
                addq.w  #1,a2
                rts


_MakeHandshake:
                ; Sendet einen Handshake an das andere System und wartet
                ; auf dessen Bestaetigung. Kommt diese nicht innerhalb eines
                ; Zeitraumes versucht die Routine es erneut und bricht
                ; danach mit TIMEOUT ab.
                ;
                ; Parameter: a0=&HandshakeString a1=&ReplyString
                ;            d0=&HandshakeLen    d1=&ReplyLen
                ;            d2=Timeout
                ; Result   : d0=Okay / Break (-1) / Timeout (0)
                ; Kommentar: Fr die Lnge der Strings gilt dasselbe
                ;            wie bei WaitForString()!
                ;            Der alte Timeout-Wert wird nicht verndert!

                movem.l d3-d4,-(SP)
                PUSHM   d1-d2/a0-a1
                move.l  (Timeout,PC),d4

                move.l  d2,(Timeout)
                moveq   #2,d3                   ; D4=Failure-Counter

...             movem.l (SP),saveregs
                bsr     _WriteSerial

                movem.l (SP),saveregs
                move.l  a1,a0
                move.l  d1,d0
                bsr     _WaitForString
                cmp.l   #-1,d0
                beq.b   .exit
                tst.l   d0
                bne.b   .exit
                subq.l  #1,d3
                bne.b   ...

.exit           POPM
                move.l  d4,(Timeout)
                movem.l (SP)+,d3-d4
                rts


_WaitForString:
                ; Wartet darauf, da ein bestimmter Text am Serial.Device
                ; ankommt.
                ;
                ; Parameter: a0=&SearchString d0=Len
                ; Result   : A0=&Buffer D0=ReadLen
                ; Kommentar: Eine Stringlnge von Null schliet das Null-
                ;            Byte mit ein, eine Lnge von -1 liest bis genau
                ;            auf das Nullbyte.
                ;            Ist die Readlnge gleich 0 trat ein Timeout
                ;            auf, ist sie gleich -1 wurde der Read unter-
                ;            brochen.

                PUSHM   d1-d4/a1-a4/a6

                tst.l   d0
                bne.b   .DontReadAll
                bsr     _Len
                addq.l  #1,d0
                bra.b   .LengthValid

.DontReadAll    cmp.l   #-1,d0
                bne.b   .LengthValid
                bsr     _Len
                tst.l   d0
                beq     .exit

.LengthValid    move.l  a0,a4
                move.l  d0,d4

                move.l  (_InternalBuffer,PC),a2
                moveq   #0,d2

.SearchLoop     move.l  a4,a3
                move.l  d4,d3
.SameLoop       bsr     _SerialBufferLen
                tst.l   d0
                bne     .CharsAvailable
                bsr     _WaitOnSerialPort
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit
.CharsAvailable move.l  (_ReadIOReq,PC),a1
                move.w  #CMD_READ,(IO_COMMAND,a1)
                move.l  #1,(IO_LENGTH,a1)
                move.l  a2,(IO_DATA,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)
                addq.l  #1,d2
                add.l   #1,(ReceivedBytesAll)
                cmp.b   (a3)+,(a2)+
                bne     .SearchLoop
                subq.l  #1,d3
                beq     .finished
                bra     .SameLoop

.finished       move.l  d2,d0
.exit           move.l  (_InternalBuffer,PC),a0
                POPM
                rts


_Converse:
                ; Wartet darauf, da ein bestimmter Text am Serial.Device
                ; ankommt und beantwortet ihn mit einem anderen Text.
                ;
                ; Parameter: a0=&SearchString a1=&ConverseString
                ;            d0=SearchStrLen  d1=ConverseStrLen
                ; Result   : d0=TRUE/FALSE
                ; Kommentar: Fr die Lnge der Strings gilt dasselbe
                ;            wie bei WaitForString()!

                move.l  d1,-(SP)
                bsr     _WaitForString
                tst.l   d0
                beq.b   .Error
                cmp.l   #-1,d0
                beq.b   .Error
                move.l  a1,a0
                move.l  (SP)+,d0
                bsr     _WriteSerial
                moveq   #-1,d0
                rts
.Error          move.l  #0,d0
                addq.w  #4,SP
                rts


_GetSerialStatus:
                ; Aktualisiert die Statusfelder in der IORequest-Struktur
                ;
                ; Parameter: keine
                ; Result   : D0.w=Status
                PUSHM   d1/a0-a1/a6
                move.l  (_SerialIOReq,PC),a1
                move.w  #SDCMD_QUERY,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)
                move.l  (_SerialIOReq,PC),a1
                moveq   #0,d0
                move.w  (IO_STATUS,a1),d0
                POPM
                rts

_SerialBufferLen:
                ; Diese Routine ermittelt die Anzahl der Zeichen, die sich
                ; im seriellen ReadPuffer befinden.
                ;
                ; Parameter: keine
                ; Result   : D0=Amount of Characters
                pea     (a1)
                bsr     _GetSerialStatus
                move.l  (_SerialIOReq,PC),a1
                move.l  (IO_ACTUAL,a1),d0
                move.l  (SP)+,a1
                rts


_WaitOnSerialPort:
                ; Wartet darauf, dass am seriellen Port ein Zeichen
                ; eintrifft. Erhaelt das Programm ein CTRL/C oder ein
                ; TIMEOUT, wird das Warten abgebrochen!m
                ;
                ; Parameter: keine
                ; Result   : D0=Amount of Chars
                PUSHM   d1-d2/a0-a3/a6

                move.l  (_AlarmIOReq,PC),a1             ; Timeout-Uhr in
                move.w  #TR_ADDREQUEST,(IO_COMMAND,a1)  ; Gang setzen!
                move.l  (Timeout,PC),(IO_SIZE+TV_SECS,a1)
                move.l  #5,(IO_SIZE+TV_MICRO,a1) ; Wenn ein Request mit
                move.l  ($4).w,a6                ; weniger als 5 Microsecs
                jsr     (_LVOSendIO,a6)          ; abgeschickt wird, strzt
                                                 ; die Maschine ab!!!
.Loop           move.l  #SIGBREAKF_CTRL_C,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVOCheckSignal,a6)
                tst.l   d0
                bne     .Break

                tst.l   (CarrierUp)
                beq.b   .SkipCarrierCheck
                bsr     _CheckCarrier
                tst.l   d0
                beq.b   .CarrierLost
.SkipCarrierCheck
                moveq   #12,d1
                jsr     (_LVODelay,a6)

                move.l  (_AlarmIOReq,PC),a1
                move.l  ($4).w,a6
                jsr     (_LVOCheckIO,a6)
                tst.l   d0
                bne.b   .Timeout

.ContinueLoop   bsr     _SerialBufferLen
                tst.l   d0
                beq.b   .Loop


.AbortTimerIO   move.l  d0,-(SP)
                move.l  (_AlarmIOReq,PC),a1
                move.l  ($4).w,a6
                jsr     (_LVOAbortIO,a6)
                move.l  (_AlarmIOReq,PC),a1
                jsr     (_LVOWaitIO,a6)

                move.l  (SP)+,d0
.exit           POPM
                rts
.Break          moveq   #MSG_LOG_BREAK,d0
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #4,SP
                moveq   #-1,d0
                bra     .AbortTimerIO

.Timeout        moveq   #MSG_LOG_TIMEOUT,d0
                move.l  (Timeout,PC),-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP
                moveq   #0,d0
                bra     .AbortTimerIO

.CarrierLost    moveq   #MSG_LOG_CARRIERLOST,d0
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #4,SP
                moveq   #0,d0
                bra     .AbortTimerIO


;------------------------ Serielle Lese-/Schreibzugriffe ------------------
_ReadSerial:
                ; Liest Zeichen aus dem seriellen Puffer
                ;
                ; Parameter: A0=&Buffer D0=Len
                ; Result   : Anzahl der gelesenen Zeichen (0=Timeout, -1=Break)
                ; Kommentar: Wird als Lnge eine 0 bergeben, wird ein
                ;            String gelesen, bei einer -1 der ganze Puffer.

                PUSHM   d1-d2/a0-a3/a6

                cmp.l   #-1,d0
                bne     .DontReadAll
                bsr     _SerialBufferLen
                tst.l   d0
                bne     .KnownLength
                bsr     _WaitOnSerialPort
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit
                bra     .KnownLength

.DontReadAll    tst.l   d0
                bne     .KnownLength

                move.l  a0,a3                   ; Read until 0-Byte!!
                move.l  (_ReadIOReq,PC),a2
                moveq   #0,d2

.ReadLoop       bsr     _SerialBufferLen
                tst.l   d0
                bne     .CharsAvailable
                bsr     _WaitOnSerialPort
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit
.CharsAvailable move.l  a2,a1
                move.l  #1,(IO_LENGTH,a1)
                move.l  a3,(IO_DATA,a1)
                move.w  #CMD_READ,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)
                addq.l  #1,d2
                add.l   #1,(ReceivedBytesAll)
                tst.b   (a3)+
                bne     .ReadLoop
                move.l  d2,d0
                bra     .exit

.KnownLength    move.l  (_ReadIOReq,PC),a2
                move.l  a2,a1
                move.l  d0,(IO_LENGTH,a1)
                add.l   d0,(ReceivedBytesAll)
                move.l  a0,(IO_DATA,a1)
                move.w  #CMD_READ,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)
.finished       move.l  (IO_ACTUAL,a2),d0

.exit           POPM
                rts


_WriteSerial:
                ; Schreibt Zeichen in den seriellen Puffer
                ;
                ; Parameter: A0=&Buffer D0=Len
                ; Result   : keins
                ; Kommentar: Ist Len gleich -1, wird bis zum ersten 0-Byte
                ;            gesendet, ist Len gleich 0, wird das 0-Byte
                ;            mitgeschickt!

                PUSHM   d0-d1/a0-a1/a6

                tst.l   d0
                bne     .NoStringSend
                bsr     _Len
                addq.l  #1,d0
                bra     .KnownLength

.NoStringSend   cmp.l   #-1,d0
                bne     .KnownLength
                bsr     _Len

.KnownLength    move.l  (_WriteIOReq,PC),a1
                move.l  d0,(IO_LENGTH,a1)
                add.l   d0,(SentBytesAll)
                move.l  a0,(IO_DATA,a1)
                move.w  #CMD_WRITE,(IO_COMMAND,a1)
                move.l  ($4).w,a6
                jsr     (_LVODoIO,a6)

.exit           POPM
                rts



;--------------------- Inititialize and free Serial.Device ----------------
_InitSerial:
                ; Initialisiert alle Devices, Buffer, Resourcen, usw...
                ; die intern bentigt werden.
                ;
                ; Parameter: A0=&SerialDevName D0=Unit D1=BaudRate
                ;            D2=LockSerial (TRUE/FALSE) D3=Bruteforce
                ; Result   : TRUE/FALSE
                ; Kommentar: Wird als Baudrate eine Null bergeben, so wird
                ;            die neuinitialisierung des Devices bersprungen.

                PUSHM   d2-d5/a2/a4/a6

                move.l  a0,a4
                move.l  d0,d4
                move.l  d1,d5

                tst.l   d2                      ; Wenn das Devicelocking
                beq.b   .SkipDevLocking         ; generell abgeschaltet ist,
                tst.l   d1                      ; oder wenn PMail im Getty-
                beq.b   .SkipDevLocking         ; Modus luft, das Device
                                                ; nicht reservieren!!
                lea     (PrgName,PC),a1
                moveq   #0,d1
                move.l  (_OwnDevUnitBase,PC),a6
                tst.l   d3                      ; Ist das Bruteforce-Flag
                bne.b   .BruteForce             ; gesetzt, kommt diese
                pea     (.BruteForceEnd,PC)     ; Routine nicht zurck,
                jmp     (_LVOAttemptDevUnit,a6) ; bevor das Device nicht
.BruteForce     jsr     (_LVOLockDevUnit,a6)    ; frei ist!
.BruteForceEnd  tst.l   d0
                bne     .Error2
                lea     (OwnDevUnit,PC),a1
                movem.l a4/d4,(a1)

.SkipDevLocking lea     (_RememberKey,PC),a0
                move.l  #BUFFERSIZE,d0
                move.l  #MEMF_ANY|MEMF_CLEAR,d1
                move.l  (_PToolBase,PC),a6
                jsr     (_LVOPTAllocRemember,a6)
                move.l  d0,(_InternalBuffer)
                beq     .Error3

                lea     (_SerialMP,PC),a2
                move.l  ($4).W,a6
                jsr     (_LVOCreateMsgPort,a6)          ; QueryMP
                move.l  d0,(a2)+
                beq     .Error1
                move.l  d0,a0
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                move.l  d0,(_SerialIOReq)
                beq     .Error1
                move.l  d0,a1
                move.b  #PARAMETERS,(IO_SERFLAGS,a1)
                move.l  a4,a0
                move.l  d4,d0
                moveq   #0,d1
                jsr     (_LVOOpenDevice,a6)
                tst.l   d0
                bne     .Error1

                jsr     (_LVOCreateMsgPort,a6)          ; ReadMP
                move.l  d0,(a2)+
                beq     .Error1
                move.l  d0,a0
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                move.l  d0,(_ReadIOReq)
                beq     .Error1
                move.l  (_SerialIOReq,PC),a0
                move.l  d0,a1
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCopyMem,a6)
                move.l  (_ReadIOReq,PC),a0
                move.l  (-4,a2),(MN_REPLYPORT,a0)

                jsr     (_LVOCreateMsgPort,a6)          ; WriteMP
                move.l  d0,(a2)+
                beq     .Error1
                move.l  d0,a0
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCreateIORequest,a6)
                move.l  d0,(_WriteIOReq)
                beq     .Error1
                move.l  (_SerialIOReq,PC),a0
                move.l  d0,a1
                moveq   #IOEXTSER_SIZE,d0
                jsr     (_LVOCopyMem,a6)
                move.l  (_WriteIOReq,PC),a0
                move.l  (-4,a2),(MN_REPLYPORT,a0)

                move.l  d5,(SerBaud)
                bne.b   .BaudRateValid
                move.l  (_SerialIOReq,PC),a1
                move.l  (IO_BAUD,a1),(SerBaud)
.BaudRateValid  bsr     _ResetSerialDevice

.finished       moveq   #-1,d0
.exit           POPM
                rts

.Error1         bsr     _CloseSerial          ; Kann das "xyz.device" nicht
                move.l  a4,a0                 ; ffnen!!
                move.l  d4,d0
                pea     (a0)
                move.l  SP,a1
                move.l  #4,d0
                bsr     _ErrorHandle
                addq.w  #4,SP
                moveq   #0,d0
                bra     .exit

.Error2         move.l  d0,-(SP)
                pea     (a4)
                move.l  SP,a1
                moveq   #10,d0
                bsr     _ErrorHandle
                addq.w  #8,SP
                moveq   #0,d0
                bra     .exit

.Error3         bsr     _CloseSerial            ; Kein Speicher!!
                pea     (PublicMemStr,PC)
                move.l  #BUFFERSIZE,-(SP)
                move.l  SP,a1
                moveq   #2,d0
                bsr     _ErrorHandle
                addq.w  #8,SP
                moveq   #0,d0
                bra     .exit


_CloseSerial:
                ; Diese Funktion schliet alle intern benutzten Resources,
                ; die von _InitSerial() geffnet wurden.
                ;
                ; Parameter: keine
                ; Result   : keins

                PUSHM   d2/a6

                lea     (OwnDevUnit,PC),a1
                tst.l   (4,a1)
                beq.b   .SkipDevUnlock
                movem.l (a1),a0/d0
                move.l  (_OwnDevUnitBase,PC),a6
                jsr     (_LVOFreeDevUnit,a6)

.SkipDevUnlock  move.l  ($4).w,a6
                tst.l   (_InternalBuffer)
                beq     .exit

                lea     (_SerialIOReq,PC),a1
                tst.l   (a1)
                beq     .RemPort
                move.l  (a1),a1
                tst.l   (IO_DEVICE,a1)
                beq     .RemIOReq
                jsr     (_LVOCloseDevice,a6)

.RemIOReq       move.l  (_SerialIOReq,PC),a0
                jsr     (_LVODeleteIORequest,a6)
                move.l  (_ReadIOReq,PC),d0
                beq     .RemPort
                move.l  d0,a0
                jsr     (_LVODeleteIORequest,a6)
                move.l  (_WriteIOReq,PC),d0
                beq     .RemPort
                move.l  d0,a0
                jsr     (_LVODeleteIORequest,a6)

.RemPort        lea     (_SerialMP,PC),a2
                moveq   #3-1,d2
...             tst.l   (a2)
                beq     .exit
                move.l  (a2)+,a0
                jsr     (_LVODeleteMsgPort,a6)
                dbra    d2,...

.exit           POPM
                rts


;***************************************************************************
;*                                                                         *
;* SEKTION: Daten                                                          *
;*                                                                         *
;***************************************************************************

BreakStr        dc.b    "+++"
ReturnStr       dc.b    $0D,$0A
RingingStr      CSTR    "#?RINGING#?"
OffHook         CSTR    "ATH",$0D

                CNOP    0,4
OwnDevUnit      ds.l    2

_SerialMP       ds.l    3
_SerialIOReq    ds.l    1
_ReadIOReq      ds.l    1
_WriteIOReq     ds.l    1
_InternalBuffer ds.l    1
SerBaud         ds.l    1
Timeout         dc.l    60                      ; Default 60 Sekunden
CarrierUp       dc.l    0
RingingBuffer   ds.b    16

                EVEN
