;***************************************************************************
;*                                                                         *
;* SEKTION: Header                                                         *
;*                                                                         *
;***************************************************************************

;----------------------------------- Switches --------------
                MC68000
                NEWSYNTAX
                DEBUG on
                SYM

;----------------------------------- Labels ----------------
                MACLIB  "LIB:Includes.preass"
                MACLIB  "LIB:Offsets.preass"
                MACFILE "A:Sources/PetiToolLibrary/PToolLibrary.i"
                MACFILE "A:Sources/PetiMail/PMail_Locale.i"

;----------------------------------- Macros ----------------
                MACFILE "INCLUDE:debug.i"
DEBUG           set     1
DGB_PCREL       set     1

PUSHM           MACRO
saveregs        setrl   \1
                movem.l saveregs,-(SP)
                ENDM
POPM            MACRO
                movem.l (SP)+,saveregs
                ENDM

;----------------------------------- References ------------
                XREF    ZModemWindow
                XREF    PrgName
                XREF    _PToolBase
                XREF    _DOSBase
                XREF    _OwnDevUnitBase
                XREF    _MetaXprBase
                XREF    _Len
                XREF    _Converse
                XREF    _ReadSerial
                XREF    _WriteSerial
                XREF    _WaitForString
                XREF    _SendModemCMD
                XREF    _CheckCarrier
                XREF    _InternalBuffer
                XREF    _MakeConnect
                XREF    _GetString
                XREF    _MakeLogEntry
                XREF    _MakeHandshake
                XREF    _ErrorHandle
                XREF    _ResetSerialDevice
                XREF    CarrierUp
                XREF    Timeout
                XREF    StartDateStamp
                XREF    SentBytesAll
                XREF    SentBytes
                XREF    ReceivedBytesAll
                XREF    ReceivedBytes
                XREF    SentFiles
                XREF    ReceivedFiles
                XREF    _InstallTransferDisplay
                XREF    _RemoveTransferDisplay
                XREF    _RememberKey

                XDEF    _LogIn
                XDEF    _MakeCall
                XDEF    _ReceiveCall

;***************************************************************************
;*                                                                         *
;* SEKTION: Unterroutinen                                                  *
;*                                                                         *
;***************************************************************************

;------------------------------ Protokoll-Routinen ------------------------
_MakeCall:
                ; Fhrt den eigentlichen Filetransfer mit einem anderen
                ; System durch.
                ;
                ; Parameter: A0=&LoginName A1=&CallingSystem A2=&Password
                ;            A3=&DialString A4=&ConnectstringPattern
                ;            A5=&Filenames D0=DefaultTimeout
                ; Result   : D0=Errorcode D1=FileRequests
                ; Kommentar: Errorcode 0 = Connect failed
                ;                      1 = LogIn failed
                ;                      2 = PMail-Handshake failed
                ;                     -1 = No Error

                movem.l d6-d7,-(SP)
                moveq   #0,d7                   ; D7=Return-Code
                move.l  d0,d6                   ; D6=DefaultTimeout
                PUSHM   d1-d5/a0-a6

                DEBUGMARK <"Set timeout to 60 secs and make the connect with '%s'.">,a3

                move.l  #60,(Timeout)
                move.l  a3,a0                   ; Connecten
                move.l  a4,a1
                bsr     _MakeConnect
                tst.l   d0
                bne.b   .Connected

                moveq   #MSG_LOG_XYZ_CONNECT,d0
                bsr     _GetString
                move.l  d0,-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                moveq   #MSG_LOG_XYZERROR,d0
                bsr     _MakeLogEntry
                addq.w  #8,SP
                bra     .exit

.Connected      moveq   #1,d7
                move.l  #-1,(CarrierUp)
                move.l  d6,(Timeout)

                movem.l (SP),saveregs           ; Login durchfhren
                DEBUGMARK <"Log in as '%s' with '%s'.">,a0,a2
                move.l  a2,a1
                bsr     _LogIn
                tst.l   d0
                bne.b   .LoggedIn
                moveq   #MSG_LOG_XYZ_LOGIN,d0
                bsr     _GetString
                move.l  d0,-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                moveq   #MSG_LOG_XYZERROR,d0
                bsr     _MakeLogEntry
                addq.w  #8,SP
                bra     .exit
.LoggedIn       moveq   #2,d7
                lea     (StartDateStamp,PC),a0
                move.l  a0,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODateStamp,a6)
                clr.l   (ReceivedBytesAll)
                clr.l   (SentBytesAll)

                move.l  d6,(Timeout)
                add.l   #10,(Timeout)
                lea     (HandshakeReceiver,PC),a0       ; auf Handshake
                move.l  #HandshakeReceiver_len,d0       ; warten
                lea     (HandshakeCaller,PC),a1
                move.l  #HandshakeCaller_len,d1
                bsr     _Converse
                tst.l   d0
                bne     .SendLoop
                moveq   #MSG_LOG_XYZ_PMHANDSHAKE,d0
                bsr     _GetString
                move.l  d0,-(SP)
                pea     (PrgName,PC)
                move.l  SP,a1
                moveq   #MSG_LOG_XYZERROR,d0
                bsr     _MakeLogEntry
                addq.w  #8,SP
                bra     .exit

.GotHandshake   DEBUGMARK <"Ahh, there's another PetiMail speaking.">
                moveq   #-1,d7
                tst.l   d1
                bne     .SendFileRequests
                DEBUGMARK <"We have no FileRequests waiting, so send NULL.">
                clr.l   -(SP)
                move.l  SP,a0
                moveq   #0,d0
                bsr     _WriteSerial
                addq.w  #4,SP
                bra     .SendLoop
.SendFileRequests:
                movem.l (SP),saveregs                   ; send FileRequests
                move.l  d1,a0
                DEBUGMARK <"Asking for the following files: FileRequests: %s">,a0
                moveq   #0,d0
                bsr     _WriteSerial

.SendLoop       bsr     _ResetSerialDevice
                move.l  #50,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODelay,a6)

                lea     (ZModemHandshake,PC),a0         ; auf Handshake
                move.l  #ZModemHandshake_len,d0         ; warten
                move.l  a0,a1
                move.l  d0,d1
                move.l  d6,d2
                DEBUGMARK <"Syncronize with ZModem-Handshake">
                bsr     _MakeHandshake
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit

                move.l  (a5),a0
                DEBUGMARK <"Send '%s'.">,a0
                moveq   #0,d0                   ; Filenamen schicken
                bsr     _WriteSerial

                move.l  (a5)+,a0
                tst.b   (a0)
                beq     .ReceiveLoop
                sub.l   a1,a1
                moveq   #%00000000000000000000000000000001,d0
                and.l   (ZModemWindow,PC),d0
                bne.b   .SendFile
                bsr     _InstallTransferDisplay
.SendFile       move.l  (_MetaXprBase,PC),a6
                jsr     (_LVOSendFile,a6)
                bsr     _RemoveTransferDisplay
                tst.l   d0
                bne     .NoSendError
                and.l   #$0000FFFF,d7
                bra     .SendLoop

.NoSendError    moveq   #MSG_LOG_SENTFILE,d0     ; Filenamen ins Logfile
                move.l  (-4,a5),-(SP)            ; schreiben
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP

                move.l  (-4,a5),a0
                move.l  (_PToolBase,PC),a6
                jsr     (_LVOGetFileSize,a6)
                cmp.l   #-1,d0
                beq.b   .DeleteIt
                add.l   d0,(SentBytesAll)
                add.l   d0,(SentBytes)
                add.w   #1,(SentFiles)

.DeleteIt       move.l  (-4,a5),d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODeleteFile,a6)
                tst.l   d0                      ; Senden bis kein File mehr
                bne     .SendLoop               ; brig ist!
                moveq   #0,d0
                pea     (.SendLoop,PC)
                bra     _ErrorHandle

.ReceiveLoop    bsr     _ResetSerialDevice
                move.l  d6,(Timeout)
                add.l   #10,(Timeout)
                lea     (ZModemHandshake,PC),a0         ; auf Handshake
                move.l  #ZModemHandshake_len,d0         ; warten
                move.l  a0,a1
                move.l  d0,d1
                DEBUGMARK <"Syncronize with ZModem-Handshake">
                bsr     _Converse
                tst.l   d0
                beq     .exit

                move.l  (_InternalBuffer,PC),a0
                moveq   #0,d0
                bsr     _ReadSerial

.Receive        move.l  (_InternalBuffer,PC),a0
                DEBUGMARK <"Receiving '%s'.">
                tst.b   (a0)
                beq.b   .ReceiveLoopEnd
                sub.l   a1,a1
                moveq   #%00000000000000000000000000000001,d0
                and.l   (ZModemWindow,PC),d0
                bne.b   .ReceiveFile
                bsr     _InstallTransferDisplay
.ReceiveFile    move.l  (_MetaXprBase,PC),a6
                jsr     (_LVOReceiveFile,a6)
                bsr     _RemoveTransferDisplay
                tst.l   d0
                bne.b   .NoReceiveError
                and.l   #$0000FFFF,d7
                bra     .ReceiveLoop

.NoReceiveError moveq   #MSG_LOG_RECEIVEDFILE,d0    ; Filenamen ins
                move.l  (_InternalBuffer,PC),-(SP)  ; Logfile schreiben
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP

                move.l  (_InternalBuffer,PC),a0
                move.l  (_PToolBase,PC),a6
                jsr     (_LVOGetFileSize,a6)
                cmp.l   #-1,d0
                beq     .ReceiveLoop
                add.l   d0,(ReceivedBytesAll)
                add.l   d0,(ReceivedBytes)
                add.w   #1,(ReceivedFiles)
                bra     .ReceiveLoop
.ReceiveLoopEnd

.exit           move.l  d7,d0                   ; Returncode laden
                clr.l   (CarrierUp)
                POPM
                movem.l  (SP)+,d6-d7
                rts


_ReceiveCall:
                ; Nimmt eine Leitung auf, in der von Getty ein Call ange-
                ; nommen wurde und fhrt den Filetransfer durch.
                ;
                ; Parameter: A0=&LoginName A1=&CallingSystem A5=&Filenames
                ;            D0=DefaultTimeout
                ; Result   : D0=Errorcode
                ; Kommentar: Errorcode 2 = PMail-Handshake failed
                ;                     -1 = No Error

                PUSHM   d1/d6-d7/a0-a6
                moveq   #2,d7                   ; D7 = Returncode
                move.l  d0,d6                   ; D6 = DefaultTimeout

                move.l  #-1,(CarrierUp)
                lea     (StartDateStamp,PC),a0
                move.l  a0,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODateStamp,a6)
                clr.l   (ReceivedBytesAll)
                clr.l   (SentBytesAll)

                lea     (HandshakeReceiver,PC),a0
                move.l  #HandshakeReceiver_len,d0
                lea     (HandshakeCaller,PC),a1
                move.l  #HandshakeCaller_len,d1
                move.l  d6,d2
                bsr     _MakeHandshake
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit

                moveq   #-1,d7
                move.l  (_InternalBuffer,PC),a0
                moveq   #0,d0
                bsr     _ReadSerial
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit
                move.l  (_InternalBuffer,PC),a0
                bsr     _HandleFileRequests

.ReceiveLoop    bsr     _ResetSerialDevice

                move.l  d6,(Timeout)
                add.l   #10,(Timeout)
                lea     (ZModemHandshake,PC),a0         ; auf Handshake
                move.l  #ZModemHandshake_len,d0         ; warten
                move.l  a0,a1
                move.l  d0,d1
                bsr     _Converse
                tst.l   d0
                beq     .exit

                move.l  (_InternalBuffer,PC),a0
                moveq   #0,d0
                bsr     _ReadSerial

                move.l  (_InternalBuffer,PC),a0
                tst.b   (a0)
                beq.b   .SendLoop
                sub.l   a1,a1
                moveq   #%00000000000000000000000000000001,d0
                and.l   (ZModemWindow,PC),d0
                bne.b   .ReceiveFile
                bsr     _InstallTransferDisplay
.ReceiveFile    move.l  (_MetaXprBase,PC),a6
                jsr     (_LVOReceiveFile,a6)
                bsr     _RemoveTransferDisplay
                tst.l   d0
                bne.b   .NoReceiveError
                and.l   #$0000FFFF,d7
                bra.b   .ReceiveLoop

.NoReceiveError moveq   #MSG_LOG_RECEIVEDFILE,d0    ; Filenamen ins
                move.l  (_InternalBuffer,PC),-(SP)  ; Logfile schreiben
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP

                move.l  (_InternalBuffer,PC),a0
                move.l  (_PToolBase,PC),a6
                jsr     (_LVOGetFileSize,a6)
                cmp.l   #-1,d0
                beq     .ReceiveLoop
                add.l   d0,(ReceivedBytesAll)
                add.l   d0,(ReceivedBytes)
                add.w   #1,(ReceivedFiles)
                bra     .ReceiveLoop

.SendLoop       tst.b   (a5)                    ; liegt ein File zum Senden
                beq     .SendLoopEnd            ; an?

                bsr     _ResetSerialDevice
                move.l  #50,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODelay,a6)

                lea     (ZModemHandshake,PC),a0         ; auf Handshake
                move.l  #ZModemHandshake_len,d0         ; warten
                move.l  a0,a1
                move.l  d0,d1
                move.l  d6,d2
                bsr     _MakeHandshake
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit

                move.l  (a5),a0
                moveq   #0,d0                   ; Filenamen schicken
                bsr     _WriteSerial

                move.l  (a5)+,a0
                sub.l   a1,a1
                moveq   #%00000000000000000000000000000001,d0
                and.l   (ZModemWindow,PC),d0
                bne.b   .SendFile
                bsr     _InstallTransferDisplay
.SendFile       move.l  (_MetaXprBase,PC),a6
                jsr     (_LVOSendFile,a6)
                bsr     _RemoveTransferDisplay
                tst.l   d0
                bne.b   .NoSendError
                and.l   #$0000FFFF,d7
                bra.b   .SendLoop

.NoSendError    moveq   #MSG_LOG_SENTFILE,d0     ; Filenamen ins Logfile
                move.l  (-4,a5),-(SP)            ; schreiben
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP

                move.l  (-4,a5),a0
                move.l  (_PToolBase,PC),a6
                jsr     (_LVOGetFileSize,a6)
                cmp.l   #-1,d0
                beq.b   .DeleteIt
                add.l   d0,(SentBytesAll)
                add.l   d0,(SentBytes)
                add.w   #1,(SentFiles)

.DeleteIt       move.l  (-4,a5),d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODeleteFile,a6)
                tst.l   d0                      ; Senden bis kein File mehr
                bne     .SendLoop               ; brig ist!
                moveq   #0,d0
                pea     (.SendLoop,PC)
                bra     _ErrorHandle
.SendLoopEnd

.SendFRLoop     lea     (_FileRequests,PC),a5
                bsr     _ResetSerialDevice
                move.l  #50,d1
                move.l  (_DOSBase,PC),a6
                jsr     (_LVODelay,a6)

                lea     (ZModemHandshake,PC),a0         ; auf Handshake
                move.l  #ZModemHandshake_len,d0         ; warten
                move.l  a0,a1
                move.l  d0,d1
                move.l  d6,d2
                bsr     _MakeHandshake
                tst.l   d0
                beq     .exit
                cmp.l   #-1,d0
                beq     .exit

                move.l  (a5),a0
                moveq   #0,d0                   ; Filenamen schicken
                bsr     _WriteSerial

                move.l  (a5)+,a0
                tst.b   (a0)
                beq.b   .SendFRLoopEnd
                sub.l   a1,a1
                moveq   #%00000000000000000000000000000001,d0
                and.l   (ZModemWindow,PC),d0
                bne.b   .SendFileReq
                bsr     _InstallTransferDisplay
.SendFileReq    move.l  (_MetaXprBase,PC),a6
                jsr     (_LVOSendFile,a6)
                bsr     _RemoveTransferDisplay
                tst.l   d0
                bne.b   .NoSendFRError
                and.l   #$0000FFFF,d7
                bra.b   .SendFRLoop

.NoSendFRError  moveq   #MSG_LOG_SENTFILE,d0     ; Filenamen ins Logfile
                move.l  (-4,a5),-(SP)            ; schreiben
                pea     (PrgName,PC)
                move.l  SP,a1
                bsr     _MakeLogEntry
                addq.w  #8,SP

                move.l  (-4,a5),a0
                move.l  (_PToolBase,PC),a6
                jsr     (_LVOGetFileSize,a6)
                cmp.l   #-1,d0
                beq     .SendFRLoop
                add.l   d0,(SentBytesAll)
                add.l   d0,(SentBytes)
                add.w   #1,(SentFiles)
                bra     .SendFRLoop
.SendFRLoopEnd

.exit           move.l  d7,d0
                clr.l   (CarrierUp)
                POPM
                rts


;-------------------------- Connect- und LogIn-Support --------------------
_LogIn:
                ; Logged in ein System ein
                ;
                ; Parameter: a0=&LoginName a1=&Password
                ; Result   : D0=TRUE/FALSE

                PUSHM   d1-d2/a0-a3/a6
                move.l  a1,a3
                move.l  (Timeout,PC),d2

                move.l  #60,(Timeout)
                move.l  a0,a1                           ; Login:
                lea     (LogInPrompt,PC),a0
                moveq   #LogInPrompt_len,d0
                moveq   #-1,d1
                bsr     _Converse
                tst.l   d0
                beq     .Error
                lea     (ReturnStr,PC),a0
                moveq   #1,d0
                bsr     _WriteSerial

                move.l  #20,(Timeout)
                move.l  a3,a1                           ; Password:
                lea     (PasswordPrompt,PC),a0
                moveq   #PasswordPrompt_len,d0
                moveq   #-1,d1
                bsr     _Converse
                tst.l   d0
                beq     .Error
                lea     (ReturnStr,PC),a0
                moveq   #1,d0
                bsr     _WriteSerial
                moveq   #-1,d0

.Error          move.l  d2,(Timeout)
                POPM
                rts


;-------------------------- FileRequest Liste wandeln ---------------------
_HandleFileRequests:
                ; Konvertiert die empfangene FileRequest-Liste in das
                ; Format, wie es die Sende-Schleife bentigt.
                ;
                ; Parameter: a0=&Textbuffer d1=Textlnge (mit 0 Byte)
                ; Result   : keins

                movem.l a1-a2,-(SP)
                PUSHM   d0-d1/a0

                move.l  #MEMF_ANY,d1
                lea     (_RememberKey,PC),a0
                move.l  (_PToolBase,PC),a6
                jsr     (_LVOPTAllocRemember,a6)
                move.l  d0,a1

                movem.l (SP),saveregs
                lea     (_FileRequests,PC),a2

.loop           moveq   #0,d1
                move.l  a1,(a2)
...             tst.b   (a0)
                beq.b   .finished
                cmp.b   #$0A,(a0)
                beq.b   .LineEnd
                move.b  (a0)+,(a1)+
                addq.l  #1,d1
                bra.b   ...

.LineEnd        addq.w  #1,a0
                tst.l   d1
                beq.b   .loop
                clr.b   (a1)+
                addq.w  #4,a2
                bra.b   .loop

.finished       move.l  a1,(a2)+                ; Endmarkierung schreiben
                clr.b   (a1)
                POPM
                movem.l (SP)+,a1-a2
                rts



;***************************************************************************
;*                                                                         *
;* SEKTION: Daten                                                          *
;*                                                                         *
;***************************************************************************


_FileRequests           dc.l    NullByte
                        ds.l    32              ; 20 FileRequests maximal

NullByte                dc.b    0

LogInPrompt             dc.b    "ogin:"
LogInPrompt_len         equ     (*-LogInPrompt)
PasswordPrompt          dc.b    "assword:"
PasswordPrompt_len      equ     (*-PasswordPrompt)

BreakStr                dc.b    "+++"
ReturnStr               dc.b    $0D,$0A

HandshakeCaller         dc.b    "PMail-Protokoll_#2: Caller"
HandshakeCaller_len     equ     (*-HandshakeCaller)

HandshakeReceiver       dc.b    "PMail-Protokoll_#2: Receiver"
HandshakeReceiver_len   equ     (*-HandshakeReceiver)

ZModemHandshake         dc.b    "ZModem-Handshake"
ZModemHandshake_len     equ     (*-ZModemHandshake)

