/*
 *      $Filename: pseudonntp.c $
 *      $Revision: 1.8 $
 *      $Date: 1995/04/17 00:09:03 $
 *
 *      written by Peter Simons <simons@peti.rhein.de>
 *
 *      This program is public domain. Use it however you like. It is
 *      distributed in the hope that it will be useful, but WITHOUT ANY
 *      WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *      FITNESS FOR A PARTICULAR PURPOSE.
 *
 *      pseudonntp has been designed mainly for usage on the Amiga in
 *      combination with CNews or wCNews. I came across the following
 *      problem one day: I wanted to feed another machine with news.
 *      Unfortunately, the software for that machine was only able to use
 *      and to understand nntp. As the Amiga networking world lacks a
 *      working nntpd and I really didn't want to write one, I thought of
 *      this solution.
 *
 *      CNews allows you to use entirely different batchers, munchers and
 *      senders. The batcher, which is the interesting part for our stuff,
 *      is called with a few parameters including the name of the TODO
 *      file, containing all articles to be transferred. pseudonntp will
 *      take this file and turn it into an AmigaDOS script, which uses the
 *      rcp command (remote copy) to copy the article.
 *
 *      While this is somewhat a hack and may require some customization,
 *      it works(tm). If you did not understand a word of what I said, you
 *      will probably not need this program anyway. :-)
 *
 *      $Id: pseudonntp.c,v 1.8 1995/04/17 00:09:03 simons Exp simons $
 */


/************************************* includes ***********/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "pseudonntp_rev.h"

/************************************* defines ************/
#define PRGNAME argv[0]
#define REMOTEPATH "news@bjoern!c:\\win32app\\qvtw\\"
#define TODO_SEND "UUNews:Out.Going/bjoern/TODO.send"

/************************************* prototypes *********/
void ConvertArticlePathToScript(char *);

/************************************* global variables ***/
static char __DOSVer[] = VERSTAG;

/************************************* program ************/
int main(int argc, char **argv)
/*
 * The correct startup template is "batchfile todofile sizelimimt", but
 * since we don't really batch anything, we can ignore them except for the
 * name of the TODO file.
 */
{
	FILE *fh_todo;		/* Filehandle of our CNews TODO file */
	FILE *fh_send;		/* Filehandle of our DOS script */
	char linebuffer[2048];	/* buffer to hold the current article's path */

	if (argc != 4) {
		fprintf(stderr, "%s: Wrong number of arguments!\n", PRGNAME);
		return 20;
	}
	if (!(fh_todo = fopen(argv[2], "r"))) {
		fprintf(stderr, "%s: Couldn't open the TODO file!\n", PRGNAME);
		return 20;
	}
	if (!(fh_send = fopen(TODO_SEND, "r+"))) /* NO TRUNCATE and WRITE */
	  if (!(fh_send = fopen(TODO_SEND, "w"))) { /* CREATE and WRITE */
		  fprintf(stderr, "%s: Couldn't open the script file!\n", PRGNAME);
		  return 20;
	  }
	fseek(fh_send, 0L, SEEK_END);

	while (fscanf(fh_todo, "%s", linebuffer) != EOF) {
		ConvertArticlePathToScript(linebuffer);
		fprintf(fh_send, "%s\n", linebuffer);
	}
	fclose(fh_send);
	fclose(fh_todo);
	chmod(TODO_SEND, S_IWRITE|S_IREAD|S_IEXECUTE|S_ISCRIPT|S_IDELETE);
	remove(argv[2]);

	return 0;
}


/************************************* subroutines ********/
void ConvertArticlePathToScript(char *article)
/*
 * This function expects a path of an article in a buffer and turns it into
 * a command line copying this article to the remote system. The buffer
 * should better be large or you'll run into problems! The remote path and
 * the system are precompiled. Sorry, but this is a hack anyway, right?
 */
{
	char *buffer = strdup(article);
	char *orig = buffer;
	char *to;
	int i = 0;

	sprintf(article, "rcp %s %s", buffer, REMOTEPATH);
	to = article + strlen(article);
	while(*orig && *orig != ':')
	  orig++;		/* skip device name */
	if (*orig)
	  orig++;
	while(*orig) {
		if (*orig == '/') {
			orig++;
			*to++ = '\\';
			i = 0;
		}
		*to++ = *orig++; /* copy current character */
		i++;

		if (i >= 8) {	/* Limit dir names to 8 characters (MS-DOS) */
			*to++ = '\\';
			while(*orig && *orig != '/')
			  orig++;		/* skip rest of dirname */
			i = 0;
			if (*orig)
			  orig++;
		}
	}
	*to = '\0';
}
