/**********************************************************
 *                                                        *
 * RequestString -- request a string from the user        *
 *                                                        *
 * written by Peter Simons <simons@peti.GUN.de>           *
 *                                                        *
 * This program is public domain!!                        *
 *                                                        *
 **********************************************************/


/************************************* Includes ***********/
#include <exec/memory.h>
#include <libraries/reqtools.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/reqtools.h>

/************************************* Defines ************/
#define TEMPLATE "ResultVar/A,Title,MaxLen/N"

/************************************* Prototypes *********/

/************************************* global Variables ***/
struct ReqToolsBase *ReqToolsBase;
struct DosLibrary *DOSBase;

static const char __Version[] = "$VER: RequestString 1.3 " __AMIGADATE__ ;


/************************************* program ************/
int __saveds main(void)
{
        struct Args {
                char *ResultVar;
                char *Title;
                long *MaxLen;
        } Args;
        struct RDArgs *RDArgs;

        int returncode = 0;
        char *buffer;
        long maxlen = 80;       /* Default length of the string to ask
                                 * for. */

        Args.Title = "Please enter a string";   /* This is the default
                                                 * title if no title is
                                                 * specified on the
                                                 * commandline. */

        if (DOSBase = (struct DosLibrary *) OpenLibrary(DOSNAME, 37L)) {
                if (RDArgs = ReadArgs(TEMPLATE, (LONG *) & Args, NULL)) {

                        /*
                         * Set default length if no length or a length
                         * smaller than 1 is provided.
                         */

                        if (Args.MaxLen || *Args.MaxLen < 1)
                                Args.MaxLen = &maxlen;

                        /* Open ReqTool.Library and ask for the string. */

                        if (ReqToolsBase = (struct ReqToolsBase *) OpenLibrary(REQTOOLSNAME, 38L)) {
                                if (buffer = AllocVec(*Args.MaxLen + 1, MEMF_ANY|MEMF_CLEAR)) {
                                        if (rtGetStringA(buffer, *Args.MaxLen, Args.Title, NULL, NULL))
                                                SetVar(Args.ResultVar, buffer, -1L, GVF_LOCAL_ONLY);
                                        else
                                                returncode = 5;
                                        FreeVec(buffer);        /* Free textbuffer. */
                                }
                                CloseLibrary((struct Library *) ReqToolsBase);
                        }
                        else {
                                Printf("FATAL ERROR: Can't open %s!\n", REQTOOLSNAME);
                                returncode = 20;
                        }
                        FreeArgs(RDArgs);
                        CloseLibrary((struct Library *) DOSBase);
                }
                else {
                        PrintFault(IoErr(), NULL);
                        returncode = 10;
                }
        }
        else
                returncode = 20;

        return returncode;
}
